!**************************************************************************
!* Spatially Explicit Individual-Based Dynamic Global Vegetation Model    *
!* SEIB-DGVM ver. 1.42 (2007-Aug-10)                                      *
!*                                                                        *
!*   All rights are reserved by Dr. Hisashi SATO                          *
!*   @ Frontier Research Center for Global Change (FRCGC),                *
!*     Japan Agency for Marine-Earth Science and Technology (JAMSTEC)     *
!*                                                                        *
!*   Latest information and code using policy can be obtained at          *
!*   http://seib-dgvm.com/                                                *
!**************************************************************************

!*************************************************************************************************
! MAIN SIMULATION LOOP
!*************************************************************************************************
SUBROUTINE main_loop ( LAT, LON, &
           tmp_air, tmp_soil, cloud, prec, humid, wind, &
           ALT, Albedo_soil0, W_fi, W_wilt, W_sat, W_mat )

USE data_structure
implicit none
common PN_s, PN_r

!_________________________ Define parameters
!Climatic data (single precision)
real,dimension(365):: tmp_air  !2m air temperature              (Celcius) 
real,dimension(365):: tmp_soil !soil temperature at 10 cm depth (Celcius) 
real,dimension(365):: cloud    !total cloudiness                (fraction)
real,dimension(365):: prec     !precipitation                   (mm day-1)
real,dimension(365):: humid    !specific humidity               (kg kg-1) 
real,dimension(365):: wind     !wind velocity                   (m s-1)   

!Location data (single precision)
real,intent(IN):: LAT          !latitude  (degree)
real,intent(IN):: LON          !longitude (degree)
real,intent(IN):: ALT          !altitude  (m above MSL)
real,intent(IN):: Albedo_soil0 !soil albedo
real,intent(IN):: W_fi         !filed capacity   (m3/m3, 0.0 -> 1.0)
real,intent(IN):: W_wilt       !wilting point    (m3/m3, 0.0 -> 1.0)
real,intent(IN):: W_sat        !saturate point   (m3/m3, 0.0 -> 1.0)
real,intent(IN):: W_mat        !matrix potential 

!PFT specific constants
real,dimension(PFT_no):: PN_s !proportion of nitrogen:stem (g N / g DM)
real,dimension(PFT_no):: PN_r !proportion of nitrogen:root (g N / g DM)

!_________________________ Define variables
!Simulation time counters
integer   counter !simulation day (1-Simulation_year*365)
integer   doy     !day of the year (1-365)
integer   year    !loop counter for year (1-Simulation_year)

!Water on/within soil
real pool_w1   !soil water content of soil layer 1 (mm)
real pool_w2   !soil water content of soil layer 2 (mm)
real pool_w3   !soil water content of soil layer 3 (mm)
real pool_snow !water equivalent snow depth (mm)

!Carbon pool on/within soil
real pool_litter   !litter pool in a stand                               (g DM / stand)
real pool_som_int  !pool of s.o.m. with intermediated decomposition rate (g DM / stand)
real pool_som_slow !pool of s.o.m. with slow decomposition rate          (g DM / stand)

!Properties of vertial forest
logical,dimension(Dived,Dived)   ::patch_vacant   !safe site for establishment of woody PFTs
real   ,dimension(Max_no,Max_hgt)::par_direct_rel !Relative intensity of direct PAR for each foliage disk (0.0-1.0)

!Charactristics of woody PFT (for each individual tree)
integer,dimension(Max_no)::pft            !Plant Functional Type
integer,dimension(Max_no)::age            !tree age (year)
integer,dimension(Max_no)::height         !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no)::bole           !bole length from 1.3m above ground (STEP)
real   ,dimension(Max_no)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real   ,dimension(Max_no)::dbh_sapwood    !sapwood   diameter at 1.3m (m)
real   ,dimension(Max_no)::crown_diameter !crown diameter      (m)
real   ,dimension(Max_no)::crown_area     !crown area          (m2)
real   ,dimension(Max_no)::la             !leaf area           (m2)
real   ,dimension(Max_no)::mass_leaf      !biomass of foliage  (g)
real   ,dimension(Max_no)::mass_trunk     !biomass of trunk    (g)
real   ,dimension(Max_no)::mass_root      !biomass of fine root(g)
real   ,dimension(Max_no)::mass_stock     !biomass for stock   (g)
real   ,dimension(Max_no)::mass_available !biomass available   (g)
real   ,dimension(Max_no)::bole_x         !x_location of bole  (m)
real   ,dimension(Max_no)::bole_y         !y_location of bole  (m)
real   ,dimension(Max_no)::crown_x        !x_location of crown (m)
real   ,dimension(Max_no)::crown_y        !y_location of crown (m)
logical,dimension(Max_no)::tree_exist     !tree exist flag
integer,dimension(Max_no)::height_limit   !maximum tree height that proximate trees permit (step)
real   ,dimension(Max_no)::radius_limit   !maximum canopy radius that proximate trees permit (m)

real   ,dimension(Max_no   )::stat_leaf               !(income-expenditure)/leaf_mass
real   ,dimension(Max_no, 7)::stat_leaf_RunningRecord !1wk running record of stat_leaf
real   ,dimension(Max_no   )::stat_leaf_crowntop      !annual sum of stat_leaf of top crown disk
real   ,dimension(Max_no,10)::stat_leaf_crownbottom   !annual sum of stat_leaf of bottom corwn disks no 1-10

real   ,dimension(Max_no)::mort_regu1 !sum of NPP within the previous year (g dm)
real   ,dimension(Max_no)::mort_regu2 !average leaf area of last year (m2)
real   ,dimension(Max_no)::mort_regu4 !stem diameter increament in last year (m year-1)

!Charactristics of grass PFT (Biomass density approach)
integer            type_grass      !grass type priority in this girid (3:C3 type, 4:C4 type)
integer            C3g_no          !PFT number of C3 grass
integer            C4g_no          !PFT number of C4 grass
real,dimension(2)::gmass_leaf      !c3/c4 grass foliage   biomass (g/m2)
real,dimension(2)::gmass_root      !c3/c4 grass root      biomass (g/m2)
real,dimension(2)::gmass_available !c3/c4 grass available biomass (g/m2)
real,dimension(2)::gmass_stock     !c3/c4 grass stock     biomass (g/m2)
real,dimension(2)::lai_grass       !c3/c4 leaf area index of grass (m2/m2)
real,dimension(2)::lai_opt_grass   !c3/c4 optimum LAI for grass (m2/m2)

!PFT status
logical,dimension(PFT_no)    ::phenology  !Phenological status (T:foliation, F:dormance)
integer,dimension(PFT_no)    ::pheno_clo  !counter number of last leaf onset
integer,dimension(PFT_no)    ::pheno_cls  !counter number of last leaf shedding

real   ,dimension(PFT_no)    ::stat_water !state of water satisfactory (0.0-1.0)
logical,dimension(PFT_no)    ::pft_exist  !flag: PFT exist or not
integer,dimension(21, PFT_no)::size_dist  !size class distribution for each woody PFT

!Other metabolic regulators
real canopy_cond !canopy conductance   (mol H2O m-2 s-1)
real gdd0        !growing-degree-day-sum of 0 Celcius base (reset on Jan 1)
real gdd5        !growing-degree-day-sum of 5 Celcius base (reset on Jan 1)
real,dimension(12)  ::aet        !actual evapotranspiration of last month (mm/month)

!Dynamics related
real    establish_regu !sum of annual precipitation (mm)
real    mort_regu3     !heat stress for each year for mortality calc.
real    moist_factor   !adjusted moisture factor for fire spread
real    fire_factor    !factor for fire probability calculation
integer fire_clo       !counter number of last fire
integer biome          !biome type (see subroutine biome_type for definition)

!Carbon flux recorder
real                    c_emission    !monthly carbon emission (g C / month / stand)
real                    c_uptake      !monthly carbon uptake   (g C / month / stand)
real,dimension(PFT_no)::gpp           !GPP for each pft (g dm / stand / day  )
real,dimension(PFT_no)::npp           !NPP for each pft (g dm / stand / day  )
real,dimension(PFT_no)::gpp_month     !GPP for each pft (g dm / stand / month)
real,dimension(PFT_no)::npp_month     !NPP for each pft (g dm / stand / month)
real,dimension(PFT_no)::gpp_annual    !GPP for each pft (g dm / stand / year )
real,dimension(PFT_no)::npp_annual    !NPP for each pft (g dm / stand / year )
real,dimension(Max_no)::gpp_month_ind !GPP for each individual (g dm / ind / month)

!Metabolic condition recorder
real,dimension(PFT_no,365)::npp_RunningRecord !365 days running  record of NPP (pft, day)
real,dimension(PFT_no    )::npp_10d_ave       !10  days running average of NPP (pft)

real,dimension(2,20)      ::lai_opt_grass_RunningRecord !running record of lai_opt_grass (m2/m2)
real,dimension(PFT_no,365)::stat_water_RunningRecord  !1yr running record of stat_water

real lai_max_wood   !Wood  LAI @ annual maximum (reset@doy1) (m2/m2)
real lai_max_grass  !Grass LAI @ annual maximum (reset@doy1) (m2/m2)
real lai_mean_wood  !Wood  LAI @ annual mean    (reset@doy1) (m2/m2)
real lai_mean_grass !Grass LAI @ annual mean    (reset@doy1) (m2/m2)

!Environmental recorder
real    pheno_gdd !GDD on 5 degree base (reset: 1 Jan for N-hemisphere, 1 July for S-hemisphere)
integer pheno_ncd !Number of chilling day, defined as number of days on which 
                  !the daily mean air temperature is below 5 Cecius degree
                  !(reset: 1 Nov for N-hemisphere, 1 May for S-hemisphere)

real tmp_soil_month_ave   !30days running average of tmp_soil (Celcius)
real tmp_coldest_20yr_ave !20yr running average of tmp_coldest_month (Celcius)
real tmp_hottest_20yr_ave !20yr running average of tmp_hottest_month (Celcius)
real gdd_20yr_ave         !20yr running average of growing-degree-day-sum

real hydRecord_preci  !sum of precipitation     (reset: first day of each month) (mm/month)
real hydRecord_runoff !sum of runoff water      (reset: first day of each month) (mm/month)
real hydRecord_ic     !sum of interception water(reset: first day of each month) (mm/month)
real hydRecord_ev     !sum of evaporated water  (reset: first day of each month) (mm/month)
real hydRecord_tr     !sum of tranpirated water (reset: first day of each month) (mm/month)

real,dimension(365)::tmp_air_RunningRecord  !365days running record  of tmp_air  (Celcius)
real,dimension(365)::tmp_soil_RunningRecord !365days running record  of tmp_soil (Celcius)
real,dimension(365)::prec_RunningRecord     !365days running record  of prec (mm/day)
real,dimension(365)::pool_w1_RunningRecord  !1yr running record of pool_w1

real,dimension( 20)::gdd_20yr_RunningRecord    !20yr running record  of growing-degree-day-sum
real,dimension( 30)::par_RunningRecord         !30days running record of par
real,dimension(365)::par_grass_RunningRecord   !365 days running record of par_grass
real,dimension( 20)::tmp_coldest_RunningRecord !20yr running record  of tmp_coldest_month (Celcius)
real,dimension( 20)::tmp_hottest_RunningRecord !20yr running record  of tmp_hottest_month (Celcius)
real,dimension( 20)::tmp_ave_C3growth_RunningRecord
                 !20yr running record of average temperature of growth phase of C3 grass (Celcius)
real,dimension( 20)::tmp_ave_C4growth_RunningRecord
                 !20yr running record of average temperature of growth phase of C4 grass (Celcius)
real,dimension(  2)::tmp_recorder_C3growth
                 !tmperature recorder for C3 growht phase of C3 grass
real,dimension(  2)::tmp_recorder_C4growth
                 !tmperature recorder for C4 growht phase of C3 grass

!_________________________ Define local variables
!following variables are recalculated on each simulation day, without referring old values
!thus they are not to be wirtten in spinup files

!Physical status of air
real co2atm      !ambient (canopy) CO2 concentration (ppmv)
real ap          !air pressure (hPa)
real vp          !vapour pressure (hPa)
real vpd         !vapour pressure deficit (hPa)
real dnsa        !density of air (kg m-3)
real vp_sat      !saturation vapour pressure (hPa)
real slope_vps   !slope of saturation vapour pressure related to tempertaure (hPa deg C-1)

!Physical status of radiation
real sl_hgt      !solar hight at midday (degree)
real dlen        !day length (hour)
real rad         !global radiation over canopy (W/m2)
real par         !photosynthetically active radiation of mid-day (micro mol photon m-2 s-1)
real par_direct  !direct PAR on canopy top of mid-day (micro mol photon m-2 s-1)
real par_diffuse !difussed PAR on canopy top of mid-day (micro mol photon m-2 s-1)
real albedo_soil !soil surface albedo
real albedo_veg  !vegetation area albedo
real radnet_soil !soil surface net radiation, short wave (W m-2)
real radnet_veg  !canopy net radiation, short wave (W m-2)
real ir_tree     !radiation-interruption-coefficient by tree corwn
real ir_grass    !radiation-interruption-coefficient by grass leaf

real par_grass   !total PAR on top of the grass layer (micro mol photon m-2 s-1)

!Photosynthesis regulator
real,dimension(PFT_no)::eK           !light attenuation coefficient, no dimension
real,dimension(Max_no)::lue          !light use efficiency (mol CO2 mol photon-1)
real,dimension(Max_no)::co2cmp       !CO2 compensation point (ppmv)
real,dimension(Max_no)::psat         !light-saturated photosynthesis rate (micro mol CO2 m-2 s-1)
real,dimension(2)     ::lue_grass    !light use efficiency (mol CO2 mol photon-1)
real,dimension(2)     ::co2cmp_grass !CO2 compensation point (ppmv)
real,dimension(2)     ::psat_grass   !light-saturated photosynthesis rate (micro mol CO2 m-2 s-1)

!Carbon pool
real pool_trunk    !biomass of woody trunk       (g DM / stand)
real pool_leaf     !biomass of woody leaf        (g DM / stand)
real pool_root     !biomass of woody fine root   (g DM / stand)
real pool_grass_up !biomass of grass aboveground (g DM / stand)
real pool_grass_dn !biomass of grass underground (g DM / stand)

!other physiological index
real,dimension(Max_no)::stat_leaf_crownbottom_daily !daily stat_leaf on the bottom of crown layer

!Relative intensity of diffused PAR for each forest layer (0.0-1.0)
real,dimension(Max_hgt)::par_diffuse_rel

!counter related
integer   counter_begin !counter number at initiation of simulation
integer   counter_end   !counter number at termination of simulation

!General usage
integer p, no, i, j, k !loop counter
real    x, y, z        !temporal variables for general usage

!_________________________ INTIALIZE VARIABLES
    Call init_value &
    (LAT, W_sat, tmp_air, tmp_soil, prec, &
    par_grass, &
    gdd0, gdd5, aet, moist_factor, fire_factor, fire_clo, &
    pool_w1, pool_w2, pool_w3, pool_snow, canopy_cond, pool_w1_RunningRecord, &
    pool_litter, pool_som_int, pool_som_slow, &
    stat_water, stat_water_RunningRecord, &
    pft, age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la,  &
    mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
    bole_x, bole_y, crown_x, crown_y, tree_exist, height_limit, radius_limit, &
    type_grass, C3g_no, C4g_no, &
    gmass_leaf, gmass_root, gmass_available, gmass_stock, &
    lai_grass, lai_opt_grass, &
    pft_exist, patch_vacant, size_dist, &
    phenology, &
    pheno_clo, pheno_cls, pheno_gdd, pheno_ncd, &
    stat_leaf, stat_leaf_RunningRecord, &
    c_emission, c_uptake, gpp, npp, gpp_month, npp_month, gpp_annual, npp_annual, gpp_month_ind, &
    establish_regu, &
    tmp_air_RunningRecord, tmp_soil_RunningRecord, prec_RunningRecord, &
    tmp_soil_month_ave, &
    tmp_coldest_RunningRecord, tmp_coldest_20yr_ave, &
    tmp_hottest_RunningRecord, tmp_hottest_20yr_ave, &
    par_RunningRecord, &
    tmp_ave_C3growth_RunningRecord, tmp_recorder_C3growth, &
    tmp_ave_C4growth_RunningRecord, tmp_recorder_C4growth, &
    gdd_20yr_RunningRecord, gdd_20yr_ave, npp_RunningRecord, npp_10d_ave, &
    lai_opt_grass_RunningRecord, &
    biome, &
    hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr, &
    mort_regu1, mort_regu2, mort_regu3, mort_regu4 )

IF (Flag_spinup_read) then
   open(File_no(21), file = 'spinup_in.txt' )
   Call spinup_in &
   (type_grass, C3g_no, C4g_no, fire_clo, biome, pheno_ncd, &
   phenology, pft_exist, &
   tree_exist, &
   pool_w1, pool_w2, pool_w3, pool_snow, &
   pool_litter, pool_som_int, pool_som_slow, &
   canopy_cond, gdd0, gdd5, &
   establish_regu, mort_regu3, moist_factor, fire_factor, &
   c_emission, c_uptake, &
   pheno_gdd, tmp_soil_month_ave, &
   tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr, &
   lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, &
   pheno_clo, pheno_cls, &
   stat_water, npp_10d_ave, &
   pft, age, height, bole, height_limit, &
   dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, &
   la, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   bole_x, bole_y, crown_x, crown_y, radius_limit, &
   stat_leaf, stat_leaf_crowntop, mort_regu1, mort_regu2, &
   stat_leaf_crownbottom, stat_leaf_RunningRecord, &
   npp_RunningRecord, stat_water_RunningRecord, &
   aet, gdd_20yr_RunningRecord, tmp_coldest_RunningRecord, tmp_hottest_RunningRecord, &
   tmp_ave_C3growth_RunningRecord, tmp_ave_C4growth_RunningRecord, &
   par_RunningRecord, tmp_soil_RunningRecord, tmp_air_RunningRecord, &
   prec_RunningRecord, pool_w1_RunningRecord, par_grass_RunningRecord, &
   tmp_recorder_C3growth, tmp_recorder_C4growth, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, lai_grass, lai_opt_grass, &
   lai_opt_grass_RunningRecord)
   close (File_no(21))   !spinup_in.txt
ENDIF

!_________________________ Prepare files and counter number for simulation loop
   !Open output files
    open ( File_no( 2), file = 'output_annual.txt'       )
    open ( File_no( 3), file = 'output_forest.txt'       )
    open ( File_no( 4), file = 'output_air.txt'          )
    open ( File_no( 5), file = 'output_climate.txt'      )
    open ( File_no( 6), file = 'output_radiation.txt'    )
    open ( File_no( 7), file = 'output_netradiation.txt' )
    open ( File_no( 8), file = 'output_water.txt'        )
    open ( File_no( 9), file = 'output_cflux.txt'        )
    open ( File_no(14), file = 'output_wflux.txt'        )
    open ( File_no(10), file = 'output_log.txt'          )
    open ( File_no(11), file = 'output_individual.txt'   )
    open ( File_no(12), file = 'output_grass.txt'        )
    open ( File_no(13), file = 'output_biomass.txt'      )
    open ( File_no(15), file = 'output_lai.txt'          )
    open ( File_no(17), file = 'output_ld_vertical.txt'  )
    open ( File_no(16), file = 'output.txt'              )
  
   !Initialize time recorder
   doy  = 0
   year = 1
   
   !Initialize grand counter
   if (Flag_spinup_read) then
      counter_begin = 1 +Spinup_year*365
      counter_end   = (Simulation_year+Spinup_year) * 365
   else
      counter_begin = 1
      counter_end   = Simulation_year * 365
   endif
   
!_________________________ SIMULATION LOOP
Do counter = counter_begin, counter_end

!_________________________ DAILY UPDATE OF FIELD STATUS
   !Update time counters
   doy = doy + 1
   if (doy == 366) then
      doy   = 1
      year  = year + 1
   endif
   
   !Carbon Flux
   if (Day_of_Month(doy)==1) then
      c_emission        = 0.0
      c_uptake          = 0.0
      gpp_month (:)     = 0.0
      npp_month (:)     = 0.0
      gpp_month_ind (:) = 0.0
   endif
   
   if (doy==1) then
      gpp_annual(:) = 0.0
      npp_annual(:) = 0.0
   endif
   
   do p = 1, PFT_no
      npp_RunningRecord(p,2:365) = npp_RunningRecord(p,1:364)
      npp_RunningRecord(p,1)     = npp(p)
      
      npp_10d_ave(p) = sum ( npp_RunningRecord(p,1:10) ) / 10.0
      
      gpp_month(p)  = gpp_month(p)  + gpp(p)
      npp_month(p)  = npp_month(p)  + npp(p)
      
      gpp_annual(p) = gpp_annual(p) + gpp(p)
      npp_annual(p) = npp_annual(p) + npp(p)
      
      gpp(p)      = 0.0
      npp(p)      = 0.0
      canopy_cond = 0.0
   end do
   
   !Soil hydrogy
   if (Day_of_Month(doy)==1) then
      aet(Month(doy))  = hydRecord_ev + hydRecord_tr
      hydRecord_preci  = 0.0  !sum of precipitation      (mm/month)
      hydRecord_runoff = 0.0  !sum of runoff water       (mm/month)
      hydRecord_ic     = 0.0  !sum of interception water (mm/month)
      hydRecord_ev     = 0.0  !sum of evaporated water   (mm/month)
      hydRecord_tr     = 0.0  !sum of tranpirated water  (mm/month)
   endif
   
   pool_w1_RunningRecord(2:365) = pool_w1_RunningRecord(1:364)
   pool_w1_RunningRecord(1)     = pool_w1
   
   !LAI: annual maximum and average
   if (doy==1) then
      lai_max_wood   = 0.0
      lai_max_grass  = 0.0
      lai_mean_wood  = 0.0
      lai_mean_grass = 0.0
   endif
   
   !LAI of woody PFTs
   x = 0.0
   do no=1, Max_no
      if (tree_exist(no)) x = x + la(no)
   enddo
   x = x / Max_loc / Max_loc
   
   lai_max_wood  = max(x, lai_max_wood)
   lai_mean_wood = lai_mean_wood  + x / 365.0
   
   !LAI of grass PFTs
   if (type_grass == 3) then
      x = 0.9*lai_grass(1) + 0.1*lai_grass(2)
   else
      x = 0.1*lai_grass(1) + 0.9*lai_grass(2)
   endif
   lai_max_grass  = max(x, lai_max_grass)
   lai_mean_grass = lai_mean_grass + x / 365.0
   
   !Fire_factor (for disturbance computation)
   if (doy==1) fire_factor=0.0
   
   x           = pool_w1 / (Depth1 * W_sat)
   fire_factor = fire_factor + exp( -PI * ((x/moist_factor)**2) ) / 365.0
   
   !for tree mortality computation (other mortality controlling)
   mort_regu3 = min(1.0, mort_regu3 + max(0.0, tmp_air(doy)-Heat_base) / 300.0 ) !Heat stress
   
   !Annual precipitation (for establishment computation)
   if (doy==1) establish_regu = 0.0
   establish_regu = establish_regu + prec(doy)
   
!_________________________ DAILY UPDATE OF METABOLIC STATUS
   !GDD (5 cecius base)
   if (doy==1) then
      gdd_20yr_RunningRecord(2:20) = gdd_20yr_RunningRecord(1:19)
      gdd_20yr_RunningRecord(1)    = gdd5
      gdd_20yr_ave = sum( gdd_20yr_RunningRecord(1:min(20, year)) ) / min(20, year)
      gdd0         = 0.0
      gdd5         = 0.0
   end if
   gdd0 = gdd0 + max(0.0, tmp_air(doy)    )
   gdd5 = gdd5 + max(0.0, tmp_air(doy)-5.0)
   
   !Water status
   do p = 1, PFT_no
      x = max(pool_w1/Depth1, pool_w2/Depth2)
      x = (x-W_wilt) / max(0.01, W_fi-W_wilt)
      stat_water(p) = max( 0.0 , min(1.0,x) )
   enddo
   if ( tmp_soil(doy) < 0.0 ) stat_water(:) = 0.0
   
   do p = 1, PFT_no
      stat_water_RunningRecord(p,2:365) = stat_water_RunningRecord(p,1:364)
      stat_water_RunningRecord(p,1)     = stat_water(p)
   enddo
   
   !Phenology 
      !type1: pheno_gdd (for type3 phenology computation)
      if ( (doy==1 .and. LAT>=0.0) .or. (doy==182 .and. LAT<0.0) ) then
           pheno_gdd = 0.0 !reset on 1 Jan for N-hemisphere, 1 July for S-hemisphere
      endif
           pheno_gdd = pheno_gdd + max(0.0, tmp_air(doy)-5.0)
      
      !type2: pheno_ncd (for type3 phenology computation)
      if ( (doy==304 .and. LAT>=0.0) .or. (doy==121 .and. LAT<0.0) ) then
           pheno_ncd = 0 !reset on 1 Nov for N-hemisphere, 1 May for S-hemisphere
      endif
      
      if ( tmp_air(doy) < 5.0 ) then
           pheno_ncd = pheno_ncd + 1
      end if
      
      !Type3: stat_leaf_RunningRecord
      do i = 1, Max_no
         stat_leaf_RunningRecord(i,2:7) = stat_leaf_RunningRecord(i,1:6) 
         stat_leaf_RunningRecord(i,1)   = stat_leaf(i)
      end do
   
   !Biomass density
   pool_trunk=0.0 ; pool_leaf=0.0 ; pool_root=0.0
   Do no=1, Max_no
      if ( .not. tree_exist(no) ) cycle
      pool_trunk = pool_trunk + mass_trunk(no) + mass_stock(no) + mass_available(no)
      pool_leaf  = pool_leaf  + mass_leaf(no)
      pool_root  = pool_root  + mass_root(no)
   End Do
   
   pool_grass_up=0.0 ; pool_grass_dn=0.0
   if (type_grass==3) then
      pool_grass_up = 0.9 * ( gmass_leaf(1)+gmass_available(1) ) &
                    + 0.1 * ( gmass_leaf(2)+gmass_available(2) )  
      pool_grass_dn = 0.9 * ( gmass_root(1)+gmass_stock(1) )     &
                    + 0.1 * ( gmass_root(2)+gmass_stock(2) )      
   else
      pool_grass_up = 0.1 * ( gmass_leaf(1)+gmass_available(1) ) &
                    + 0.9 * ( gmass_leaf(2)+gmass_available(2) )  
      pool_grass_dn = 0.1 * ( gmass_root(1)+gmass_stock(1) )     &
                    + 0.9 * ( gmass_root(2)+gmass_stock(2) )      
   endif
   pool_grass_up = pool_grass_up * Max_loc * Max_loc
   pool_grass_dn = pool_grass_dn * Max_loc * Max_loc
   
   if (Logging==1) write (File_no(10),*) '    climate_stat'      !log
   Call climate_stat (                                    &
   tmp_air(doy), tmp_soil(doy), prec(doy), par_grass,     &
   year, doy, counter, phenology, pheno_clo, pheno_cls,   &
   C3g_no, C4g_no,                                        &
   tmp_air_RunningRecord, prec_RunningRecord,             &
   tmp_soil_RunningRecord,  tmp_soil_month_ave,           &
   tmp_coldest_RunningRecord, tmp_coldest_20yr_ave,       &
   tmp_hottest_RunningRecord, tmp_hottest_20yr_ave,       &
   tmp_ave_C3growth_RunningRecord, tmp_recorder_C3growth, &
   tmp_ave_C4growth_RunningRecord, tmp_recorder_C4growth, &
   par_grass_RunningRecord                                )
   
!_________________________ DAILY PSYSIOLOGICAL PROCESSES
   if (Logging==1) write (File_no(10),*) '    co2'        !log
   !Call co2_density (co2atm, doy, LAT)
   co2atm = 355.0
   
   if (Logging==1) write (File_no(10),*) '    air', tmp_air(doy), humid(doy), wind(doy)   !log
   Call air &
   (tmp_air(doy), humid(doy), wind(doy), ALT, &
   ap, vp, vpd, dnsa, vp_sat, slope_vps)
   
   if (Logging==1) write (File_no(10),*) '    radiation'  !log
   Call radiation &
   (LAT, doy, cloud(doy), &
   sl_hgt, dlen, rad, par, par_diffuse, par_direct, par_RunningRecord)
   
   if (Logging==1) write (File_no(10),*) '    light_condition: diffuse'  !log
   Call diffused_radiation &
   (par_diffuse_rel, pft, height, bole, la, tree_exist, phenology)
   
   if (mod(doy,30)==0) then
   if (Logging==1) write (File_no(10),*) '    light_condition: direct'  !log
   Call direct_radiation &
   (par_direct_rel, par_diffuse_rel, sl_hgt, doy, LAT, phenology, &
    tree_exist, pft, height, bole, crown_diameter, crown_area, la, crown_x, crown_y)
   end if
   
   if (Logging==1) write (File_no(10),*) '    light_condition: interruption index'  !log
   Call ir_index &
   (ir_tree, ir_grass, par_grass, type_grass, C3g_no, C4g_no, &
   par, par_direct, par_diffuse, phenology, eK, pft, tree_exist, la, lai_grass)
   
!_________________________ DAILY BIOLOGICAL PROCESSES
   if (Logging==1) write (File_no(10),*) '    photosynthesis_condition'  !log
   Call photosynthesis_condition &
   (par, par_grass, par_diffuse, par_direct, &
   vpd, co2atm, sl_hgt, stat_water, tmp_air(doy), &
   tmp_air_RunningRecord, prec_RunningRecord, &
   tmp_ave_C3growth_RunningRecord, tmp_ave_C4growth_RunningRecord, &
   C3g_no, C4g_no, phenology, pft, tree_exist, &
   la, height, bole, par_direct_rel, par_diffuse_rel, &
   eK, lue, co2cmp, psat, lue_grass, co2cmp_grass, psat_grass)
   
   if (Logging==1) write (File_no(10),*) '    photosynthesis'     !log
   Call photosynthesis &
   (par_direct, par_diffuse, par_grass, par_diffuse_rel, par_direct_rel, &
   doy, c_uptake, dlen, &
   pft_exist, phenology, eK, lue, psat, co2cmp, lue_grass, psat_grass, co2cmp_grass, &
   gpp, npp, gpp_month_ind, &
   tree_exist, pft, height, bole, crown_area, la, mass_leaf, mass_available, &
   mort_regu1, stat_leaf_crownbottom, stat_leaf_crowntop, stat_leaf, &
   stat_leaf_crownbottom_daily, &
   type_grass, lai_grass, gmass_available, &
   co2atm, vpd, canopy_cond, C3g_no, C4g_no)
   
   if (Logging==1) write (File_no(10),*) '    lai_optimum'        !log
   Call lai_optimum &
   (lai_opt_grass, lai_opt_grass_RunningRecord, C3g_no, C4g_no, &
   pft_exist, eK, lue_grass, psat_grass, &
   dlen, tmp_air(doy), tmp_soil(doy), par, par_grass,&
   counter, pheno_clo, pheno_cls)
   
   if (Logging==1) write (File_no(10),*) '    maintenance_resp'   !log
   Call maintenance_resp &
   (doy, pft, height, bole, tree_exist, dbh_heartwood, dbh_sapwood, la,  &
   mass_leaf, mass_trunk, mass_root, mass_available, mass_stock, &
   stat_leaf_crownbottom, stat_leaf_crowntop, stat_leaf, stat_leaf_crownbottom_daily, &
   type_grass, lai_grass, C3g_no, C4g_no, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, &
   pool_litter, c_emission, phenology, npp, mort_regu1, tmp_air(doy), tmp_soil(doy) )
   
   if (Logging==1) write (File_no(10),*) '    turnover'           !log
   Call turnover &
   (tree_exist, pft, mass_available, mass_leaf, mass_root, dbh_heartwood, dbh_sapwood, la, &
   type_grass, lai_grass, gmass_leaf, gmass_root, gmass_available, &
   pool_litter, counter, pheno_clo, pheno_cls, C3g_no, C4g_no )
   
   if (Logging==1) write (File_no(10),*) '    leaf_season'        !log
   Call leaf_season &
   (LAT, doy, counter, c_uptake, c_emission, pool_litter, &
   tmp_air_RunningRecord, tmp_coldest_20yr_ave, tmp_soil(doy), C3g_no, C4g_no, &
   pft_exist, npp, npp_10d_ave, stat_water_RunningRecord, &
   phenology, pheno_clo, pheno_cls, pheno_gdd, pheno_ncd, &
   pft, tree_exist, la, mass_leaf, mass_stock, mass_available, &
   type_grass, lai_grass, gmass_leaf, gmass_stock, gmass_available, &
   lai_opt_grass, lai_opt_grass_RunningRecord )
   
   if (Logging==1) write (File_no(10),*) '    growth_wood_daily'        !log
   Call growth_wood &
   (doy, phenology, pheno_clo, stat_leaf_RunningRecord, counter, &
   pft, height, bole, dbh_heartwood, dbh_sapwood, crown_area, crown_diameter, tree_exist, &
   la, mort_regu2, mass_leaf, mass_root, mass_available, mass_stock, stat_leaf_crownbottom_daily, &
   pool_litter, c_emission, npp, mort_regu1 )
   
   if (Logging==1) write (File_no(10),*) '    growth_grass'        !log
   Call growth_grass &
   (doy, counter, pool_litter, c_emission, npp, phenology, pheno_clo, &
   type_grass, lai_grass, gmass_leaf, gmass_root, gmass_available, &
   gmass_stock, lai_opt_grass_RunningRecord, C3g_no, C4g_no)

!_________________________ MONTHLY BIOLOGICAL PROCESSES
IF ( Day_of_Month(doy) == Day_in_month(Month(doy)) ) then
   
   !spacial limitation on tree growth
   if (Logging==1) write (File_no(10),*) '    spacial_limitation'  !log
    Call spacial_limitation &
   (height_limit, radius_limit, &
   pft, height, bole, crown_diameter, crown_x, crown_y, tree_exist, phenology)
   
   !tree growth
   if (Logging==1) write (File_no(10),*) '    growth_stem'         !log
    Call growth_stem &
    (counter, pheno_clo, phenology, &
    pool_litter, c_emission, &
    height_limit, radius_limit, pft, tree_exist, &
    height, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, &
    mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
    gpp_month_ind, npp, mort_regu1, mort_regu4 )
   
   !Decomposition (from LPJ)
   if (Logging==1) write (File_no(10),*) '    decompostion'        !log
   Call decomposition &
   (pool_litter, pool_som_int, pool_som_slow, c_emission, &
   sum(aet), tmp_soil_month_ave, pool_w1, pool_w2, W_sat)
  
END IF

!_________________________ ANNUALY BIOLOGICAL PROCESSES
IF ( doy==365 ) then
   
   if (Logging==1) write (File_no(10),*) '    crown_adjust'      !log
   Call crown_adjust &
   (height, pft, tree_exist, bole, la, mass_leaf, pool_litter, &
    stat_leaf_crownbottom, stat_leaf_crowntop)
   
   if (Logging==1) write (File_no(10),*) '    crown_shake'      !log
   Call crown_shake &
   (crown_x, crown_y, bole_x, bole_y, bole, height, pft, crown_diameter, tree_exist)
   
   if (Logging==1) write (File_no(10),*) '    mortality'         !log
   Call mortality &
   (tree_exist, pft, age, height, bole, dbh_heartwood, dbh_sapwood, &
   crown_diameter, crown_area, la, &
   mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   bole_x, bole_y, height_limit, radius_limit, &
   stat_leaf, stat_leaf_RunningRecord, stat_leaf_crowntop, stat_leaf_crownbottom, &
   mort_regu1, mort_regu2, mort_regu3, mort_regu4, &
   pool_litter, tmp_coldest_20yr_ave, tmp_hottest_20yr_ave)
   
   mort_regu1(:) = 0.0 !NPP annual (g / individual)
   mort_regu2(:) = 0.0 !average leaf area of last year (m2/day) {update on: growth_wood}
   mort_regu3    = 0.0 !heat stress for each year
   mort_regu4(:) = 0.0 !stem diameter increament in last year (m year-1)
   
   !establishment of woody PFTs
   x = sum(prec_RunningRecord(1:365))            !annual precipitation (mm/year)
   y = sum(tmp_air_RunningRecord(1:365)) / 365.0 !annual mean temperature (Celcius)
   if ( x > max(100.0, 20.0*y) ) then            !from Koppen's criteria (1936)
      
      if (Logging==1) write (File_no(10),*) '    ground_vacant'     !log
       Call ground_vacant &
       (patch_vacant, bole, crown_diameter, bole_x, bole_y, crown_x, crown_y, tree_exist)
       
      if (Logging==1) write (File_no(10),*) '    establish'         !log
       Call establish &
       (counter, fire_clo, biome, tmp_coldest_20yr_ave, gdd_20yr_ave, c3g_no, c4g_no, &
       par, par_grass, par_grass_RunningRecord, stat_water_RunningRecord, &
       pool_litter, c_uptake, pft_exist, patch_vacant, &
       pft, age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
       mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
       bole_x, bole_y, crown_x, crown_y, &
       tree_exist, height_limit, radius_limit, mort_regu1, mort_regu2, &
       stat_leaf, stat_leaf_RunningRecord, stat_leaf_crowntop, stat_leaf_crownbottom)
       
   end if
   
   if (Logging==1) write (File_no(10),*) '    grass priority change' !log
   Call grass_priority &
   (type_grass, C3g_no, C4g_no, &
   phenology, pheno_clo, pheno_cls, pool_litter, npp_annual, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock)
   
   if (Logging==1) write (File_no(10),*) '    fire_regime'           !log
   Call fire_regime &
   (fire_factor, fire_clo, c_emission, phenology, &
   pool_litter, pool_trunk, pool_leaf, pool_root, pool_grass_up, &
   pft, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, &
   age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
   lai_grass, type_grass, counter )
   
   if (Logging==1) write (File_no(10),*) '    moist_factor_calc' !log
   Call moist_factor_calc &
   (moist_factor, pft, &
   mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, type_grass, C3g_no, C4g_no)
   
   if (Logging==1) write (File_no(10),*) '    pft_present'       !log
   Call pft_present (pft_exist, tree_exist, pft, C3g_no, C4g_no)
   
   Call size_class_update &
    (size_dist, tree_exist, pft, dbh_heartwood, dbh_sapwood)
   
   if (Logging==1) write (File_no(10),*) '    biome_determine'   !log
   Call biome_determine &
   (biome, C3g_no, C4g_no, W_fi, W_wilt, gdd0, gdd5, lai_max_wood, lai_max_grass, &
   npp_annual, pool_w1_RunningRecord)
   
END IF

!_________________________ DAILY UPDATE NET-RADIATION & SOIL-WATER STATUS
   if (Logging==1) write (File_no(10),*) '    net_radiation'      !log
   Call net_radiation &
   (eK, lai_grass, type_grass, C3g_no, C4g_no, biome, &
   tmp_air(doy), cloud(doy), Albedo_soil0, &
   pool_snow, vp, rad, par, par_grass, ir_tree, ir_grass, &
   albedo_soil, albedo_veg, radnet_soil, radnet_veg)
   
   if (Logging==1) write (File_no(10),*) '    waterbudget'        !log
   Call waterbudget &
   (doy, tmp_air(doy), tmp_soil(doy), prec(doy), wind(doy), tmp_soil_RunningRecord, &
   W_fi, W_wilt, W_sat, W_mat, &
   dlen, radnet_veg, radnet_soil, vpd, dnsa, slope_vps, canopy_cond, &
   biome, pft, la, tree_exist, mass_root, pool_litter, &
   type_grass, gmass_root, lai_grass, C3g_no, C4g_no, &
   pool_w1, pool_w2, pool_w3, pool_snow, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr)
   
!_________________________ MAKE OUTPUT FILES
   if (Flag_output_write) then
   Call output_point &
   (year, Day_of_Month(doy), doy, counter, &
   tmp_air(doy), tmp_soil(doy), cloud(doy), prec(doy), humid(doy), wind(doy), &
   W_fi, W_wilt, W_sat, W_mat, &
   co2atm, ap, vp, vpd, dnsa, vp_sat, slope_vps, &
   sl_hgt, dlen, rad, par, par_grass, &
   albedo_soil, albedo_veg, radnet_soil, radnet_veg, &
   mort_regu3, gdd5, moist_factor, fire_factor, &
   pool_w1, pool_w2, pool_snow, canopy_cond, &
   pool_litter, pool_som_int, pool_som_slow, &
   stat_water, eK, &
   pft, age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
   mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   bole_x, bole_y, crown_x, crown_y, tree_exist, height_limit, radius_limit,&
   type_grass, gmass_leaf, gmass_root, gmass_available, gmass_stock, &
   lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, lai_grass, lai_opt_grass, &
   pft_exist, biome, C3g_no, C4g_no, &
   phenology, pheno_clo, pheno_cls, pheno_gdd, pheno_ncd, &
   c_emission, c_uptake, gpp, npp, gpp_annual, npp_annual, mort_regu1, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr)
   
   Call output_for_viewer &
   (LAT, LON, ALT, Albedo_soil0, W_fi, W_wilt, W_sat, W_mat, &
   biome, c_emission, c_uptake, pool_litter, pool_som_int, pool_som_slow, &
   counter, year, doy, &
   tmp_air(doy), tmp_soil(doy), cloud(doy), prec(doy), humid(doy), wind(doy), &
   co2atm, ap, vp, vp_sat, dnsa, slope_vps, &
   sl_hgt, dlen, par, par_direct, par_diffuse, par_grass, &
   rad, radnet_soil, radnet_veg, albedo_soil, albedo_veg, ir_tree, ir_grass, &
   canopy_cond, pool_w1, pool_w2, pool_w3, pool_snow, &
   phenology, lue, co2cmp, psat, lue_grass, co2cmp_grass, psat_grass, gpp_month, npp_month, &
   tree_exist, pft, la, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   dbh_heartwood, dbh_sapwood, &
   type_grass, C3g_no, C4g_no, &
   lai_grass, gmass_leaf, gmass_root, gmass_available, gmass_stock, lai_opt_grass, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr )
   
   endif

END DO !The end of daily loop

!_________________________ After Main-simulation-loop procedures

   close (File_no( 2))   !output_annual.txt
   close (File_no( 3))   !output_forest.txt
   close (File_no( 4))   !output_air.txt
   close (File_no( 5))   !output_climate.txt
   close (File_no( 6))   !output_radiation.txt
   close (File_no( 7))   !output_netradiation.txt
   close (File_no( 8))   !output_water.txt
   close (File_no( 9))   !output_flux.txt
   close (File_no(14))   !output_wflux.txt
   close (File_no(10))   !output_log.txt
   close (File_no(11))   !output_individual.txt
   close (File_no(12))   !output_grass.txt
   close (File_no(13))   !output_biomass.txt
   close (File_no(15))   !output_lai.txt
   close (File_no(17))   !output_ld_vertical.txt
   close (File_no(16))   !output.txt

IF ( Flag_spinup_write ) then
   open(File_no(22), file = 'spinup_out.txt')
   Call spinup_out &
   (type_grass, C3g_no, C4g_no, fire_clo, biome, pheno_ncd, &
   phenology, pft_exist, &
   tree_exist, &
   pool_w1, pool_w2, pool_w3, pool_snow, &
   pool_litter, pool_som_int, pool_som_slow, &
   canopy_cond, gdd0, gdd5, &
   establish_regu, mort_regu3, moist_factor, fire_factor, &
   c_emission, c_uptake, &
   pheno_gdd, tmp_soil_month_ave, &
   tmp_coldest_20yr_ave, tmp_hottest_20yr_ave, gdd_20yr_ave, &
   hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr, &
   lai_max_wood, lai_max_grass, lai_mean_wood, lai_mean_grass, &
   pheno_clo, pheno_cls, &
   stat_water, npp_10d_ave, &
   pft, age, height, bole, height_limit, &
   dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, &
   la, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
   bole_x, bole_y, crown_x, crown_y, radius_limit, &
   stat_leaf, stat_leaf_crowntop, mort_regu1, mort_regu2, &
   stat_leaf_crownbottom, stat_leaf_RunningRecord, &
   npp_RunningRecord, stat_water_RunningRecord, &
   aet, gdd_20yr_RunningRecord, tmp_coldest_RunningRecord, tmp_hottest_RunningRecord, &
   tmp_ave_C3growth_RunningRecord, tmp_ave_C4growth_RunningRecord, &
   par_RunningRecord, tmp_soil_RunningRecord, tmp_air_RunningRecord, &
   prec_RunningRecord, pool_w1_RunningRecord, par_grass_RunningRecord, &
   tmp_recorder_C3growth, tmp_recorder_C4growth, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, lai_grass, lai_opt_grass, &
   lai_opt_grass_RunningRecord)
   close (File_no(22))   !spinup_out.txt
END IF

END SUBROUTINE main_loop
