!**************************************************************************************************
! Calculation of photosynthesis parameters (for each PFT)
!**************************************************************************************************
SUBROUTINE photosynthesis_condition &
   (par, par_grass, par_diffuse, par_direct, &
   vpd, CO2, sl_hgt, stat_water, tmp_air, &
   tmp_air_RunningRecord, prec_RunningRecord, &
   tmp_ave_C3growth_RunningRecord, tmp_ave_C4growth_RunningRecord, &
   C3g_no, C4g_no, phenology, pft, tree_exist, &
   la, height, bole, par_direct_rel, par_diffuse_rel, &
   eK, lue, co2cmp, psat, lue_grass, co2cmp_grass, psat_grass)

USE data_structure
implicit none
common PN_s, PN_r

!_____________ Define Auguments
!PFT specific constants
real,dimension(PFT_no)::PN_s !proportion of nitrogen:stem (gN / gBiomass)
real,dimension(PFT_no)::PN_r !proportion of nitrogen:root (gN / gBiomass)

!Radiation rekated 
real,intent(IN)::par         !PAR of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_grass   !PAR under tree canopy (micro mol photon m-2 s-1)
real,intent(IN)::par_diffuse !PAR of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_direct  !PAR of mid-day (micro mol photon m-2 s-1)

!Define augments
real                  ,intent(IN)::vpd          !vapour pressure  deficit (hPa)
real                  ,intent(IN)::CO2          !ambient (canopy) CO2 concentration (ppmv)
real                  ,intent(IN)::sl_hgt       !solar hight at midday (degree)
real,dimension(PFT_no),intent(IN)::stat_water   !state of water satisfactory (0.0-1.0)
real                  ,intent(IN)::tmp_air
real,dimension(365)   ,intent(IN)::tmp_air_RunningRecord !365days running record  of tmp_air (C)
real,dimension(365)   ,intent(IN)::prec_RunningRecord    !365days running record  of prec (mm/day)
real,dimension(20)    ,intent(IN)::tmp_ave_C3growth_RunningRecord
                 !20yr running record of average temperature of growth phase of C3 grass (Celcius)
real,dimension(20)    ,intent(IN)::tmp_ave_C4growth_RunningRecord
                 !20yr running record of average temperature of growth phase of C4 grass (Celcius)

integer,intent(IN)::C3g_no !PFT number of C3 grass
integer,intent(IN)::C4g_no !PFT number of C4 grass

logical,dimension(PFT_no)   ,intent(IN)::phenology       !Phenological status
integer,dimension(Max_no) ,intent(IN)::pft             !Plant Functional Type
logical,dimension(Max_no) ,intent(IN)::tree_exist      !tree exist flag
real   ,dimension(Max_no) ,intent(IN)::la              !leaf area           (m2)
integer,dimension(Max_no) ,intent(IN)::height          !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no) ,intent(IN)::bole            !bole length from 1.3m above ground (STEP)
real   ,dimension(Max_hgt),intent(IN)::par_diffuse_rel 
real   ,dimension(Max_no, Max_hgt),intent(IN)::par_direct_rel

real,dimension(PFT_no),intent(OUT)::eK           !light attenuation coefficient, no dimension
real,dimension(Max_no),intent(OUT)::psat         !light-saturated phtosynthesis rate
real,dimension(Max_no),intent(OUT)::lue          !light use efficiency (mol CO2 mol photon-1)
real,dimension(Max_no),intent(OUT)::co2cmp       !CO2 compensation point (ppmv)
real,dimension(2)     ,intent(OUT)::lue_grass    !light use efficiency (mol CO2 mol photon-1)
real,dimension(2)     ,intent(OUT)::co2cmp_grass !CO2 compensation point (ppmv)
real,dimension(2)     ,intent(OUT)::psat_grass   !light-saturated photosynthesis rate

!Define local variables
real   gs_top   !midday stomatal conductance on top of the leaf layer (mol H2O m-2 s-1)
real   co2cell  !monthly intercellular CO2 concentration (ppmv)
real   p_ave    !average photosynthetic rate
real   topt     !adjusted optimum  temperature (deg C)
real   ce_tmp   !temperature effect on photosynthesis
real   ce_co2   !stomatal effect on photosynthesis via intercellular CO2 concentration
real,dimension(PFT_no)::ce_water !non-stomatal effect on photosynthesis

!_____________ Define local variables
real                  :: a1, a2, x
integer               :: loop, p, i, no, gn
real,dimension(Max_no):: par_ave       !average PAR intensity receiving for each tree
real,dimension(2)     :: par_ave_grass !average PAR intensity receiving for grass PFTs

!_____________ The Condition that plants can survive by Koppen (1936) (optional)
   a1 = sum(prec_RunningRecord(1:365))            !annual precipitation (mm/year)
   a2 = sum(tmp_air_RunningRecord(1:365)) / 365.0 !annual mean temperature (Celcius)
   if ( a1 < max(100.0, 10.0*a2) ) then
      lue          (:) = 0.0
      co2cmp       (:) = 0.0
      psat         (:) = 0.0
      lue_grass    (:) = 0.0
      co2cmp_grass (:) = 0.0
      psat_grass   (:) = 0.0
      return
   endif
   
!_____________ Preparation of PFT specific varables
   Do p=1, PFT_no
      !eK: adjusted light-attenuation-coefficient
      eK(p) = EK0(p) / min(1.0, max(0.3, sin(sl_hgt * DtoR)))
      
      !ce_water: limitation on photosynthesis via soil water
      ce_water(p) = min( 1.0 , max(0.001, stat_water(p)) )
      ce_water(p) = ce_water(p) ** 0.5   
      
   End do
   
!_____________ Preparation relative light intensity
   !par_ave: relative PAR intensity for each tree
   par_ave(:) = 0.0
   
   !x: fraction of direct PAR
   x = par_direct / max(0.0001, par_direct+par_diffuse)
   x = max(x, 0.0)
   x = min(x, 1.0)
   
   DO no=1, Max_no
      if ( .not. tree_exist(no)     ) cycle
      if ( .not. phenology(pft(no)) ) cycle
      if ( la(no) <=0.0             ) cycle
      
      do i = height(no), bole(no)+1, -1
         par_ave(no) = par_ave(no)  + x * par_direct_rel(no,i) + (1.0-x) * par_diffuse_rel(i)
      end do
      par_ave(no) = (0.5*par) * par_ave(no) / (height(no)-bole(no))
      
   END DO
   
   !for each grass PFT (temporal way of calculation)
   par_ave_grass(1) = 0.5 * par_grass !C3 grass
   par_ave_grass(2) = 0.5 * par_grass !C4 grass
   
!_____________ Comupte photosynthesis condition for each tree
Do no = 1, Max_no
   
   if ( .not. tree_exist(no)     ) cycle
   if ( .not. phenology(pft(no)) ) cycle
   if ( la(no) <=0.0             ) cycle
   
   !initialize
   p        = pft(no)
   psat(no) = Pmax(p) * 0.7 !initial psat, initial light_saturated photosynthesis rate
   co2cell  = CO2     * 0.7 !initial intercellular CO2 concentration
   
   !loop for parameter convergence
do loop = 0, 5
    !------------------------------------------------------------!
    !lue: light use efficiency (mol CO2 mon photon-1)
    !------------------------------------------------------------!
    a1      = (52.0-tmp_air) / (3.5+0.75*(52.0-tmp_air)) !temperature dependence factor
    a2      = co2cell / (90.0+0.6*co2cell)               !CO2 dependence factor
    lue(no) = Lue0(p) * a1 * a2
    
    !------------------------------------------------------------!
    ! ce_tmp: temperature effect on photosynthesis
    !------------------------------------------------------------!
    ! optimum photosynthesis temperature
    topt = Topt0(p) + 0.01 * co2cell
    
    ce_tmp = (tmp_air-Tmax(p)) * (tmp_air-Tmin(p)) &
             / ( (tmp_air-Tmax(p)) * (tmp_air-Tmin(p)) - (tmp_air-topt)**2 )
    ce_tmp = min(1.0, max(0.0, ce_tmp ))
    if ( tmp_air <= Tmin(p) ) ce_tmp=0.0
    if ( tmp_air >= Tmax(p) ) ce_tmp=0.0
    
    !------------------------------------------------------------!
    ! ce_co2: stomatal limitation on photosynthesis via co2cell
    !------------------------------------------------------------!
    ! compcd: CO2 compensation point
    a1         = 1.00+0.0451*(tmp_air-20.0)+0.000347*((tmp_air-20.0)**2)
    co2cmp(no) = CO2cmp0(p) * max(0.0, a1) 
    ce_co2     = 0.30 + 0.70 * (co2cell-co2cmp(no))/(KM(p)+co2cell)
    ce_co2     = min(1.0, max(0.0, ce_co2))
    
    !------------------------------------------------------------!
    !psat: light-saturated photosynthesis rate (micro mol CO2 m-2 s-1)
    !------------------------------------------------------------!
    psat(no)= Pmax(p) * ce_tmp * ce_co2 * ce_water(p)
    
    !------------------------------------------------------------!
    !ptop: canopy-top photosynthetic rate (micro mol CO2 m-2 s-1)
    !------------------------------------------------------------!
    a1 = psat(no) * lue(no) * par_ave(no)
    a2 = psat(no) + lue(no) * par_ave(no)
    
    if (a2 > 0.0) then; p_ave = a1/a2
    else              ; p_ave = 0.0  
    endif                           
    
    !------------------------------------------------------------!
    !gs_top: stomatal conductance on top of the leaf layer (mol H2O m-2 s-1)
    !        Leuning (1995)
    !------------------------------------------------------------!
    gs_top = GS_b1(p) + ( GS_b2(p)*p_ave ) / ( (CO2-co2cmp(no))*(1.0+vpd/GS_b3(p)) )
    
    !------------------------------------------------------------!
    !co2cell: intercellular CO2 concentration (ppmv)
    !------------------------------------------------------------!
    a1      = gs_top / 1.56    !1.56: conversion from H2O to CO2 conductance
    co2cell = CO2 - p_ave / a1 !Leuning (1995)
    co2cell = max(0.0, min(CO2, co2cell))
    
end do

END DO

!_____________ Comupte photosynthesis condition for each grass PFT
Do gn = 1, 2
   
   if (gn == 1) then
      p=C3g_no
   else
      p=C4g_no
   endif
   
!loop for parameter convergence
   psat_grass(gn) = Pmax(p) * 0.7 !initial psat, initial light_saturated photosynthesis rate
   co2cell        = CO2     * 0.7 !initial intercellular CO2 concentration
   
do loop = 0, 5
    !------------------------------------------------------------!
    !lue: light use efficiency (mol CO2 mon photon-1)
    !------------------------------------------------------------!
    if (gn==1) then
       a1 = (52.0-tmp_air) / (3.5+0.75*(52.0-tmp_air)) !temperature dependence factor
       a2 = co2cell / (90.0+0.6*co2cell)               !CO2 dependence factor
    else
       a1 = 1.0                                        !temperature insensitive
       a2 = 1.0                                        !CO2 insensitive 
    endif
    lue_grass(gn) = Lue0(p) * a1 * a2
    
    !------------------------------------------------------------!
    ! ce_tmp: temperature effect on photosynthesis
    !------------------------------------------------------------!
    ! optimum photosynthesis temperature
    if (gn==1) then
       topt = Topt0(p) + 0.01 * co2cell
    else
       topt = Topt0(p)
    end if
    
    if (gn==1) then
       topt = sum( tmp_ave_C3growth_RunningRecord(1:20) ) / 20.0
       topt = max(10.0, topt)
       topt = min(30.0, topt)
    else
       topt = sum( tmp_ave_C4growth_RunningRecord(1:20) ) / 20.0
       topt = max(20.0, topt)
       topt = min(40.0, topt)
    endif
    
    ce_tmp = (tmp_air-Tmax(p)) * (tmp_air-Tmin(p)) &
             / ( (tmp_air-Tmax(p)) * (tmp_air-Tmin(p)) - (tmp_air-topt)**2 )
    ce_tmp = min(1.0, max(0.0, ce_tmp ))
    if ( tmp_air <= Tmin(p) ) ce_tmp=0.0
    if ( tmp_air >= Tmax(p) ) ce_tmp=0.0
    
    !------------------------------------------------------------!
    ! ce_co2: stomatal limitation on photosynthesis via co2cell
    !------------------------------------------------------------!
    ! compcd: CO2 compensation point
    if(gn==1) then
       a1               = 1.00+0.0451*(tmp_air-20.0)+0.000347*((tmp_air-20.0)**2)
       co2cmp_grass(gn) = CO2cmp0(p) * max(0.0, a1) 
       ce_co2           = 0.30 + 0.70 * (co2cell-co2cmp_grass(gn))/(KM(p)+co2cell)
    else
       co2cmp_grass(gn) = CO2cmp0(p)
       ce_co2           = 0.50 + 0.50 * (co2cell-co2cmp_grass(gn))/(KM(p)+co2cell)
    endif
    ce_co2 = min(1.0, max(0.0, ce_co2))
    
    !------------------------------------------------------------!
    !psat: light-saturated photosynthesis rate (micro mol CO2 m-2 s-1)
    !------------------------------------------------------------!
    psat_grass(gn)= Pmax(p) * ce_tmp * ce_co2 * ce_water(p)
    
    !------------------------------------------------------------!
    !ptop: canopy-top photosynthetic rate (micro mol CO2 m-2 s-1)
    !------------------------------------------------------------!
    a1 = psat_grass(gn) * lue_grass(gn) * par_ave_grass(gn)
    a2 = psat_grass(gn) + lue_grass(gn) * par_ave_grass(gn)
    
    if (a2 > 0.0) then; p_ave = a1/a2
    else              ; p_ave = 0.0  
    endif                           
    
    !------------------------------------------------------------!
    !gs_top: stomatal conductance on top of the leaf layer (mol H2O m-2 s-1)
    !        Leuning (1995)
    !------------------------------------------------------------!
    gs_top = GS_b1(p) + ( GS_b2(p)*p_ave ) / ( (CO2-co2cmp_grass(gn))*(1.0+vpd/GS_b3(p)) )
    
    !------------------------------------------------------------!
    !co2cell: intercellular CO2 concentration (ppmv)
    !------------------------------------------------------------!
    a1      = gs_top/1.56        !1.56: conversion from H2O to CO2 conductance
    co2cell = CO2 - p_ave / a1   !Leuning (1995)
    co2cell = max(0.0, min(CO2, co2cell))
    
end do

END DO

END SUBROUTINE photosynthesis_condition



!**************************************************************************************************
! Daily GPP and Canopy_conductance
!**************************************************************************************************
SUBROUTINE photosynthesis &
          (par_direct, par_diffuse, par_grass, par_diffuse_rel, par_direct_rel, &
          doy, c_uptake, dlen, &
          pft_exist, phenology, eK, lue, psat, co2cmp, lue_grass, psat_grass, co2cmp_grass, &
          gpp, npp, gpp_month_ind, &
          tree_exist, pft, height, bole, crown_area, la, mass_leaf, mass_available, &
          mort_regu1, stat_leaf_crownbottom, stat_leaf_crowntop, stat_leaf, &
          stat_leaf_crownbottom_daily, &
          type_grass, lai_grass, gmass_available, &
          co2atm, vpd, canopy_cond, c3g_no, c4g_no)

USE data_structure
implicit none

!_____________ Define Arguments
real,intent(IN)::par_direct  !direct PAR of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_diffuse !difussed PAR of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_grass   !total PAR on top of the grass layer (micro mol photon m-2 s-1)

real,dimension(Max_hgt)          ,intent(IN)::par_diffuse_rel
  !Relative intensity of diffused PAR for each forest layer (0.0-1.0)

real,dimension(Max_no, Max_hgt),intent(IN)::par_direct_rel
  !Relative intensity of direct PAR for each foliage layer (0.0-1.0)

integer,intent(IN)   ::doy      !day of the year (1-365)
real   ,intent(INOUT)::c_uptake !monthly carbon uptake (C g / month stand)
real   ,intent(IN)   ::dlen     !day length (hour)

logical,dimension(PFT_no),intent(IN) ::pft_exist !PFT exist or not (.TRUE. or .FALSE.)
logical,dimension(PFT_no),intent(IN) ::phenology !phenological status (T:foliation, F:dormance)
real   ,dimension(PFT_no),intent(IN) ::eK        !light attenuation coefficient, no dimension
real   ,dimension(Max_no),intent(IN) ::lue       !light use efficiency (mol CO2 mol photon-1)
real   ,dimension(Max_no),intent(IN) ::psat      !light-saturated photosynthesis rate
real   ,dimension(Max_no),intent(IN) ::co2cmp    !CO2 compensation point (ppmv)
real   ,dimension(2)     ,intent(IN) ::lue_grass    !light use efficiency (mol CO2 mol photon-1)
real   ,dimension(2)     ,intent(IN) ::co2cmp_grass !CO2 compensation point (ppmv)
real   ,dimension(2)     ,intent(IN) ::psat_grass   !light-saturated photosynthesis rate

real,dimension(PFT_no),intent(OUT)  ::gpp           !GPP for each PFT (g dm / day / stand)
real,dimension(PFT_no),intent(OUT)  ::npp           !NPP for each PFT (g dm / day / stand)
real,dimension(PFT_no),intent(INOUT)::gpp_month_ind !GPP for each individual (g dm / ind / month)

logical,dimension(Max_no),intent(IN)   ::tree_exist !tree exist or not
integer,dimension(Max_no),intent(IN)   ::pft        !plant functional type
integer,dimension(Max_no),intent(IN)   ::height     !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no),intent(IN)   ::bole       !bole length from 1.3m above ground (STEP)
real   ,dimension(Max_no),intent(IN)   ::crown_area     !crown area (m2)
real   ,dimension(Max_no),intent(IN)   ::la             !leaf area (m2)
real   ,dimension(Max_no),intent(IN)   ::mass_leaf      !biomass of foliage (g)
real   ,dimension(Max_no),intent(INOUT)::mass_available !biomass available (g)
real   ,dimension(Max_no),intent(INOUT)::mort_regu1     !annual NPP (g / individual)
real   ,dimension(Max_no,10),intent(INOUT)::stat_leaf_crownbottom !annual NPP of foliage bottom
real   ,dimension(Max_no),intent(INOUT)::stat_leaf_crowntop    !annual NPP of foliage top
real   ,dimension(Max_no),intent(INOUT)::stat_leaf !benefit of producing new leaf mass (g/g Leaf)

!Daily physiological parameters
real,dimension(Max_no),intent(INOUT)::stat_leaf_crownbottom_daily !stat_leaf on the bottom of crown layer

integer               ,intent(IN)   ::type_grass      !priority grass type (3:C3, 4:C4)
real   ,dimension(2),intent(IN)   ::lai_grass       !lai of grass (m2/m2)
real   ,dimension(2),intent(INOUT)::gmass_available !grass available biomass (g/m2)

real                  ,intent(IN) ::co2atm      !ambient (canopy) CO2 concentration (ppmv)
real                  ,intent(IN) ::vpd         !vapour pressure deficit (hPa)
real                  ,intent(OUT)::canopy_cond !stomatal conductance   (mol H2O m-2 s-1)

integer,intent(IN)::c3g_no !PFT number of C3 grass
integer,intent(IN)::c4g_no !PFT number of C4 grass

!_____________ Define local variables
real    const0, const1, const2, const3, const4, const5
real    lai                !leaf area index (m2/m2)
real    gpp_ind
real    a0, a1, a2, a3, a4 !for general usage
integer i, j, no, p, gn    !for loop counter
integer count              !counter

!_____________ Define Local parameter
real Unit_converter
     Unit_converter = 3600.0 * 12.0 / 1000000.0 / C_in_drymass
   ! constant for unit conversion
   ! from (micro mol CO2 m-2 s-1) to (g dm m-2 hour-1)
   ! micro mol  -> mol       :* 10^-6
   ! mol        -> g(Carbon) :* 12
   ! secound^-1 -> hour^-1   :* 3600

!_____________ Woody PFTs
count = 0
DO no=1, Max_no
   
   !initialize gpp of a tree
   gpp_ind = 0.000
   p = pft(no)
   
   if ( .not. tree_exist(no) ) cycle
   if ( .not. phenology(p)   ) cycle
   if ( la(no)  <=0.0        ) cycle
   if ( psat(no) <= 0.0      ) cycle
   
   const0 = GS_b2(p) * psat(no) / max(0.01, co2atm-co2cmp(no)) / (1.0 + vpd/GS_b3(p))
   const4 = Unit_converter * dlen * psat(no)  !
   const5 = la(no) / (height(no)-bole(no))    !leaf area per crown layer (m2/step)
   do i = height(no), bole(no)+1, -1
      !Photosynthesis
      a1      = par_diffuse*par_diffuse_rel(i) + par_direct*par_direct_rel(no,i) !PAR income
      a2      = 1.0 - (1.0+a1*lue(no)/psat(no))**(-0.5)
      a4      = const4*const5*a2                             !GPP (g dm/layer/day)
      gpp_ind = gpp_ind + a4
      
      !Canopy conductance
      canopy_cond = canopy_cond + const5 * ( GS_b1(p) + const0 * a2 )
      
      !Production recorder for each foliage disk
      if (i-bole(no)<=10) stat_leaf_crownbottom(no, i-bole(no)) = & 
                          stat_leaf_crownbottom(no, i-bole(no)) + a4
      if (i==height(no) ) stat_leaf_crowntop(no)                = & 
                          stat_leaf_crowntop(no)                + a4
      if (i==bole(no)+1 ) stat_leaf_crownbottom_daily(no)       = & 
                          stat_leaf_crownbottom_daily(no)       + a4
   end do
   
   !Increase mass_available & c_uptake
   mass_available (no) = mass_available(no) + gpp_ind
   gpp(p)              = gpp(p)             + gpp_ind
   npp(p)              = npp(p)             + gpp_ind
   gpp_month_ind(no)   = gpp_month_ind(no)  + gpp_ind
   
   mort_regu1(no)      = mort_regu1(no)     + gpp_ind
   c_uptake            = c_uptake           + gpp_ind * C_in_drymass
   
   !Benefit of producing new leaf mass (g / g Leaf)
   stat_leaf(no) = gpp_ind / mass_leaf(no)
   
END DO

!_____________ Herbaceous PFTs
DO gn=1,2
   !set PFT number, p
                p = c4g_no
   if ( gn==1 ) p = c3g_no
   
   !obtain coverage area: a0
                                    a0 = 0.1 * Max_loc*Max_loc
   if ( gn==1 .and. type_grass==3 ) a0 = 0.9 * Max_loc*Max_loc
   if ( gn==2 .and. type_grass==4 ) a0 = 0.9 * Max_loc*Max_loc
   
   
   !obtain GPP per unit area
   if (psat_grass(gn)>0.0) then
     const0 = GS_b2(p) * psat_grass(gn) / max(0.01, co2atm-co2cmp_grass(gn)) / (1.0 + vpd/GS_b3(p))
     const1 = Unit_converter * 2.0 * dlen * psat_grass(gn) / eK(p)
     const2 = eK(p) * lue_grass(gn) / psat_grass(gn)
     
      !obtain daily gpp per unit area (g dm / m2 / day)
      a2     = 1.0 + sqrt( 1.0 + par_grass * const2 )
      a3     = 1.0 + sqrt( 1.0 + par_grass * const2 * exp(-1.0*eK(p)*lai_grass(gn)) )
      const3 = log(a2/a3)
      gpp(p) = const1 * const3
      
      !increase gmass_available (g dm / m2 / day)
      gmass_available(gn) = gmass_available(gn) + gpp(p)
      
      !carbon balance
      gpp(p)   = gpp(p) * a0
      npp(p)   = gpp(p)
      c_uptake = c_uptake + gpp(p) * C_in_drymass
      
      !canopy conductance
      canopy_cond = canopy_cond + a0*( GS_b1(p)*lai_grass(gn) + const0*(2.0/eK(p))*const3 )
   endif
END DO

!Adjust canopy conductance
canopy_cond = canopy_cond / Max_loc / Max_loc

END SUBROUTINE photosynthesis



!**************************************************************************************************
! Optimum Leaf Area Index: daily computation using running mean of PAR
!**************************************************************************************************
SUBROUTINE lai_optimum &
    (lai_opt_grass, lai_opt_grass_RunningRecord, c3g_no, c4g_no, &
     pft_exist, eK, lue_grass, psat_grass, &
     dlen, tmp_air, tmp_soil, par, par_grass, &
     counter, pheno_clo, pheno_cls)

USE data_structure
implicit none
common PN_s, PN_r

!Define common variables
real,dimension(PFT_no)::PN_s !proportion of nitrogen:stem (gN / gBiomass)
real,dimension(PFT_no)::PN_r !proportion of nitrogen:root (gN / gBiomass)

!Define argments
real   ,intent(IN)::dlen      !day length (hour)
real   ,intent(IN)::tmp_air   !2m air temperature (Celcius)
real   ,intent(IN)::tmp_soil  !soil temperature at 10 cm  depth (Celcius)
real   ,intent(IN)::par       !PAR of mid-day (micro mol photon m-2 s-1)
real   ,intent(IN)::par_grass !PAR on top of the grass layer (micro mol photon m-2 s-1)
integer,intent(IN)::c3g_no    !PFT number of C3 grass
integer,intent(IN)::c4g_no    !PFT number of C4 grass

logical,dimension(PFT_no),intent(IN)::pft_exist  !PFT exist or not (.TRUE. or .FALSE.)
real   ,dimension(PFT_no),intent(IN)::eK         !light attenuation coefficient, no dimension
real   ,dimension(2)     ,intent(IN)::lue_grass  !monthly quantum yield (mol CO2 mol photon-1)
real   ,dimension(2)     ,intent(IN)::psat_grass !light-saturated rate (micro mol CO2 m-2 s-1)

real,dimension(2)     ,intent(OUT)  ::lai_opt_grass  !Optimum LAI of grass layer (m2/m2)
real,dimension(2,20),intent(INOUT)::lai_opt_grass_RunningRecord !RunningRecord of lai_opt_grass

integer                  ,intent(IN)::counter   !simulation day (1-Simulation_year*365)
integer,dimension(PFT_no),intent(IN)::pheno_clo !counter number of last leaf onset
integer,dimension(PFT_no),intent(IN)::pheno_cls !counter number of last leaf shedding

!Define local variables
real    qt                      !QT10 of maintenance respiration
real    tmp_sensibility_air     !temperature sensibility multiplier for foliage and stem
real    tmp_sensibility_soil    !temperature sensibility multiplier for root
real    x, y, a0, a1, a2, a3    !for general usage
integer gn, p                   !loop counter
real    const1, const2          !temporal usage

!Prepare temperature sensibility
   qt                   = 2.0 * exp( -0.009 * (tmp_air - 15.0) )
   tmp_sensibility_air  = exp( (tmp_air  - 15.0) * log(qt) / 10.0 )
   qt                   = 2.0 * exp( -0.009 * (tmp_soil - 15.0) )
   tmp_sensibility_soil = exp( (tmp_soil - 15.0) * log(qt) / 10.0 )

!Each PFT procedures
lai_opt_grass(:) = 0.0
do gn = 1 , 2
   
              p=C3g_no
   if (gn==2) p=C4g_no
   
   if (dlen           <= 0.1   ) cycle
   if (psat_grass(gn) <= 0.001 ) cycle
   if (lue_grass(gn)  <= 0.001 ) cycle
   if (eK(p)          <= 0.001 ) cycle
   if (par_grass      <= 1.00  ) cycle
   
   !a0: maintenance respiration rate of leaf (gDM / gDM / day)
   a0 = RM(p) * PN_f(p) * tmp_sensibility_air
   
   !a1: turnover cost of leaf (g DM g DM-1 day-1)
   if ( counter-pheno_clo(p)>365 ) then
      !evergreen
      a1 = (TO_f(p)/365.0) * ( RG_f(p) - RG_f_suck(p) )
   else
       !summer or rain green
      x  = max( 1   , 365 - (pheno_clo(p)-pheno_cls(p)) ) !x: number of growth day
      y  = max( 0.0 , TO_f(p) - 365/x )                   !y: corrected turnover rate
      a1 = (y      /365.0) * ( RG_f(p) - RG_f_suck(p) )
   endif
   
   !a2: maintenance respiration rate of root per unit leaf-mass (g DM g DM-1 day-1)
   a2 = RM(p) * PN_r(p) * tmp_sensibility_soil / FR_ratio(p)
   
   !a3: turnover cost of root (g DM g DM-1 day-1)
   !a3 = 0.0
   a3 = (TO_r(p)/365.0) * RG_r(p) / FR_ratio(p)
   
   !const1: daily maintenance cost of leaf (gDM / m2)
   const1 = (a0+a1+a2+a3)/SLA(p)
   const1 = max(0.0, const1)
   
   !optimum leaf index
   const2 = 1.0 - const1 / ( 0.09093*dlen*psat_grass(gn) )
   const2 = 1.0 / (const2*const2) - 1.0
   const2 = (psat_grass(gn)/lue_grass(gn)) * const2
   
   if ( const2>0.0 ) then
      lai_opt_grass(gn) = ( log(par_grass/const2) ) / eK(p)
      lai_opt_grass(gn) = max(0.0, lai_opt_grass(gn))
   else
      lai_opt_grass(gn) = 0.0
   endif
   
end do

!Procedures running records   
   lai_opt_grass_RunningRecord(1, 2:20) = lai_opt_grass_RunningRecord(1, 1:19)
   lai_opt_grass_RunningRecord(1, 1)    = lai_opt_grass(1)
   
   lai_opt_grass_RunningRecord(2, 2:20) = lai_opt_grass_RunningRecord(2, 1:19)
   lai_opt_grass_RunningRecord(2, 1)    = lai_opt_grass(2)
   
END SUBROUTINE lai_optimum



!**************************************************************************************************
! Leaf phenology (daily computation)
!**************************************************************************************************
SUBROUTINE leaf_season &
            (LAT, doy, counter, c_uptake, c_emission, pool_litter, &
            tmp_air_RunningRecord, tmp_coldest_20yr_ave, tmp_soil, c3g_no, c4g_no, &
            pft_exist, npp, npp_10d_ave, stat_water_RunningRecord, &
            phenology, pheno_clo, pheno_cls, pheno_gdd, pheno_ncd, &
            pft, tree_exist, la, mass_leaf, mass_stock, mass_available, &
            type_grass, lai_grass, gmass_leaf, gmass_stock, gmass_available, &
            lai_opt_grass, lai_opt_grass_RunningRecord)

USE data_structure
implicit none

!Define arguments
   real   ,intent(IN)   ::LAT         !latitude  (degree)
   integer,intent(IN)   ::doy         !loop for day of year (1-365)
   integer,intent(IN)   ::counter     !simulation day (1-Simulation_year*365)
   real   ,intent(INOUT)::c_uptake    !monthly carbon uptake   (C g / month stand)
   real   ,intent(INOUT)::c_emission  !monthly carbon emission (gC/month/stand)
   
   real   ,intent(INOUT)::pool_litter !litter pool in a stand (g drymatter / stand)
   
   real,dimension(365),intent(IN)::tmp_air_RunningRecord !365days running record  of tmp_air
   real               ,intent(IN)::tmp_coldest_20yr_ave  !20yr running average of tmp_coldest_month
   real               ,intent(IN)::tmp_soil
   
   logical,dimension(PFT_no)    ,intent(IN)   ::pft_exist   !PFT exist or not (True or Faulse)
   real   ,dimension(PFT_no)    ,intent(INOUT)::npp         !NPP for each pft (g dm / stand / day)
   real   ,dimension(PFT_no)    ,intent(IN)   ::npp_10d_ave !10 days running average of NPP
   real   ,dimension(PFT_no,365),intent(IN)   ::stat_water_RunningRecord
   
   logical,dimension(PFT_no),intent(INOUT)::phenology !phenological status (True or Faulse)
   integer,dimension(PFT_no),intent(INOUT)::pheno_clo !counter-number of last leaf onset
   integer,dimension(PFT_no),intent(INOUT)::pheno_cls !counter-number of last leaf shedding
   real                     ,intent(IN)   ::pheno_gdd !GDD on 5 degree base
   integer                  ,intent(IN)   ::pheno_ncd !Number of chilling day
   
   integer,intent(IN)::c3g_no           !PFT number of C3 grass
   integer,intent(IN)::c4g_no           !PFT number of C4 grass
   
   integer,dimension(Max_no),intent(IN)   ::pft            !Plant FUnctional Type
   logical,dimension(Max_no),intent(IN)   ::tree_exist     !
   real   ,dimension(Max_no),intent(INOUT)::la             !leaf area   (m2)
   real   ,dimension(Max_no),intent(INOUT)::mass_leaf      !foliage mass (g)
   real   ,dimension(Max_no),intent(INOUT)::mass_stock     !stock mass (g)
   real   ,dimension(Max_no),intent(INOUT)::mass_available !mass available (g)
   
   integer                 ,intent(IN)   ::type_grass      !grass type priority (3:C3, 4:C4)
   real,dimension(2)     ,intent(INOUT)::lai_grass       !leaf area index of grass (m2/m2)
   real,dimension(2)     ,intent(INOUT)::gmass_leaf      !foliage mass of grass    (g/m2)
   real,dimension(2)     ,intent(INOUT)::gmass_stock     !stock mass of grass      (g/m2)
   real,dimension(2)     ,intent(INOUT)::gmass_available !available mass of grass  (g/m2)
   real,dimension(2)     ,intent(IN)   ::lai_opt_grass   !Optimum LAI for grass   (m2/m2)
   real,dimension(2,20),intent(IN)   ::lai_opt_grass_RunningRecord
   
!Define local variables
   real    stockmass_min_tree  !minimum stock mass for dormance (g dm/individual)
   real    stockmass_min_grass !minimum stock mass for dormance (g dm/m2)
   
   real tmp_air_10d_ave !1- days running mean of air temperature (Celcius)
   
   integer i, j, no    !loop counter
   logical flag_season !flag for seasonal cycle
   logical flag        !flag for phenology change
   integer k,l,m,n   !temporal integer value
   real    x,y         !temporal real value
   
   real mass_assm    !biomass assimirated (g DM/day/stand)
   real mass_combust !biomass combust     (g DM/day/stand)
   real mass_total   !total biomass before procedure --for checker-- (g DM/stand)
   real a1, a2, a3   !for general usage
   
!set parameters
   stockmass_min_tree  = 10.0
   stockmass_min_grass = 50.0
!prepare climatic factor 1
   tmp_air_10d_ave = sum( tmp_air_RunningRecord(1:10) ) / 10.0
   
!prepare climatic factor 2
   m = 0 ; n = 0 ; l = 7
   do i = 0, 4
      x = sum( tmp_air_RunningRecord(1+i*l    :l+i*l    ) )
      y = sum( tmp_air_RunningRecord(1+(i+1)*l:l+(i+1)*l) )
      if ( x > y ) then
         m = m + 1
      else
         n = n + 1
      endif
   enddo
   
   if (m>n) then
      flag_season = .false.  !temperature increasing
   else
      flag_season = .true.   !temperature decreasing
   endif
   
!for carbon leaking checker
   a1 = gmass_leaf(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = (0.9*a1 + 0.1*a2) * (Max_loc**2)
   else
      a3 = (0.1*a1 + 0.9*a2) * (Max_loc**2)
   end if
   mass_total = a3 + sum(mass_leaf)+sum(mass_available)+sum(mass_stock) + pool_litter
   
!For every available PFTs
mass_assm    = 0.000
mass_combust = 0.000

DO i = 1, PFT_no
IF (.not. pft_exist(i))     cycle
IF (Phenology_type(i) == 0) cycle
    
    flag = .false.
   
   !************ On foliation phase *****************************************
   IF (phenology(i)) then
    
    !----------------- Checker (Foliage -> Domance) ------------------------
    x = tmp_air_10d_ave
    select case ( Phenology_type(i))
      
      case (1) !BoNS
        !if ( (x<2.0) .or. (x<10.0+tmp_coldest_20yr_ave) )    flag=.true.
        flag=.true.
        do j = 1, 7
           if (tmp_air_RunningRecord(j) >= -5.0)             flag=.false.
        enddo
        
      case (2) !BoBS
        if ( tmp_soil<2.0 )                                  flag=.true.
        
      case (3) !TeBS
        if ( (x<9.0) .or. (x<10.0+tmp_coldest_20yr_ave) )    flag=.true.
        
      case (4) !water-controlling PFTs (Botta et al. 2000)
        if ( sum(stat_water_RunningRecord(i,1:10)) / 10.0 < 0.5 ) flag=.true.
        
      case (5) !C3 grass
         flag=.true.
         do k=1, 7
           if (lai_opt_grass_RunningRecord(1,k)>0.01)        flag=.false.
         enddo
        
      case (6) !C4 grass
         flag=.true.
         do k=1, 7
           if (lai_opt_grass_RunningRecord(2,k)>0.01)        flag=.false.
         enddo
        
    end select
    if ( counter - pheno_clo(i) < 60 ) flag=.false.
    !-----------------------------------------------------------------------
    
    !----------------- Procedure (Foliage -> Dormance) ---------------------
    if (flag) then
      
      if (Logging==1) write (*,*) 'Foliage phase ---> Domance phase', i, doy
      phenology(i) = .false.
      pheno_cls(i) = counter
      
      !for woody PFTs
      if ( (i.ne.c3g_no) .and. (i.ne.c4g_no) ) then
         do no=1, Max_no
         if (tree_exist(no) .and. pft(no)==i) then
             
             mass_available(no) = mass_available(no) + mass_leaf(no) * RG_f_suck(i)
             pool_litter        = pool_litter        + mass_leaf(no) * ( 1.0-RG_f_suck(i) )
             mass_leaf(no)      = 0.0
             la(no)             = 0.0
             
             !give minimum stock mass
             if (mass_stock(no) < stockmass_min_tree) then
                pool_litter    = pool_litter + mass_stock(no)
                pool_litter    = pool_litter - stockmass_min_tree
                mass_stock(no) = stockmass_min_tree
             end if
             
         end if
         end do
     
     !for C3 grass PFTs
     elseif (i==c3g_no) then
          if (type_grass==3) then; x = 0.9 * (Max_loc**2)
          else                   ; x = 0.1 * (Max_loc**2)
          endif
          
          gmass_available(1) = gmass_available(1) + gmass_leaf(1) * RG_f_suck(i)           
          pool_litter        = pool_litter        + gmass_leaf(1) * (1.0- RG_f_suck(i)) * x
          gmass_leaf(1)      = 0.0
          lai_grass (1)      = 0.0
          
          !give minimum stock mass
          if (gmass_stock(1) < stockmass_min_grass) then
             pool_litter    = pool_litter - (stockmass_min_grass-gmass_stock(1)) * x
             gmass_stock(1) = stockmass_min_grass
             
             if ( pool_litter < 0.0 ) then
                mass_assm   = mass_assm - pool_litter
                pool_litter = 0.0
             endif
             
          end if
          
      !for C4 grass PFTs
     else
          if (type_grass==4) then; x = 0.9 * (Max_loc**2)
          else                   ; x = 0.1 * (Max_loc**2)
          endif
          
          gmass_available(2) = gmass_available(2) + gmass_leaf(2) *  RG_f_suck(i)          
          pool_litter        = pool_litter        + gmass_leaf(2) * (1.0- RG_f_suck(i)) * x
          gmass_leaf(2)      = 0.0
          lai_grass (2)      = 0.0
          
          !give minimum stock mass
          if (gmass_stock(2) < stockmass_min_grass) then
             pool_litter    = pool_litter - (stockmass_min_grass-gmass_stock(2)) * x
             gmass_stock(2) = stockmass_min_grass
             
             if ( pool_litter < 0.0 ) then
                mass_assm   = mass_assm - pool_litter
                pool_litter = 0.0
             endif
             
          end if
          
      endif
      
   endif
    !-----------------------------------------------------------------------
    
   !************ On dormance phase ****************************************
   ELSE
    
    !----------------- Checker (Dormance -> Foliage) -----------------------
    !                 [ Botta et al 2000 procedure ]                        
    select case ( Phenology_type(i) )
      
      case (1) !boreal deciduous needle-leaved-woods
        j = 0
        do k = 1, 60
            if (tmp_air_RunningRecord(k) > -5.0) j=j+1
        end do
        
        if ( j >= 15 )                                          flag = .true. 
        if ( LAT>=0.0 .and. (doy< 30+LAT .or. doy>130+LAT) )    flag = .false.
        if ( LAT< 0.0 .and. (doy<212-LAT .or. doy>312-LAT) )    flag = .false.
        
      case (2:3) !other temperature-controlling PFTs
        if ( pheno_gdd > -68.0 + 638.0 * exp(-0.01*pheno_ncd) ) flag = .true. 
        if ( LAT>=0.0 .and. (doy< 30+LAT .or. doy>130+LAT) )    flag = .false.
        if ( LAT< 0.0 .and. (doy<212-LAT .or. doy>312-LAT) )    flag = .false.
        
      case (4) !water-controlling deciduous
        !if ( sum( stat_water_RunningRecord(i,51:55) ) > &
        !     sum( stat_water_RunningRecord(i,56:60) )   )           flag = .true.
        !
        !j = 0
        !do k = 25, 50
        !   if ( stat_water_RunningRecord(i,k) > stat_water_RunningRecord(i,k+1) ) j=j+1
        !end do
        !if ( j<13 )                                                 flag = .false.
        if ( sum(stat_water_RunningRecord(i,1:10)) / 10.0 > 0.5 )    flag = .true.
        
      case (5) !C3grass
        flag=.true.
        do k=1, 7
           if (lai_opt_grass_RunningRecord(1,k)<=0.01)   flag=.false.
        enddo
        
        if ( LAT>=0.0 .and. (doy< 30+LAT .or. doy>130+LAT) )         flag = .false.
        if ( LAT< 0.0 .and. (doy<212-LAT .or. doy>312-LAT) )         flag = .false.
        
      case (6) !C4grass
        flag=.true.
        do k=1, 7
           if (lai_opt_grass_RunningRecord(2,k)<=0.01)   flag=.false.
        enddo
        
        if ( LAT>=0.0 .and. (doy< 30+LAT .or. doy>130+LAT) )         flag = .false.
        if ( LAT< 0.0 .and. (doy<212-LAT .or. doy>312-LAT) )         flag = .false.
        
    end select
    if ( counter - pheno_cls(i) < 60 ) flag=.false.
    
    !-----------------------------------------------------------------------
    
    !----------------- Procedure (Dormance -> Foliage) ---------------------
    if (flag) then
        if (Logging==1) write (*,*) 'Domance phase ---> Foliage phase', i, doy
        phenology(i) = .true.
        pheno_clo(i) = counter
    endif
    !-----------------------------------------------------------------------
    
    ENDIF
    
   !************ gradual release of stock energy (trees)  *****************
    l = 14 !day lengh required for full foliation for woody PFTs
    IF ( (i.ne.c3g_no) .and. (i.ne.c4g_no) )   then
    IF (pheno_clo(i)+l > counter) then
        do no=1, Max_no
        if (tree_exist(no) .and. pft(no)==i) then
            !x: stock mass for releasing in the day (g)
            x = mass_stock(no)/(l+pheno_clo(i)-counter)
            
            !release process
            mass_available (no) = mass_available(no) + x / RG_stock_out
            mass_stock     (no) = mass_stock    (no) - x
            mass_combust        = mass_combust       + (x - x / RG_stock_out)
            npp            (i)  = npp(i)             - (x - x / RG_stock_out)
        endif
        enddo
    ENDIF
    ENDIF
    
   !************ gradual release of stock energy (C3 grass)  *****************
    l = 14 !day lengh required for full foliation for C3 grass
    IF ( i == c3g_no                  ) then
    IF ( pheno_clo(i)+l   >  counter  ) then
    IF ( phenology(c3g_no)            ) then
       
       !x: stock mass consumed within a day (g/m2)
       x = gmass_stock(1) / (l+pheno_clo(c3g_no)-counter)
       
       !a1: fraction of cover on forest floor
       if (type_grass==3) then 
         a1 = 0.9
       else
         a1 = 0.1
       endif
       
       !release process
       gmass_available(1) = gmass_available(1) + x / RG_stock_out
       gmass_stock(1)     = gmass_stock(1)     - x
       mass_combust       = mass_combust       + (x - x / RG_stock_out)*Max_loc*Max_loc*a1
       npp(c4g_no)        = npp(c4g_no)        - (x - x / RG_stock_out)*Max_loc*Max_loc*a1
       
    ENDIF
    ENDIF
    ENDIF
    
   !************ gradual release of stock energy (C4 grass)  *****************
    l = 14 !day lengh required for full foliation for C4 grass
    IF ( i == c4g_no                   ) then
    IF ( pheno_clo(c4g_no)+l > counter ) then
    IF ( phenology(c4g_no)             ) then
       
       !x: stock mass consumed within a day (g/m2)
       x = gmass_stock(2) / (l+pheno_clo(c4g_no)-counter)
       
       !a1: coverage proportion
       if (type_grass==4) then 
         a1 = 0.9
       else
         a1 = 0.1
       endif
       
       !release process
       gmass_available(2) = gmass_available(2) + x / RG_stock_out
       gmass_stock(2)     = gmass_stock(2)     - x
       mass_combust       = mass_combust  + (x - x / RG_stock_out)*Max_loc*Max_loc*a1
       npp(c4g_no)         = npp(c4g_no)    - (x - x / RG_stock_out)*Max_loc*Max_loc*a1
       
    ENDIF
    ENDIF
    ENDIF
    
END DO

!carbon leak checker
   a1 = gmass_leaf(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = (0.9*a1 + 0.1*a2) * (Max_loc**2)
   else
      a3 = (0.1*a1 + 0.9*a2) * (Max_loc**2)
   end if
   a3 = a3 + sum(mass_leaf)+sum(mass_available)+sum(mass_stock) + pool_litter
   a3 = a3 + mass_combust - mass_assm
   
   if ( abs(mass_total-a3)>20.0 .and. Logging==1 ) then
      write (*,*) 'Error @ Leaf_season: leaking mass (g) ->', doy, mass_total-a3
   endif
   
!Total carbon emission by maintenance respiration
   c_emission = c_emission + mass_combust * C_in_drymass
   c_uptake   = c_uptake   + mass_assm    * C_in_drymass
   
END SUBROUTINE leaf_season



!**************************************************************************************************
! Maintenance respiration (daily computation)
!**************************************************************************************************
SUBROUTINE maintenance_resp &
        (doy, pft, height, bole, tree_exist, dbh_heartwood, dbh_sapwood, la, &
        mass_leaf, mass_trunk, mass_root, mass_available, mass_stock, &
        stat_leaf_crownbottom, stat_leaf_crowntop, stat_leaf, stat_leaf_crownbottom_daily, &
        type_grass, lai_grass, c3g_no, c4g_no, &
        gmass_leaf, gmass_root, gmass_available, gmass_stock, &
        pool_litter, c_emission, phenology, npp, mort_regu1, tmp_air, tmp_soil )

USE data_structure
implicit none
common PN_s, PN_r

!Set parameter
real,parameter::frac_organ_remove=0.01 !daily proportion biomass remove
                                       !when resource for maintenance respiration resource absent

!PFT specific constants
real,dimension(PFT_no)::PN_s !proportion of nitrogen:stem (gN / gBiomass)
real,dimension(PFT_no)::PN_r !proportion of nitrogen:root (gN / gBiomass)

!Define arguments
integer,intent(IN)::doy         !day of the year (1-365)

real,intent(IN)   ::tmp_air     !air  temperature (Cecius)
real,intent(IN)   ::tmp_soil    !soil temperature (Cecius)
real,intent(INOUT)::pool_litter !litter pool in a stand (g / stand)
real,intent(INOUT)::c_emission  !monthly carbon emission (C g / month stand)

logical,dimension(PFT_no)  ,intent(IN)   ::phenology  !phenological status
real   ,dimension(PFT_no)  ,intent(INOUT)::npp        !NPP for each pft (g dm / stand / day)
real   ,dimension(Max_no),intent(INOUT)::mort_regu1 !annual NPP (g dm)

integer,dimension(Max_no)   ,intent(IN)   ::pft        !Plant Functional Type (1-PFT_no)
integer,dimension(Max_no)   ,intent(IN)   ::height     !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no)   ,intent(IN)   ::bole       !bole length from 1.3m above ground (STEP)
logical,dimension(Max_no)   ,intent(IN)   ::tree_exist     !tree exist or not
real,dimension(Max_no),intent(INOUT)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real,dimension(Max_no),intent(INOUT)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real,dimension(Max_no),intent(INOUT)::la             !leaf area (m2)
real,dimension(Max_no),intent(INOUT)::mass_leaf      !biomass of foliage (g)
real,dimension(Max_no),intent(IN)   ::mass_trunk     !biomass of stem (g)
real,dimension(Max_no),intent(INOUT)::mass_root      !biomass of root (g)
real,dimension(Max_no),intent(INOUT)::mass_available !biomass available (g)
real,dimension(Max_no),intent(INOUT)::mass_stock     !biomass for stock (g)
real,dimension(Max_no,10),intent(INOUT)::stat_leaf_crownbottom !annual NPP of foliage bottom
real,dimension(Max_no),intent(INOUT)::stat_leaf_crowntop !annual NPP of foliage top
real,dimension(Max_no),intent(INOUT)::stat_leaf !benefit of producing new leaf mass (g/g Leaf)
real,dimension(Max_no),intent(INOUT)::stat_leaf_crownbottom_daily !daily npp on the bottom of crown layer

integer,intent(IN)::c3g_no           !PFT number of C3 grass
integer,intent(IN)::c4g_no           !PFT number of C4 grass

integer                ,intent(IN)  ::type_grass      !grass priority (3:C3 type, 4:C4 type)
real   ,dimension(2),intent(INOUT)::lai_grass       !leaf area index of grass (m2/m2)
real   ,dimension(2),intent(INOUT)::gmass_leaf      !foliage mass of grass (g/m2)
real   ,dimension(2),intent(INOUT)::gmass_root      !root mass of root (g/m2)
real   ,dimension(2),intent(INOUT)::gmass_available !available mass of grass (g/m2)
real   ,dimension(2),intent(INOUT)::gmass_stock     !herbaceous stock biomass (g/m2)

!Define local variables
real    qt                    !QT
real    tmp_sensibility_air   !temperature sensibility multiplier for foliage and stem
real    tmp_sensibility_soil  !temperature sensibility multiplier for root
real    resp_15C_air          !maintenance respiration mass of organ in air  at 15C (g)
real    resp_15C_soil         !maintenance respiration mass at organ in soil at 15C (g)
real    mass_required         !resource required for maintenance respiration (g)
real    mass_resp             !resource usage for mainatenance respiration (g)
real    proportion_of_sapwood !sapwood_mass / stem_mass

real    mass_total       !total biomass before procedure --for checker-- (g DM/stand)
real    mass_combust     !biomass combust (g DM/day/stand)
real    mass_resp_grass  !respiration by c3 grass (g dm / m2 / day)
real    litter_sum_grass !mass of litter production by c3 grass (g dm/m2/day)

integer no, i, p, gn  !loop counter
real::a1, a2, a3      !for general usage
real::x, y, z         !for general usage

!**Record total carbon in this virtual forest for carbon-leak-chacking
   !woody PFTs
   mass_total = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock)+pool_litter
   !grass PFTs
   a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = (0.90*a1 + 0.10*a2) * (Max_loc**2)
   else
      a3 = (0.10*a1 + 0.90*a2) * (Max_loc**2)
   end if
   mass_total = mass_total + a3
   
!*** Set initial values and 
   !dry-mass to combust (g dm / stand)
   mass_combust = 0.000
   
   !temperature sensibility, for foliage and sapwood
       !for leaves and stem
   qt                   = 2.0 * exp( -0.009 * (tmp_air-15.0) )
   tmp_sensibility_air  = exp( (tmp_air-15.0) * log(qt) / 10.0 )
       !for root
   qt                   = 2.0 * exp( -0.009 * (tmp_soil-15.0) )
   tmp_sensibility_soil = exp( (tmp_soil - 15.0) * log(qt) / 10.0 )

!*** Woody PFTs
DO no=1, Max_no
If ( tree_exist(no)      ) then  !when tree exist
   
   !set PFT number
   p = pft(no)
   
   !Calculate daily resource required for maintenance respiration
   proportion_of_sapwood = 1.0-(dbh_heartwood(no)**2)/( (dbh_sapwood(no)+dbh_heartwood(no))**2 ) 
   resp_15C_air         = RM(p) * PN_f(p) * mass_leaf(no) + &
                          RM(p) * PN_s(p) * mass_trunk(no) * proportion_of_sapwood
   resp_15C_soil        = RM(p) * PN_r(p) * mass_root(no)
   mass_required        = resp_15C_air  * tmp_sensibility_air  + &
                          resp_15C_soil * tmp_sensibility_soil    
   
   !phase 1: use mass_available
   mass_resp            = min(mass_required, mass_available(no))
   mass_required        = mass_required      - mass_resp
   mass_available(no)   = mass_available(no) - mass_resp
   mass_combust         = mass_combust       + mass_resp
   npp(p)               = npp(p)             - mass_resp
   mort_regu1(no)       = mort_regu1(no)     - mass_resp
   
   !phase 2: use mass_stock
   if (mass_required > 0.001) then
      mass_resp            = min(mass_required, mass_stock(no)/ RG_stock_out)
      mass_required        = mass_required  - mass_resp
      mass_stock(no)       = mass_stock(no) - mass_resp * RG_stock_out
      mass_combust         = mass_combust   + mass_resp * RG_stock_out
      npp(p)               = npp(p)         - mass_resp * RG_stock_out
      mort_regu1(no)       = mort_regu1(no) - mass_resp * RG_stock_out
   end if
   
   !phase 3: when resource absent, a portion of leaves drop
   if (mass_required > 0.001) then
      a1 = frac_organ_remove * mass_leaf(no)
      a2 = frac_organ_remove * mass_root(no)
      
      pool_litter   = pool_litter + a1 + a2
      mass_leaf(no) = mass_leaf(no)  - a1
      mass_root(no) = mass_root(no)  - a2
      la(no)        = mass_leaf(no) * SLA(p)
      
      x = dbh_sapwood(no)
      y = dbh_heartwood(no)
      z = sqrt( y*y + frac_organ_remove * ( x*x + 2.0*x*y ) ) - y
      dbh_sapwood(no) = dbh_sapwood(no) - z
      dbh_heartwood(no) = dbh_heartwood(no) + z
   end if
      
      !record crown-bottom-layer stat_leaf
      a1 = mass_leaf(no) / ( height(no)-bole(no) )             !leaf mass per crown disk (g/step)
      a2 = a1 * PN_f(p) * RM(p) * tmp_sensibility_air + &
           a1 * (TO_f(p)/365) * (RG_f(p)-RG_f_suck(p))         !leaf maintenance cost (g/day/step)
      a3 = (a1/FR_ratio(p)) * PN_r(p) * RM(p) * tmp_sensibility_soil + &
           (a1/FR_ratio(p)) * (TO_r(p)/365) * RG_r(p)          !root maintenance cost (g/day/step)
      do i = 1, 10
      stat_leaf_crownbottom(no,i)     = stat_leaf_crownbottom(no,i)      - a2 - a3
      end do
      stat_leaf_crowntop(no)          = stat_leaf_crowntop(no)           - a2 - a3
      stat_leaf_crownbottom_daily(no) = stat_leaf_crownbottom_daily (no) - a2 - a3
      
      !benefit of producing new leaf mass (g / g Leaf)
      stat_leaf(no) = stat_leaf(no) - (a2+a3) / max(0.01,a1)
      
End if
End DO

!**Carbon-leak-checker1
   a3 = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock) &
        + pool_litter + mass_combust
   
   a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = a3 + (0.90*a1 + 0.10*a2) * (Max_loc**2)
   else
      a3 = a3 + (0.10*a1 + 0.90*a2) * (Max_loc**2)
   end if
   
   if ( abs(mass_total-a3)>20.0 .and. Logging==1 ) then
       write (*,*) 'Error maintenance_respiration1: leaking mass (g) ->', abs(mass_total-a3), doy
   end if
   
!Herbaceous PFTs
DO gn=1, 2 !( gn1->C3, gn2->C4 )
   !reset mass respiration 
   mass_resp_grass  = 0.000
   
   !set pft number, p
   if (gn==1) then
      p = c3g_no
   else
      p = c4g_no
   endif
   
   !calculate daily mass_required for maintenance respiration (g/m2)
   resp_15C_air  = RM(p) * PN_f(p) * gmass_leaf(gn)
   resp_15C_soil = RM(p) * PN_r(p) * gmass_root(gn)
   mass_required = resp_15C_air*tmp_sensibility_air + resp_15C_soil*tmp_sensibility_soil
   
   !Source 1: mass_available
   x                  = min(mass_required, gmass_available(gn))
   mass_required      = mass_required       - x
   gmass_available(gn)= gmass_available(gn) - x
   mass_resp_grass    = x
   
   !Source 2: mass_stock
   if (mass_required > 0.001) then
    x               = min(mass_required, gmass_stock(gn) / RG_stock_out)
    mass_required   = mass_required   - x
    gmass_stock(gn) = gmass_stock(gn) - x * RG_stock_out
    mass_resp_grass = mass_resp_grass + x * RG_stock_out
   endif
    
   !Source 3: when resource absent
   litter_sum_grass     = 0.0
   if (mass_required > 0.001) then
    litter_sum_grass   = frac_organ_remove * ( gmass_leaf(gn) + gmass_root(gn) )
    gmass_leaf(gn)     = (1.0 - frac_organ_remove) * gmass_leaf(gn)
    gmass_root(gn)     = (1.0 - frac_organ_remove) * gmass_root(gn)
    lai_grass(gn)      = gmass_leaf(gn) * SLA(p)
   endif
   
   !carbon balance
                                  a1 = 0.100 * (Max_loc**2)
   if (gn==1 .and. type_grass==3) a1 = 0.900 * (Max_loc**2)
   if (gn==2 .and. type_grass==4) a1 = 0.900 * (Max_loc**2)
   
   mass_combust = mass_combust + mass_resp_grass  * a1
   npp(p)       = npp(p)       - mass_resp_grass  * a1
   pool_litter  = pool_litter  + litter_sum_grass * a1
   
END DO

!**Carbon-leak-checker2
   a3 = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock) &
        + pool_litter + mass_combust
   
   a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = a3 + (0.9*a1 + 0.1*a2) * (Max_loc**2)
   else
      a3 = a3 + (0.1*a1 + 0.9*a2) * (Max_loc**2)
   end if
   
   if ( abs(mass_total-a3)>20.0 .and. Logging==1 ) then
       write (*,*) 'Error maintenance_respiration2: leaking mass (g) ->', abs(mass_total-a3), doy
   end if
   
!Total carbon emission by maintenance respiration
   c_emission = c_emission + mass_combust * C_in_drymass
   
END SUBROUTINE maintenance_resp



!**************************************************************************************************
! Turnover (daily computation)
!**************************************************************************************************
SUBROUTINE turnover &
         (tree_exist, pft, mass_available, mass_leaf, mass_root, dbh_heartwood, dbh_sapwood, la, &
          type_grass, lai_grass, gmass_leaf, gmass_root, gmass_available, &
          pool_litter, counter, pheno_clo, pheno_cls, c3g_no, c4g_no )

USE data_structure
implicit none

!Define arguments
real                     ,intent(INOUT)::pool_litter !litter pool in a stand (g / stand )
integer                  ,intent(IN)   ::counter     !simulation day (1-Simulation_year*365)
integer,dimension(PFT_no),intent(IN)   ::pheno_clo   !counter number of last leaf onset
integer,dimension(PFT_no),intent(IN)   ::pheno_cls   !counter number of last leaf shedding
integer                  ,intent(IN)   ::c3g_no       !PFT number of C3 grass
integer                  ,intent(IN)   ::c4g_no       !PFT number of C4 grass

logical,dimension(Max_no),intent(IN)   ::tree_exist     !tree exist or not (.TRUE. or .FALSE.)
integer,dimension(Max_no),intent(IN)   ::pft            !Plant Functional Type (1-PFT_no)
real   ,dimension(Max_no),intent(INOUT)::mass_available !biomass available   (g)
real   ,dimension(Max_no),intent(INOUT)::mass_leaf      !biomass of foliage  (g)
real   ,dimension(Max_no),intent(INOUT)::mass_root      !biomass of root (g)
real   ,dimension(Max_no),intent(INOUT)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real   ,dimension(Max_no),intent(INOUT)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real   ,dimension(Max_no),intent(INOUT)::la             !leaf area (m2)

integer               ,intent(IN)   ::type_grass !priority grass type (3:C3, 4:C4)
real   ,dimension(2),intent(INOUT)::lai_grass  !leaf area index of grass (m2/m2)
real   ,dimension(2),intent(INOUT)::gmass_leaf !grass foliage biomass in a unit area (g/m2)
real   ,dimension(2),intent(INOUT)::gmass_root !grass root  biomass in a unit area   (g/m2)
real   ,dimension(2),intent(INOUT)::gmass_available !c3/c4 grass available biomass (g/m2)

!Define local variables
real    t_rate      !daily turnover rate
real    t_mass      !daily turnover mass
real    mass_total  !for carbon leak chaker (g DM/stand)
integer p           !for PFT number
integer no, i, gn   !for loop counter
real    x, y, z     !for general usage
real    a1, a2      !for general usage

!carbon leak checker1
mass_total = sum(mass_leaf) + sum(mass_root) + pool_litter

!Woody PFTs
DO no=1, Max_no 
if (tree_exist(no)) then
   !set pft number
   p = pft(no)
   
   !foliage
   if ( Phenology_type(p)==0 .or. counter-pheno_clo(p)>365 ) then
      !evergreen
      t_rate = TO_f(p) / 365.000
   else
      !summer or rain green
      x      = max( 1 , 365-(pheno_clo(p)-pheno_cls(p)) ) !number of growth day
      y      = max(0.0, TO_f(p) - 365/x)                  !corrected turnover rate
      t_rate = y / 365.000
   end if
   t_mass             = mass_leaf(no)      * t_rate !leaf mass to be drop (g DM)
   mass_leaf(no)      = mass_leaf(no)      - t_mass 
   mass_available(no) = mass_available(no) + t_mass * RG_f_suck(p)
   pool_litter        = pool_litter        + t_mass * (1.0 - RG_f_suck(p))
   la(no)             = mass_leaf(no) * SLA(p)
   
   !root
   t_mass        = mass_root(no) * ( TO_r(p) / 365.0000 )  !root mass to be drop (g DM)
   mass_root(no) = mass_root(no) - t_mass
   pool_litter   = pool_litter   + t_mass
   
   !sapwood
   if ( Life_type(p)==1 ) then
      x = ALM5(p)       * (dbh_sapwood(no) + dbh_heartwood(no)) !new sapwood diameter   (m)
      y = (1.0-ALM5(p)) * (dbh_sapwood(no) + dbh_heartwood(no)) !new heartwood diameter (m)
      
      dbh_sapwood(no)   = x
      dbh_heartwood(no) = y
      
   else
      x                 = dbh_sapwood(no)
      y                 = dbh_heartwood(no)
      z                 = TO_s(pft(no)) / 365.0000
      t_rate            = sqrt( y*y + z*( x*x + 2*x*y ) ) - y
      dbh_sapwood(no)   = dbh_sapwood(no) - t_rate
      dbh_heartwood(no) = dbh_heartwood(no) + t_rate
   endif
   
end if
END DO

!carbon leak checker1
   x = sum(mass_leaf) + sum(mass_root) + pool_litter
   if ( abs(mass_total-x)>200.0 .and. Logging==1 ) then
      write (*,*) 'Error turnover1: leaking mass (g) ->', mass_total-x
   end if

!carbon leak checker2
   if (type_grass==3) then
      a1 = 0.9*(gmass_leaf(1)+gmass_root(1)) + 0.1*(gmass_leaf(2)+gmass_root(2))
   else
      a1 = 0.1*(gmass_leaf(1)+gmass_root(1)) + 0.9*(gmass_leaf(2)+gmass_root(2))
   endif
   mass_total = a1*Max_loc*Max_loc + pool_litter

!Herbaceous PFTs
DO gn=1, 2
   !set pft number, p
   p = c4g_no; if (gn==1) p = c3g_no
   
   !a1: proportion of coverage
                                  a1=0.1 * Max_loc * Max_loc
   if (gn==1 .and. type_grass==3) a1=0.9 * Max_loc * Max_loc
   if (gn==2 .and. type_grass==4) a1=0.9 * Max_loc * Max_loc
   
   !foliage turnover
   if ( counter-pheno_clo(p)>365 ) then
     !evergreen
     t_rate = TO_f(p)/365.0000
   else
     !summer or rain green
     x      = max( 1 , 365-(pheno_clo(p)-pheno_cls(p)) ) !number of growth day
     y      = max(0.0, TO_f(p) - 365/x)                  !corrected turnover rate
     t_rate = y/365.000
   end if
   
   t_mass              = gmass_leaf(gn) * t_rate     !leaf mass to drop (g/m2)
   gmass_leaf(gn)      = gmass_leaf(gn)      - t_mass
   gmass_available(gn) = gmass_available(gn) + t_mass * RG_f_suck(p)
   pool_litter         = pool_litter         + t_mass *(1.0-RG_f_suck(p))*a1
   lai_grass(gn)       = gmass_leaf(gn) * SLA(p)
   
   !root turnover
   t_rate         = TO_r(p) / 365.0000
   pool_litter    = pool_litter    + gmass_root(gn)*t_rate*a1
   gmass_root(gn) = gmass_root(gn) - gmass_root(gn)*t_rate
END DO

!carbon leak checker2
   if (type_grass==3) then
      a1 = 0.9*(gmass_leaf(1) + gmass_root(1)) + 0.1*(gmass_leaf(2) + gmass_root(2))
   else
      a1 = 0.1*(gmass_leaf(1) + gmass_root(1)) + 0.9*(gmass_leaf(2) + gmass_root(2))
   endif
   a2 = mass_total - (a1*Max_loc*Max_loc + pool_litter)
   
   if ( abs(a2)>200.0 .and. Logging==1 ) then
      write (*,*) 'Error turnover2: leaking mass (g) ->', a2
   end if

END SUBROUTINE turnover



!**************************************************************************************************
! growth woody PFTs (daily computation)
!**************************************************************************************************
SUBROUTINE growth_wood &
   (doy, phenology, pheno_clo, stat_leaf_RunningRecord, counter, &
   pft, height, bole, dbh_heartwood, dbh_sapwood, crown_area, crown_diameter, tree_exist, &
   la, mort_regu2, mass_leaf, mass_root, mass_available, mass_stock, stat_leaf_crownbottom_daily, &
   pool_litter, c_emission, npp, mort_regu1 )

USE data_structure
implicit none

!_____________ Define arguments
integer                        ,intent(IN)::doy        !day of the year (1-365)

logical,dimension(PFT_no)      ,intent(IN)::phenology  !phenological status
integer,dimension(PFT_no)      ,intent(IN)::pheno_clo  !counter number of last leaf onset
real   ,dimension(Max_no,1:7),intent(IN)::stat_leaf_RunningRecord
integer                        ,intent(IN)::counter    !simulation day (1-Simulation_year*365)

integer,dimension(Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(Max_no),intent(IN)::height         !Tree height from 1.3m above ground   (STEP)
integer,dimension(Max_no),intent(IN)::bole           !Bole length (STEP)
real   ,dimension(Max_no),intent(IN)::dbh_heartwood  !Hertwood diameter at 1.3m (m)
real   ,dimension(Max_no),intent(IN)::dbh_sapwood    !Sapwood diameter at 1.3m (m)
real   ,dimension(Max_no),intent(IN)::crown_area     !Crown area (m2)
real   ,dimension(Max_no),intent(IN)::crown_diameter !crown diameter (m)

logical,dimension(Max_no),intent(IN)::tree_exist !tree exist or not (.TRUE. or .FALSE.) 

real,dimension(Max_no),intent(INOUT)::la              !Leaf area         (m2)
real,dimension(Max_no),intent(INOUT)::mort_regu2      !average leaf area of last year (m2/day)
real,dimension(Max_no),intent(INOUT)::mass_leaf       !biomass of foliage (g)
real,dimension(Max_no),intent(INOUT)::mass_root       !biomass of root    (g)
real,dimension(Max_no),intent(INOUT)::mass_available  !biomass available  (g)
real,dimension(Max_no),intent(INOUT)::mass_stock      !biomass for stock  (g)
real,dimension(Max_no),intent(INOUT)::stat_leaf_crownbottom_daily !daily npp on the bottom of crown layer

real                    ,intent(INOUT)::pool_litter !litter pool in a stand (g DM / stand)
real                    ,intent(INOUT)::c_emission  !carbon emission (g C / month / stand)
real,dimension(PFT_no)  ,intent(INOUT)::npp         
real,dimension(Max_no),intent(INOUT)::mort_regu1  !annual NPP (g dm)

!_____________ Define local variables
integer p                  !for PFT number
integer no                 !for loop counter
real    a1, a2, a3, a4, a5 !for temporal usage
real    mass_combust       !biomass combust (g DM/day/stand)
real    mass_total         !total biomass before procedure --for checker-- (g DM/stand)

!_____________ Initialize
mass_combust = 0.000
mass_total   = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock)+pool_litter

!_____________ Each individual of woody PFT procedure
DO no=1, Max_no 
IF ( .not. tree_exist(no)     ) cycle
IF ( .not. phenology(pft(no)) ) cycle
    
    !SET PFT NUMBER
    p = pft(no)
    
    !ROOT MASS SUPPLEMENT
    !a1: required root mass (g)
    a1 = max( 0.00000, mass_leaf(no)/FR_ratio(p) - mass_root(no) )
    !a2: maximum root mass increament allowded by available resource (g)
    a2 = mass_available(no) / RG_r(p)
    !a3 = actual increament of Root mass with considering resource_available
    a3 = min( a1, a2 )
    !root mass supplement precedure
    mass_root(no)      = mass_root(no)      +  a3                
    mass_available(no) = mass_available(no) -  a3 * RG_r(p)      
    mass_combust       = mass_combust       + (a3 * RG_r(p) - a3)
    npp(p)             = npp(p)             - (a3 * RG_r(p) - a3)
    mort_regu1(no)     = mort_regu1(no)     - (a3 * RG_r(p) - a3)
    
    !STOCK RESOURCE SUPPLEMENT
    if (counter > pheno_clo(p)+30) then !after more than 30 days from foliation
       !a1 = resource requirement for stock (g)
       a1 = max( 0.00000, mass_leaf(no) - mass_stock(no) )
       !a2 = maximum increment of mass_stock (g)
       a2 = mass_available(no) / RG_stock_in
       !a3:actual increment of mass_stock (g/m2)
       a3 = min( a1, a2 )
       !stock resource supplement precedure
       mass_stock(no)     = mass_stock(no)     +  a3                    
       mass_available(no) = mass_available(no) -  a3 * RG_stock_in      
       mass_combust       = mass_combust       + (a3 * RG_stock_in - a3)
       npp(p)             = npp(p)             - (a3 * RG_stock_in - a3)
       mort_regu1(no)     = mort_regu1(no)     - (a3 * RG_stock_in - a3)
    end if
    
    !FOLIAGE MASS SUPPLEMENT
    !a1: deficit of foliage mass until fullfill existing crown size (g)
    if ( Life_type(p)==1 ) then
       a1 = crown_area(no)
       a1 = a1 * LA_max(p) / SLA(p)
       a1 = max(0.0, a1-mass_leaf(no) )
    else
       a1 = crown_area(no) + PI * crown_diameter(no) * real(height(no)-bole(no)) * STEP
       a1 = a1 * LA_max(p) / SLA(p)
       a1 = max(0.0, a1-mass_leaf(no) )
    endif
    
    !a2: deficit of foliage mass until fullfill existing Sapwood (g)
    a2 = ALM1(p) * (1/SLA(p)) &
        * (PI/4) * (dbh_sapwood(no)**2 + 2*dbh_sapwood(no)*dbh_heartwood(no))
    a2 = max(0.0, a2-mass_leaf(no) )
    
    !a3: maximum foliage mass with current resource available (g)
    a3 = mass_available(no) / RG_f(p)
    
    !a4: maximum foliage mass that can increase within a day (g)
    a4 = 0.1 * crown_area(no) / SLA(p)
    
    !a5: acutual increament of foliage mass (g)
    a5 = min (a1, a2, a3)
    
    if ( Life_type(p)==1 ) then
       a5 = min (a1, a3, a4)
       if ( stat_leaf_crownbottom_daily(no)<0.0 ) a5=0.0
    endif
    
    !reset daily leaf area stat on bottom of crown layer
    stat_leaf_crownbottom_daily(no) = 0.0
    
    !a5: actual increament (g)
    if ( a5 > 0.0) then
        mass_leaf(no)      = mass_leaf(no)       +  a5                
        mass_available(no) = mass_available(no)  -  a5 * RG_f(p)      
        mass_combust       = mass_combust        +  a5 * RG_f(p) - a5 
        npp(p)             = npp(p)              - (a5 * RG_f(p) - a5)
        mort_regu1(no)     = mort_regu1(no)      - (a5 * RG_f(p) - a5)
        la(no)             = mass_leaf(no) * SLA(p)
    end if
    
    !update mort_regu2 (annumal mean of leaf area)
    mort_regu2(no) = mort_regu2(no) + la(no)/365
    
END DO

!_____________ carbon leak checker
!a1 = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock)+pool_litter+mass_combust
!a2 = abs(mass_total - a1)
!if ( a2>100.0 .and. Logging==1 ) then
!   write (*,*) 'Error @ growth_wood: leaking mass (g) ->', a2, 'doy ->',doy
!end if

!_____________ carbon emission
c_emission = c_emission + mass_combust * C_in_drymass

End subroutine growth_wood



!**************************************************************************************************
! growth_grass (daily computation)
!**************************************************************************************************
SUBROUTINE growth_grass &
      (doy, counter, pool_litter, c_emission, npp, phenology, pheno_clo, &
      type_grass, lai_grass, gmass_leaf, gmass_root, gmass_available, &
      gmass_stock, lai_opt_grass_RunningRecord, c3g_no, c4g_no)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)   ::doy         !day of the year (1~365)
integer,intent(IN)   ::counter     !simulation day (1~Simulation_year*365)
real   ,intent(INOUT)::pool_litter !litter pool in a stand (g / stand)
real   ,intent(INOUT)::c_emission  !monthly carbon emission (C g / month / stand)

integer,intent(IN)   :: c3g_no      !PFT number of C3 grass
integer,intent(IN)   :: c4g_no      !PFT number of C4 grass

real   ,dimension(PFT_no),intent(INOUT)::npp
logical,dimension(PFT_no),intent(IN)   ::phenology !phenological status
integer,dimension(PFT_no),intent(IN)   ::pheno_clo !counter number of last leaf onset
                                                   !running record of lai_opt_grass (m2/m2)

integer                  ,intent(IN)   ::type_grass      !grass type (3:C3, 4:C4)
real,dimension(2)      ,intent(INOUT)::lai_grass       !leaf area index of grass (m2/m2)
real,dimension(2)      ,intent(INOUT)::gmass_leaf      !grass foliage   biomass  (g/m2)
real,dimension(2)      ,intent(INOUT)::gmass_root      !grass root      biomass  (g/m2)
real,dimension(2)      ,intent(INOUT)::gmass_available !grass available biomass  (g/m2)
real,dimension(2)      ,intent(INOUT)::gmass_stock     !grass stock     biomass  (g/m2)
real,dimension(2,20) ,intent(IN)   ::lai_opt_grass_RunningRecord

!Define local variables
real    lai_opt      !optimum LAI of grass layer (m2/m2)
real    mass_combust !biomass combust (g dm/day/stand)
real    mass_total   !total biomass before procedure --for checker-- (g dm/m2)
real    area_cover   !coverage area (m2)
integer p            !PFT number
integer i, gn        !for loop counter
real    a1, a2, a3   !for temporal usage

!carbon leak checker (before procedure)
   a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      mass_total = (0.9*a1 + 0.1*a2)*Max_loc*Max_loc + pool_litter
   else
      mass_total = (0.1*a1 + 0.9*a2)*Max_loc*Max_loc + pool_litter
   end if
   
!initialize local variables
   mass_combust = 0.000
   
!each grass type procedure
DO gn = 1, 2
    
    !set pft number: p
    p=c4g_no; if (gn==1) p=c3g_no
    
IF ( phenology(p) ) then
    
    !set coverage area within the stand: area_cover (m2)
                                    area_cover = 0.1 * Max_loc * Max_loc
    if ( gn==1 .and. type_grass==3) area_cover = 0.9 * Max_loc * Max_loc
    if ( gn==2 .and. type_grass==4) area_cover = 0.9 * Max_loc * Max_loc
    
    !set lai_opt, running mean of optimum leaf area index
    lai_opt = sum(lai_opt_grass_RunningRecord(gn,1:7)) / 7.0
    
    !excess foliage senescence procedure (optional)
    ! if (lai_grass(gn) > lai_opt) then
    !    !a1: excess gmass_leaf (g drymatter/m2)
    !    a1             = (lai_grass(gn)-lai_opt) / SLA(p)
    !    !a2: gmass_leaf that senescence (g drymatter/m2)
    !    a2             = a1 * 0.020
    !    gmass_leaf(gn) = gmass_leaf(gn) - a2
    !    pool_litter    = pool_litter    + a2 * area_cover
    !    lai_grass(gn)  = gmass_leaf(gn) * SLA(p)
    ! end if
    
    !root mass supplement
       !a1: required root mass (g dm / m2)
       a1 = max(0.0, gmass_leaf(gn)/FR_ratio(p) - gmass_root(gn))
       !a2: maximum root mass increament (g dm / m2)
       a2 = gmass_available(gn) / RG_r(p)
       !a3: actual increamet of root mass (g dm / m2)
       a3 = min(a1, a2)
       !root mass supplement procedure
       gmass_root(gn)      = gmass_root(gn)      + a3          
       gmass_available(gn) = gmass_available(gn) - a3 * RG_r(p)
       mass_combust        = mass_combust + (a3 * RG_r(p) - a3) * area_cover
       npp(p)              = npp(p)       - (a3 * RG_r(p) - a3) * area_cover
    
    !stock resource supplement
    if ( counter > pheno_clo(p)+30 ) then !after more than 30 days from foliation
       !a1:resource requirement for stock (g/m2)
        a1 = max( 0.0, gmass_leaf(gn) - gmass_stock(gn) )
       !a2:maximum increment of mass_stock with resource available (g/m2)
        a2 = gmass_available(gn) / RG_stock_in
       !a3:actual increment of mass_stock (g/m2)
        a3 = min( a1, a2 )
       !stock mass increment precedure
       gmass_stock(gn)    = gmass_stock(gn)     +  a3               
       gmass_available(gn)= gmass_available(gn) -  a3 * RG_stock_in 
       mass_combust       = mass_combust + (a3 * RG_stock_in - a3) * area_cover
       npp(p)             = npp(p)       - (a3 * RG_stock_in - a3) * area_cover
    endif
    
    !foliage growth
       !a1: maximum foliage mass requirement (g/m2)
        a1 = (lai_opt - lai_grass(gn)) / SLA(p)
        a1 = max(a1, 0.0)
       !a2: maximum increament of folige mass that available resource permits (g/m2)
        a2 = gmass_available(gn) / RG_f(p)
       !a3: actual increament of foliage mass (g/m2)
        a3 = min(a1, a2)
       !foliage mass increament procedure
        gmass_leaf(gn)      = gmass_leaf(gn)      +  a3          
        gmass_available(gn) = gmass_available(gn) -  a3 * RG_f(p)
        mass_combust        = mass_combust   + (a3 * RG_f(p) - a3) * area_cover
        npp(p)              = npp(p)         - (a3 * RG_f(p) - a3) * area_cover
        lai_grass(gn)       = gmass_leaf(gn) * SLA(p)
    
    !reproduction: excess resource will be added to litter
    if ( counter > pheno_clo(p)+30    ) then !after more than 30 days from foliation
    if ( gmass_stock(gn) > 100.0 ) then !minimum stock resource for reproduction (g dm /m2)
       pool_litter         = pool_litter + gmass_available(gn)*area_cover
       gmass_available(gn) = 0.0000
    endif
    endif
    
END IF
END DO

!carbon emission
   c_emission = c_emission + mass_combust * C_in_drymass

!carbon leak checker (after procedure)
   a1 = gmass_leaf(1)+gmass_root(1)+gmass_available(1)+gmass_stock(1)
   a2 = gmass_leaf(2)+gmass_root(2)+gmass_available(2)+gmass_stock(2)
   
   if (type_grass==3) then
      a3 = (0.9*a1 + 0.1*a2)*Max_loc*Max_loc + pool_litter + mass_combust
   else
      a3 = (0.1*a1 + 0.9*a2)*Max_loc*Max_loc + pool_litter + mass_combust
   end if
   a3 = abs(a3-mass_total)
   
   if ( a3>1.0 .and. Logging==1 ) then
      write (*,*) 'Error @ growth grass: leaking mass (g/m2) ->', a3, '   doy ->', doy
   end if
   
End subroutine growth_grass



!**************************************************************************************************
! Crown depth adjustment
!**************************************************************************************************
SUBROUTINE crown_adjust &
   (height, pft, tree_exist, bole, la, mass_leaf, &
   pool_litter, stat_leaf_crownbottom, stat_leaf_crowntop)

USE data_structure
implicit none

!Define parameter (minimum crown depth in STEP)
integer,parameter::Crown_depth_min = 10

!Define Arguments
integer,dimension(Max_no)    ,intent(IN)::height     !tree height from 1.3m above ground (STEP)
integer,dimension(Max_no)    ,intent(IN)::pft        !Plant Functional Type (1-PFT_no)
logical,dimension(Max_no)    ,intent(IN)::tree_exist !tree exist or not (.TRUE. or .FALSE.)

integer,dimension(Max_no)    ,intent(INOUT)::bole        !bole length (STEP)
real   ,dimension(Max_no)    ,intent(INOUT)::la          !leaf area (m2)
real   ,dimension(Max_no)    ,intent(INOUT)::mass_leaf   !biomass of foliage (g)
real                           ,intent(INOUT)::pool_litter !litter pool in a stand (g/stand)

real   ,dimension(Max_no, 10),intent(INOUT)::stat_leaf_crownbottom !annual NPP of foliage bottom
real   ,dimension(Max_no    ),intent(INOUT)::stat_leaf_crowntop    !annual NPP of foliage top

!Define local variables
real    a1,a2,x,hgt !for general usage
integer no,i        !for general usage
real    mass_total  !mass before procedure --for checker-- (g)
real    la_step     !Leaf area for each foliage layer (m2/STEP)
integer count       !number of crown layers to purge

!Save mass_total for carbon-leak-checker
mass_total = sum(mass_leaf) + pool_litter

!Bole-lengh adjustment
DO no=1, Max_no 
if ( .not. tree_exist(no) )                   cycle
if ( height(no)-bole(no) <= Crown_depth_min ) cycle
   
   !determine number of crown disks that will be purged
   count = 0
   Select case ( Life_type(pft(no)) )
   Case (1) !Tropical evergren trees
      hgt = height(no) * STEP + 1.3 !tree height (m)
      
      if ( hgt > 34.0 ) then
         x = 0.18
      else
         x = 0.0002 * hgt * hgt - 0.0182 * hgt + 0.6
      endif
      x = x * hgt                    !new crown depth (m)
      i = int( height(no) - x/STEP ) !new bole height (STEP)
      count = max(0, i - bole(no))
      
   Case default !default standard
       do i = 1, min(10, height(no)-bole(no)-1)
          if ( sum(stat_leaf_crownbottom(no,1:i))/ i  < &
               stat_leaf_crowntop(no) * ALM4(pft(no)) ) count = i
       end do
   
   End select
   count = min(count, height(no)-bole(no)-Crown_depth_min)
   
    !when maximum_crown_depth was assumed
    ! max_depth = 30
    ! if ( height(no)-bole(no)-max_depth > 0 ) then
    !    count = max(count, height(no)-bole(no)-max_depth )
    ! end if
    
    !crown adjust procedures
    if (count > 0) then
    !  la_step     = la(no) / real( height(no)-bole(no) )
    !  la(no)      = la(no)      - la_step * count
    !  mass_leaf(no)  = mass_leaf(no)  - la_step * count / SLA(pft(no))
    !  pool_litter = pool_litter + la_step * count / SLA(pft(no))
       bole(no)    = bole(no)    + count
    endif
   
END DO

!reset bole-height-adjust control variables
stat_leaf_crownbottom(:,:) = 0.0
stat_leaf_crowntop(:)      = 0.0

!carbon leak checker
a1 = sum(mass_leaf) + pool_litter !total carbon
a2 = abs(a1 - mass_total)      !difference in total carbon
if ( a2>5.0 .and. Logging==1 ) then
     write (*,*) 'Error @ crown_adjust: leaking mass (g) ->', a2
end if

END SUBROUTINE crown_adjust



!**************************************************************************************************
! Monthly growth and reproduction of woody PFT (each individual_tree procedure)
!**************************************************************************************************
SUBROUTINE growth_stem &
    (counter, pheno_clo, phenology, &
    pool_litter, c_emission, &
    height_limit, radius_limit, pft, tree_exist, &
    height, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, &
    mass_leaf, mass_trunk, mass_root, mass_stock, mass_available,&
    gpp_month_ind, npp, mort_regu1 , mort_regu4)

USE data_structure
implicit none

!_____________ Define local parameter
!fraction of available resource allocates to reproduction 
   real,parameter::Frac_mass_reproduction = 0.10
   
!delay of stem growth and reproduction process after foliation (day)
   integer,parameter::Delay_from_foliation = 21
   
!Threshold biomass that initiate reproduction (DM g)
   real,parameter::Thres_mass_reproduct = 10000.0
   
!_____________ Define arguments
integer                  ,intent(IN)::counter    !simulation day (1-Simulation_year*365)
integer,dimension(PFT_no),intent(IN)::pheno_clo  !counter number of last leaf onset
logical,dimension(PFT_no),intent(IN)::phenology  !phenological status (T:foliation, F:dormance)

real              ,intent(INOUT)::pool_litter  !litter pool in a stand (g / stand)
real              ,intent(INOUT)::c_emission   !monthly carbon emission (C g / stand)

integer,dimension(Max_no),intent(IN)   ::height_limit   !max tree height in this point (step)
real   ,dimension(Max_no),intent(IN)   ::radius_limit   !max canopy radius in this point (m)
integer,dimension(Max_no),intent(IN)   ::pft            !Plant Functional Type (1-PFT_no)
logical,dimension(Max_no),intent(IN)   ::tree_exist     !tree exist or not (.TRUE. or .FALSE.)
integer,dimension(Max_no),intent(INOUT)::height         !tree over 1.3m above ground (STEP)
real   ,dimension(Max_no),intent(INOUT)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real   ,dimension(Max_no),intent(INOUT)::dbh_sapwood    !sapwood   diameter at 1.3m (m)
real   ,dimension(Max_no),intent(INOUT)::crown_diameter !crown diameter (m)
real   ,dimension(Max_no),intent(INOUT)::crown_area     !crown area (m2)

real   ,dimension(Max_no),intent(IN)   ::mass_leaf      !biomass of foliage (g)
real   ,dimension(Max_no),intent(INOUT)::mass_trunk     !biomass of stem    (g)
real   ,dimension(Max_no),intent(IN)   ::mass_root      !biomass of root    (g)
real   ,dimension(Max_no),intent(IN)   ::mass_stock     !biomass for stock  (g)
real   ,dimension(Max_no),intent(INOUT)::mass_available !biomass available  (g)

real,dimension(Max_no),intent(IN)   ::gpp_month_ind !GPP for each individual (gDM/ind/month)
real,dimension(PFT_no),intent(INOUT)::npp           !NPP for each PFT (gDM/stand/yr)
real,dimension(Max_no),intent(INOUT)::mort_regu1    !sum of NPP within the previous year (gDM)
real,dimension(Max_no),intent(INOUT)::mort_regu4    !stem diameter increament in last year (m)

!_____________ Define local variables
real    ca_limit       !crown area limit
real    diameter_new   !new diameter (m)
real    mass_trunk_tmp !stem biomass for temporal usage (dm g)
integer height_new     !new tree height (STEP)
real    dn, md, up     !for convergence loop
real    mass_combust   !biomass combust (g DM/day/stand)
real    mass_total     !total biomass before procedure --for checker-- (g DM/stand)
real    x, y           !for general usage
integer no, p, count   !
logical flag           !

!_____________ For carbon leak checker
mass_combust = 0.000
mass_total   = sum(mass_trunk) + sum(mass_available) + pool_litter + mass_combust

!_____________ Growth process for each tree
DO no=1, Max_no 

p = pft(no)

IF ( tree_exist(no)                              ) then !when tree exist
IF ( phenology(p)                                ) then !when on foliation phase
IF ( counter > pheno_clo(p)+Delay_from_foliation ) then !after specific day from foliation
IF ( mass_available(no) > 0.0                    ) then !when mass available exist
   
   !Reproduction
   x = mass_leaf(no)+mass_trunk(no)+mass_root(no)+mass_stock(no)+mass_available(no)
   If (x > Thres_mass_reproduct) then
      pool_litter        = pool_litter + mass_available(no) *      Frac_mass_reproduction  
      mass_available(no) = mass_available(no)               * (1.0-Frac_mass_reproduction )
   Endif
   
   !Adjust mass available
   If (Life_type(p)==1) then
      x = ( (dbh_heartwood(no)+dbh_sapwood(no)) / DBH_limit(p) )**2
      x = min(1.0, x)      !diminishing factor due to size limitation
      y = mass_available(no) / max(0.01, gpp_month_ind(no))
      y = min(1.0, y)
      
      mass_combust       = mass_combust       + mass_available(no) * x * y
      npp(p)             = npp(p)             - mass_available(no) * x * y
      mort_regu1(no)     = mort_regu1(no)     - mass_available(no) * x * y !Annual NPP (gDM)
      mass_available(no) = mass_available(no) - mass_available(no) * x * y
   End if
   
   !Allocate all remaining mass_available to sapwood production
   x                  = mass_available(no) / RG_s(p)
   mass_trunk(no)     = mass_trunk(no) +  x
   mass_combust       = mass_combust   + (mass_available(no) - x)
   npp(p)             = npp(p)         - (mass_available(no) - x)
   mort_regu1(no)     = mort_regu1(no) - (mass_available(no) - x)
   mass_available(no) = 0.0
   
   !determine DBH increament
   up = 0.5000000; dn = 0.0000000 !lengh to examine convergence
   count = 0
   Do
      count = count + 1
      !current sapwood diameter increament (m)
      md             = (up+dn) / 2.0
      !current diameter (m)
      diameter_new   = dbh_heartwood(no) + dbh_sapwood(no) + md
      !current height (STEP)
      height_new     = max(height(no), min(height_limit(no), h(diameter_new, p) ))
      !current trunk mass (g)
      mass_trunk_tmp = stem_weight(diameter_new, height_new, p)
      !x: difference in stem mass (g)
      x = mass_trunk_tmp - mass_trunk(no)
      !examine convergence
      if    (x < 0.0000000) then ; dn = md
      else                       ; up = md
      end if                               
      if (up-dn < 0.0001) exit !error should be less than 1/10 (mm/month)
      if (count > 1000)   exit !avoid infinite loop
   End do
   
   !adjust trunc diameter and height
   height(no)      = height_new
   dbh_sapwood(no) = dbh_sapwood(no) + md 
   mort_regu4(no)  = mort_regu4(no)  + md !annual increament of DBH
   
   !adjust crown diameter
   ca_limit           = max(crown_area(no), PI * radius_limit(no) * radius_limit(no) )
   crown_area(no)     = max(0.001         , min(ca_limit, ca(diameter_new, p))       )
   crown_diameter(no) = 2 * sqrt(crown_area(no)/PI)
   
END IF
END IF
END IF
END IF
END DO

!_____________ Carbon leak checker
!x = sum(mass_trunk) + sum(mass_available) + pool_litter + mass_combust
!y = abs(mass_total - x)
!if ( y>100.0 .and. Logging==1 ) then
!   write (*,*) 'Error @ growth_stem: leaking mass (DM g) ->', y
!end if

!_____________ Carbon emission
c_emission = c_emission + mass_combust * C_in_drymass

END SUBROUTINE growth_stem



!**************************************************************************************************
! Calculation of photosynthesis parameters (for each PFT)
!**************************************************************************************************
SUBROUTINE decomposition &
         (pool_litter, pool_som_int, pool_som_slow, c_emission, &
         aet, tmp_soil_month_ave, pool_w1, pool_w2, W_sat)

USE data_structure
implicit none

!*** Define common arguments ***

!soil carbon
real,intent(INOUT)::pool_litter   !litter pool in a stand (g dm / stand)
real,intent(INOUT)::pool_som_int  !intermediated SOM pool (g dm / stand)
real,intent(INOUT)::pool_som_slow !slow SOM pool          (g dm / stand)

!carbon flux recorder
real,intent(INOUT)::c_emission    !monthly carbon emission (g C / month / stand)

!climate and soilmoisture
real,intent(IN)   ::aet                !sum of actual evapotranspiration of last year (mm/year)
real,intent(IN)   ::tmp_soil_month_ave !30days running average of tmp_soil (Celcius)
real,intent(IN)   ::pool_w1            !soil water content of soil layer 1 (mm)
real,intent(IN)   ::pool_w2            !soil water content of soil layer 2 (mm)
real,intent(IN)   ::W_sat              !saturate point   (m3/m3, 0.0 -> 1.0)

!*** Define local variable ***
real effect_tmp      !temperature coefficient for decomposition
real effect_mois     !moisture    coefficient for decomposition

real d_rate_litter   !decomposition rate of litter   (month-1)
real d_rate_som_int  !decomposition rate of som_int  (month-1)
real d_rate_som_slow !decomposition rate of som_slow (month-1)

real mass_decomp     !decomposed organic matter (g dm month-1 stand-1)

real x               !for general usage

real c_before        !total carbon before procedure   -for checker- (g C/stand)
real c_after         !total carbon before procedure   -for checker- (g C/stand)

!Checker
   c_before = (pool_litter+pool_som_int+pool_som_slow)*C_in_drymass + c_emission
   
!Calculatet decomposition coefficients
  !effect_tmp     = exp( 0.230259 * (tmp_soil_month_ave-20.0) ) !Q10 method
   effect_tmp     = exp( 308.56 * ( 1.0/66.02 - 1.0/( max(-45.0, tmp_soil_month_ave)+46.02) ) )
   effect_mois    = 0.25 + 0.75 * (pool_w1 / W_sat / Depth1)
   
!Calculate decomposition rates
!   d_rate_litter   = max(0.0 , 0.000597*aet-0.0131369 )   !Meentemeyer(1978)
   d_rate_litter   = 10**(0.0014175*aet-1.4553)            !Foley(1995)
   d_rate_litter   = d_rate_litter / 12.0
   d_rate_som_int  = (effect_tmp * effect_mois) * (TO_fast  / 12.0)
   d_rate_som_slow = (effect_tmp * effect_mois) * (TO_slow / 12.0)
   
   d_rate_litter   = min(1.0, d_rate_litter  )
   d_rate_som_int  = min(1.0, d_rate_som_int )
   d_rate_som_slow = min(1.0, d_rate_som_slow)

!decomposition procedure1, litter
   mass_decomp     = pool_litter * d_rate_litter
   !write (*,*) '1 ->', pool_litter, d_rate_litter, mass_decomp
   pool_litter     = pool_litter   - mass_decomp
   pool_som_int    = pool_som_int  + mass_decomp * (1.00-F_air) * F_inter
   pool_som_slow   = pool_som_slow + mass_decomp * (1.00-F_air) * (1.00 - F_inter)
   c_emission      = c_emission    + mass_decomp * F_air * C_in_drymass
   
!decomposition procedure2, s.o.m. with intermediated composition rate
   mass_decomp     = pool_som_int * d_rate_som_int
   pool_som_int    = pool_som_int - mass_decomp
   c_emission      = c_emission   + mass_decomp * C_in_drymass
   
!decomposition procedure3, s.o.m. with slow composition rate
   mass_decomp     = pool_som_slow * d_rate_som_slow
   pool_som_slow   = pool_som_slow - mass_decomp
   c_emission      = c_emission    + mass_decomp * C_in_drymass
   
!carbon leak checker
   c_after = (pool_litter+pool_som_int+pool_som_slow)*C_in_drymass + c_emission
   x = c_before - c_after
   
   if ( abs(x)>10.0 .and. Logging==1 ) then
      write (*,*) 'Error @ decomposiotion: leaking mass (g) ->', x
   end if
   

END SUBROUTINE decomposition