!**************************************************************************************************
! CO2 concentration
! Note: this subroutine will be removed when GCM was connected
!**************************************************************************************************
SUBROUTINE co2_density (co2atm, doy, LAT)

USE data_structure
implicit none

!_____________ Define augments
real   ,intent(OUT)::co2atm !background CO2 concentration (ppmv)
integer,intent(IN) ::doy    !loop for day of year (1-365)
real   ,intent(IN) ::LAT    !latitude  (degree)

!_____________ Define local variables
real x                                      !for temporal usage
real a0, a1, a2, a3, a4, a5, a6, a7, a8, a9 !for temporal usage

!_____________ Atmospheric CO2  conditions [Call cd_trend]
   !year of examine (A.D. from 1953)
   x   = 1990.0
   !annual mean increase, a6
   a0 = 1904299.0
   a1 = -3322.4242
   a2 = 1.6541596
   a3 = 1.3362655  /10000.0
   a4 = -3.0828809 /10000000.0
   a5 = 6.2121261  /100000000000.0
    a6 = a0 + x * (a1 + x * (a2 + x * (a3 + x * (a4 + x * a5))))
   !increment according to the year, a7 (constant CO2)
   a7   = 0.0 ! (-52.11+0.026984*x)*(DBLE(doy)-5.5)/12.0
   !north-south gradient, a8
   a8   = 1.6*((LAT)/85.0)
   !seasonal oscillation, a9
   if ((LAT)>=0.0) then
      a9 = exp(0.04*(LAT)) / 2.0*sin( (real(doy-  1.0))/365.0*2.0*PI )
   else 
      a9 = exp(0.04*(LAT)) / 2.0*sin( (real(doy+181.0))/365.0*2.0*PI )
   end if
   !sumup
   co2atm = a6 + a7 + a8 + a9   !background CO2 concentration

END SUBROUTINE co2_density



!**************************************************************************************************
! Air process [dynmcL]
!**************************************************************************************************
SUBROUTINE air (tmp_air, humid, wind, ALT, ap, vp, vpd, dnsa, vp_sat, slope_vps)

USE data_structure
implicit none

!_____________ Define augments
real,intent(IN) ::tmp_air   !2m air temperature (Celcius)
real,intent(IN) ::humid     !specific humidity (kg  kg-1)
real,intent(IN) ::wind      !wind velocity (m s-1)
real,intent(IN) ::ALT       !altitude (m above MSL)
real,intent(OUT)::ap        !air pressure (hPa)
real,intent(OUT)::vp        !vapour pressure (hPa)
real,intent(OUT)::vpd       !vapour pressure  deficit (hPa)
real,intent(OUT)::dnsa      !density of air (kg m-3)
real,intent(OUT)::vp_sat    !saturation vapour pressure (hPa)
real,intent(OUT)::slope_vps !slope of saturation vapour pressure (hPa deg C-1)

!_____________ Define local variables
real a1, a2  !for temporal memory

!_____________ Program body
!ap: air pressure (hPa)
   ap = 1013.25 * exp ( -0.2838472*ALT /(8.3144*(tmp_air+ZAT)) )
   
!vp: vapour pressure (hPa)
   vp = ap * humid / ( 0.622 + 0.378 * humid )
   
!vp_sat: saturated vapour pressure (hPa)
   if(tmp_air > 0.0)then   !@water surface
      vp_sat = 6.1078*(10.0**((7.5*tmp_air)/(237.3+tmp_air)))
   else                    !@ice surface
      vp_sat = 6.1078*(10.0**((9.5*tmp_air)/(265.3+tmp_air)))
   endif
   vp_sat = max(0.0, vp_sat)
   
!vpd: vapour pressure deficit (hPa)
   vpd = max(0.0, vp_sat - vp)
   
!slope_vps: slope of saturated vapour pressure as a function of temperature
   if(tmp_air > 0.0)then  !  at  water  surface
      a1 = 6.1078 * (2500.0-2.4*tmp_air)
      a2 = ( 10.0**((7.5*tmp_air)/(237.3+tmp_air)) )
   else                 !  at  ice  surface
      a1 = 6.1078 * 2834.0
      a2 = (10.0**((9.5*tmp_air)/(265.3+tmp_air)))
   endif
   slope_vps = ( a1 / (0.4615*(ZAT+tmp_air)*(ZAT+tmp_air)) ) * a2
   
!dnsa: air density
   dnsa = 1.293 * (ZAT / (tmp_air+ZAT)) * (ap / 1013.25) * (1.0 - 0.378 * vp / ap)
   
END SUBROUTINE air



!**************************************************************************************************
! Radiation (Daily)
!**************************************************************************************************
SUBROUTINE radiation &
         (LAT, doy, cloud, &
         sl_hgt, dlen, rad, par, par_diffuse, par_direct, par_RunningRecord)

USE data_structure
implicit none

!_____________ Define augments
real   ,intent(IN)::LAT        !latitude  (degree)
integer,intent(IN)::doy        !loop for day of year (1-365)
real   ,intent(IN)::cloud      !total cloudness (fraction)

real,intent(OUT)::sl_hgt      !solar hight at midday (degree)
real,intent(OUT)::dlen        !day length (hour)
real,intent(OUT)::rad         !global radiation over canopy (W/m2)
real,intent(OUT)::par         !par of mid-day (micro mol photon m-2 s-1)
real,intent(OUT)::par_diffuse !par of mid-day (micro mol photon m-2 s-1)
real,intent(OUT)::par_direct  !par of mid-day (micro mol photon m-2 s-1)
real,dimension(30),intent(INOUT)::par_RunningRecord !30days running record of par

!_____________ Define local variables
real rad_stratosphere !shortwave radiation at the stratosphere (W/m2)
real sl_dec           !solar  declination (degree)
real ha               !angle from sun-rise to median passage (degree)
real rad_diffuse      !diffused radiation in rad (W/m2)
real rad_direct       !direct radiation in rad (W/m2)

real x,y               !for general usage
real a1, a2           !for general usage

!_____________ Program body
!sl_dec: solar declination of doy (degree)
   sl_dec = 23.45 * sin(DtoR * 360.0 * ( real(doy) - 81.0 ) / 365.0)
   
!sl_hgt: solar hight at midday (degree)
   x      = sin(LAT * DtoR) * sin(sl_dec * DtoR) + &
            cos(LAT * DtoR) * cos(sl_dec * DtoR)
   x      = min(1.0, max(-1.0, x))
   sl_hgt = asin(x) * RtoD
   
!trap for high and low latitude
if (sl_hgt<=0.1) then
   dlen = 0.0
   rad  = 0.0
   par  = 0.0
   par_diffuse = 0.0
   par_direct  = 0.0
   par_RunningRecord(2:30) = par_RunningRecord(1:29)
   par_RunningRecord(1)    = 0.0
   return
endif

!dlen: day length (hour)
   x      = -tan(LAT * DtoR) * tan(sl_dec * DtoR)
   ha     = RtoD * acos( min(1.0, max(-1.0, x)) ) !angle from sun-rise to median passage
   dlen   = 2.0 * (ha / 15.0)
   
!rad_stratosphere: shortwave radiation at the atmosphere-top (W/m2) 
    a1 = DtoR * 360.0 * (doy/365.0) !seasonal angle of the earth's orbit (radian)
    a2 = 1.00011 + 0.034221 * cos(a1) + 0.00128 * sin(a1) + &
         0.000719 * cos(2.0*a1) + 0.000077 * sin(2.0*a1)
    rad_stratosphere = max(0.0, 1367.0 * sin(sl_hgt * DtoR) * a2)
    
!rad: global radiation at the ground surface (W/m2)
   x = 0.803 - 0.34*cloud - 0.458*cloud*cloud  !Black's
   x = 0.8964 - 0.5392 * cloud   !new regression based on NCEP/NCAR data
   x = min(1.0,  max(0.0, x))
   rad = rad_stratosphere * x
   
!par: photosynthetically active radiation in mid_day (micro mol photon m-2 s-1)
!   based  on  the  empirical  Tooming's  equation
   if(rad_stratosphere /= 0.0) then
      rad_diffuse = max( 0.0 , rad * (0.958 - 0.982*(rad/rad_stratosphere)) )
      rad_direct  = max( 0.0 , rad - rad_diffuse )
      
      par_diffuse = 4.2 * 0.57 * rad_diffuse
      par_direct  = 4.6 * 0.43 * rad_direct 
      par         = par_diffuse + par_direct
   else
      par         = 0.0
      par_diffuse = 0.0
      par_direct  = 0.0
   endif
   
!Update running record
   par_RunningRecord(2:30) = par_RunningRecord(1:29)
   par_RunningRecord(1)    = par
   
END SUBROUTINE radiation



!**************************************************************************************************
! Net radiation
!**************************************************************************************************
SUBROUTINE net_radiation &
          (eK, lai_grass, type_grass, C3g_no, C4g_no, biome, &
          tmp_air, cloud, Albedo_soil0, &
          pool_snow, vp, rad, par, par_grass, ir_tree, ir_grass, &
          albedo_soil, albedo_veg, radnet_soil, radnet_veg)

USE data_structure
implicit none

!_____________ Define augments
real,dimension(PFT_no),intent(IN)::eK         !light attenuation coefficient (no dimension)
real,dimension(1:2)   ,intent(IN)::lai_grass  !leaf area index of grass (m2/m2)
integer               ,intent(IN)::type_grass !grass type (3:C3 type, 4:C4 type)
integer               ,intent(IN)::C3g_no     !PFT number of C3 grass
integer               ,intent(IN)::C4g_no     !PFT number of C4 grass
integer               ,intent(IN)::biome      !biome type (see subroutine biome_type)

real,intent(IN)::tmp_air      !2m air temperature (Celcius)
real,intent(IN)::cloud        !total cloudness, fraction
real,intent(IN)::Albedo_soil0 !albedo, default
real,intent(IN)::pool_snow    !water equivalent snow depth (mm)
real,intent(IN)::vp           !vapour pressure (hPa)
real,intent(IN)::rad          !global radiation at the ground surface (W/m2)
real,intent(IN)::par          !mid-day PAR over tree canopy (micro mol photon m-2 s-1)
real,intent(IN)::par_grass    !mid-day PAR under tree canopy (micro mol photon m-2 s-1)
real,intent(IN)::ir_tree      !radiation-interruption-coefficient by tree corwn
real,intent(IN)::ir_grass     !radiation-interruption-coefficient by grass leaf

real,intent(OUT)::albedo_soil !soil surface albedo
real,intent(OUT)::albedo_veg  !vegetation area albedo
real,intent(OUT)::radnet_veg  !canopy net radiation, short wave (W/m2)
real,intent(OUT)::radnet_soil !soil surface net radiation, short wave (W/m2)

!_____________ Define local variables
real    radnet_long  !net longwave radiation (W/m2)
real    ir           !interception by leavies

!_____________ Program body
!albedo_soil: soil surface albedo
   albedo_soil = Albedo_soil0 + (0.7-Albedo_soil0) / (1+exp(-0.05*(pool_snow-70.0)))
   albedo_soil = max(0.05, min(0.70, albedo_soil))

!albedo_veg: canopy albedo (temporal)
   select case (biome)
      case (3:10)   !forests
         albedo_veg = 0.15
      case default  !grass land, desert, etc
         albedo_veg = 0.24
   end select

!radnet_long: net longwave radiation
   radnet_long = (5.6703/100000000.0) * ((tmp_air + ZAT)**4.0) &
                  * (0.39 + 0.058/(vp+1.0)) * (1.0 - 0.65*cloud)

!ir: interception by canopy
   !by tree crown
   ir = min(1.0, par_grass / max(0.01, par) )
   
   !by grass layer
   if (type_grass==3) then
      ir = ir * exp( -1.0 * ( 0.9*lai_grass(1)*eK(C3g_no) + 0.1*lai_grass(2)*eK(C4g_no) ) )
   else
      ir = ir * exp( -1.0 * ( 0.1*lai_grass(1)*eK(C3g_no) + 0.9*lai_grass(2)*eK(C4g_no) ) )
   end if
   ir = min(1.0, max(0.0, ir) )
   
!radnet_veg: net radiation of plant canopy
   radnet_veg   = ( rad*(1.0-albedo_veg) - radnet_long ) * (1.0 - ir_tree * ir_grass)
   radnet_veg   = max (0.0, radnet_veg)
   
!radnet_soil: net radiation of soil surface
   radnet_soil   = ( rad*(1.0-albedo_soil) - radnet_long ) * ir_tree * ir_grass
   radnet_soil   = max (0.0, radnet_soil)
   
END SUBROUTINE net_radiation



!**************************************************************************************************
! Water budget module, daily computation
!**************************************************************************************************
SUBROUTINE waterbudget &
      (doy, tmp_air, tmp_soil, prec, wind, tmp_soil_RunningRecord, &
      W_fi, W_wilt, W_sat, W_mat, &
      dlen, radnet_veg, radnet_soil, vpd, dnsa, slope_vps, canopy_cond, &
      biome, pft, la, tree_exist, mass_root, pool_litter, &
      type_grass, gmass_root, lai_grass, C3g_no, C4g_no, &
      pool_w1, pool_w2, pool_w3, pool_snow, &
      hydRecord_preci, hydRecord_runoff, hydRecord_ic, hydRecord_ev, hydRecord_tr)

USE data_structure
implicit none

!_____________ Define augments
integer,intent(IN)::doy         !day of the year (1-365)
real   ,intent(IN)::tmp_air     !air temperature (Celcius)
real   ,intent(IN)::tmp_soil    !ground surface temperature (Celcius)
real   ,intent(IN)::prec        !precipitation,  mm day-1
real   ,intent(IN)::wind        !wind velocity (m s-1)

real,dimension(365),intent(IN)::tmp_soil_RunningRecord

real   ,intent(IN)::W_fi        !filed capacity   (m3/m3, 0.0 -> 1.0)
real   ,intent(IN)::W_wilt      !wilting point    (m3/m3, 0.0 -> 1.0)
real   ,intent(IN)::W_sat       !saturate point   (m3/m3, 0.0 -> 1.0)
real   ,intent(IN)::W_mat       !matrix potential

real   ,intent(IN)::dlen        !day length (hour)
real   ,intent(IN)::radnet_veg  !canopy net radiation, short wave (W m-2)
real   ,intent(IN)::radnet_soil !soil surface net radiation, short wave (W m-2)
real   ,intent(IN)::vpd         !vapour pressure  deficit (hPa)
real   ,intent(IN)::dnsa        !density of air (kg m-3)
real   ,intent(IN)::slope_vps   !slope of saturation vapour pressure related to temp (hPa deg C-1)
real   ,intent(IN)::canopy_cond !monthly stomatal conductance   (mol H2O m-2 s-1)

integer                  ,intent(IN)::biome      !biome type (definition-> subroutine biome_type)
integer,dimension(Max_no),intent(IN)::pft        !Plant Functional Type (1-PFT_no)
real   ,dimension(Max_no),intent(IN)::la         !leaf area (m2)
logical,dimension(Max_no),intent(IN)::tree_exist !tree exist flag
real   ,dimension(Max_no),intent(IN)::mass_root  !biomass of root (g)
real                     ,intent(IN)::pool_litter !litter pool in a stand (g DM / stand)

integer              ,intent(IN)::type_grass  !grass type (3:C3 type, 4:C4 type)
real   ,dimension(2),intent(IN)::gmass_root  !grass root biomass (g/m2)
real   ,dimension(2),intent(IN)::lai_grass   !leaf area index of grass (m2/m2)
integer             ,intent(IN)::C3g_no      !PFT number of C3 grass
integer             ,intent(IN)::C4g_no      !PFT number of C4 grass

real,intent(INOUT)::pool_w1   !soil water content of soil layer 1 (mm)
real,intent(INOUT)::pool_w2   !soil water content of soil layer 2 (mm)
real,intent(INOUT)::pool_w3   !soil water content of soil layer 3 (mm)
real,intent(INOUT)::pool_snow !water equivalent snow depth (mm)

real,intent(INOUT)::hydRecord_preci  !sum of precipitation      (mm/month)
real,intent(INOUT)::hydRecord_runoff !sum of runoff water       (mm/month)
real,intent(INOUT)::hydRecord_ic     !sum of interception water (mm/month)
real,intent(INOUT)::hydRecord_ev     !sum of evaporated water   (mm/month)
real,intent(INOUT)::hydRecord_tr     !sum of tranpirated water  (mm/month)

!_____________ Define local variables
real    flux_snow  !snow fall (mm/day)
real    flux_rain  !rain fall (mm/day)
real    flux_ic    !intercepted rain (mm/day)
real    flux_tw    !snow thaw water (mm/day)
real    flux_sfc   !available water on soil surface (mm/day)
real    flux_ev    !evaporation flux (mm/day)
real    flux_tr    !transpiration flux (mm/day)
real    flux_ev_pm !potential evaporation rate  (mm/day)
real    flux_tr_pm !potential transpiration rate  (mm/day)

real    lai        !leaf area index (m2 m-2)
real    rain_no    !expected number of rain event (day-1)

real    c_aero     !aerodynamic conductance (s m-1)
real    c_soil     !soil conductance (s m-1)
real    c_leaf     !canopy conductance (s m-1)

logical frozen_soil1 ! if true, soil layer 1 is frozen
logical frozen_soil2 ! if true, soil layer 1 is frozen

real    water_available  
real    h_veg            !height of vegetaiton surface (m)
real    pool_total       !for checker (total water available in this virtual field)

real    x, y, z, a, b, c !for general usage
integer i, j, p, no      !for general usage

!_____________ checker
    pool_total  = prec + pool_snow + pool_w1 + pool_w2 + pool_w3

!_____________ Calculate Potential Eveporation & Transpiration
   
   !*** Preparetion for Evapotranspiration
   !c_aero: aerodynamic conductance (s m-1)
   select case (biome)
      case (3:10)   !forests
         h_veg = 10.0
      case default  !grass land, desert, etc
         h_veg = 1.0
   end select
   x = (10.0 + h_veg - 0.78*h_veg) / (0.07*h_veg)
   c_aero = (0.41*0.41*wind) / (log(x)**2.0)
   
   !c_soil: surface soil conductance (s m-1)
   x = pool_w1 / (W_fi*Depth1) ; x = min(1.0, x)
   c_soil = 0.0224 * (x**2.0)   !c_soil = 0.0224 * x
   
!TMP
!c_soil = c_soil * ( 2.7182818**(-0.1*pool_litter/Max_loc/Max_loc/1000) )
   
   !c_leaf: canopy conductance (m s-1)
   c_leaf = 0.0224 * canopy_cond
   
   !*** Potential transpiration
   if (c_leaf>0.001) then
      a          = slope_vps * radnet_veg * 0.5
      b          = dnsa * 1012 * vpd * c_aero
      c          = 695.0 * ( slope_vps + 0.667 * (1.0+c_aero/c_leaf) )
      flux_tr_pm = dlen * ( a + b ) / c
   else
      flux_tr_pm = 0.0
   endif
   
   !*** Potential evaporation
   if (c_soil>0.001) then
      a          = slope_vps * radnet_soil * 0.5
      b          = dnsa * 1012 * vpd * c_aero
      c          = 695.0 * ( slope_vps + 0.667 * (1.0+c_aero/c_soil) )
      flux_ev_pm = dlen * ( a + b ) / c

!TMP
!flux_ev_pm = dlen * a  / c

   else
      flux_ev_pm = 0.0
   endif
   
   if (tmp_soil<=0.0) flux_ev_pm = 0.0

!_____________ Frag permafrost
x = tmp_soil
y = sum ( tmp_soil_RunningRecord(1:365) )

frozen_soil1 = .false. ; if (x<0.0) frozen_soil1 = .true.
frozen_soil2 = .false. ; if (y<0.0) frozen_soil2 = .true.

!_____________ Fluxus of snow, rain, and thaw water
   !x: proportion of snow in precipitation
   x = 1.0 / ( 1.0 + exp(0.75*tmp_air - 1.5) )
   
   !y: proportion of snow melt in this day
   y = 1.0 / ( 1.0 + exp( -0.3*(tmp_soil-10.0) ) )
   
   !water balance
   flux_snow   = prec * x
   flux_rain   = prec * (1.0-x)
   flux_tw     = pool_snow  * y
   pool_snow   = pool_snow   + flux_snow - flux_tw
   hydRecord_preci = hydRecord_preci + prec
   
   !checker #1
   y          = flux_rain + flux_tw + pool_snow + pool_w1 + pool_w2 + pool_w3
   if ( Logging==1 .and. abs(pool_total-y)>0.1 ) then
      write(*,'(a,2f7.1)') 'error@waterbudget_1', pool_total, y
   endif
   
!_____________ Canopy Interception
   
   !calculate LAI
   lai = 0.0
   
   do no=1, Max_no
      if (tree_exist(no)) lai = lai + la(no)
   end do
   lai = lai / Max_loc / Max_loc
   
   if ( type_grass == 3 ) then
      lai = lai + 0.9*lai_grass(1) + 0.1*lai_grass(2)
   else
      lai = lai + 0.1*lai_grass(1) + 0.9*lai_grass(2)
   end if
   
   !determine fraction interception
   rain_no = 0.1 * flux_rain
   if ( flux_tr_pm + flux_ev_pm <= 50.0 * 12.0 / 365.0 ) then
      rain_no = min( rain_no,  5.0 * 12.0 / 365.0 )
   else
      rain_no = min( rain_no, 10.0 * 12.0 / 365.0 )
   endif
   
   flux_ic = 3.0 * rain_no * ( 1.0 - exp(-1.0*lai) )
   flux_ic = min(flux_rain, flux_ic)          
   
   !water balance
   !flux_ic: intercepted water (mm)
   hydRecord_ic = hydRecord_ic + flux_ic           
   flux_sfc     = flux_rain    - flux_ic + flux_tw 
   
   !cheker #2
   pool_total = pool_total - flux_ic
   y = flux_sfc + pool_snow + pool_w1 + pool_w2 + pool_w3
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_2', pool_total, y
   endif
   
!_____________ Surface runoff when soil surface was frozen
!++++ water holding capacity of litter might be taken into consideration +++
!   if (frozen_soil1) then
!      hydRecord_runoff = hydRecord_runoff + flux_sfc !recorder
!      pool_total       = pool_total       - flux_sfc !checker
!      flux_sfc = 0.0 
!   end if
   
!_____________ Infiltration: soil_surface -> soil_layer_1
   
   !infiltration by matrix potential
   if (flux_sfc > 0.0) then
      !x: absorbed water by matrix potential
      x = max(0.0, W_mat*Depth1 - pool_w1) !maximum volume of absorbed water by matrix potential
      x = min(flux_sfc, x)                 !actual  volume of absorbed water by matrix potential
      !water balance
      pool_w1  = pool_w1  + x
      flux_sfc = flux_sfc - x
   endif
   
   !infiltration by field capacity & Runoff from soil surface
   if (flux_sfc > 0.0) then
      !x: infiltration water (mm)
      y = (pool_w1/Depth1 - W_mat) / max(0.01, W_sat-W_mat)
      x = 1.0 - max(0.001, y) ** K0_infilt
      x = flux_sfc * min(1.0, max(0.0, x))
      x = min (x , W_sat*Depth1 - pool_w1)
      
      !water balance
      pool_w1          = pool_w1          + x            !infiltration
      hydRecord_runoff = hydRecord_runoff + (flux_sfc-x) !recorder
      pool_total       = pool_total       - (flux_sfc-x) !checker
   end if
   
   !cheker #3
   y = pool_snow + pool_w1 + pool_w2 + pool_w3
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_3', pool_total, y
   endif
   
!_____________ Penetration 1: soil_layer_1 -> soil_layer_2
   !x: penetrate water from upper to lower layer (mm)
   if     (pool_w1 > W_fi*Depth1 + 1.0 ) then
      !condition1: flood
      x = K1_s_up * (pool_w1-W_fi*Depth1) * &
          (( (pool_w1/Depth1-W_fi) / max(0.01, W_sat-W_fi) )**K2_s_up)
   elseif (pool_w1 > W_mat*Depth1 + 1.0 ) then
      !condition2: moderate moisture
      x = K1_u_up * (pool_w1-W_mat*Depth1) * &
          (( (pool_w1/Depth1-W_mat) / max(0.01, W_sat-W_mat) )**K2_u_up)
   else
      !condition3: dry
      x = 0.0
   endif
   
      !condition4: soil layer 2 is forzen
!      if (frozen_soil2) x = 0.0
   
   x = min(x, W_sat*Depth2 - pool_w2)
   x = max(x, 0.0)
   
   !water balance
   pool_w1 = pool_w1 - x
   pool_w2 = pool_w2 + x
   
   !cheker #4
   y = pool_snow + pool_w1 + pool_w2 + pool_w3
   if ( Logging==1 .and. abs(pool_total-y)>0.1 ) then
      write(*,'(a,2f7.1)') 'error@waterbudget_4', pool_total, y
   endif
   
!_____________ Penetration 2: soil_layer_2 -> soil_layer_3
   !x: penetrate water from upper to lower layer (mm)
   if     (pool_w2 > W_fi*Depth2 + 1.0 ) then
       !condition1: flood
       x = K1_s_low * (pool_w2-W_fi*Depth2) * &
           (( (pool_w2/Depth2-W_fi) / max(0.01, W_sat-W_fi) )**K2_s_low)
   elseif (pool_w2 > W_mat*Depth2 + 1.0 ) then
       !condition2: moderate moisture
       x = K1_u_low * (pool_w2-W_mat*Depth2) * &
           (( (pool_w2/Depth2-W_mat) / max(0.01, W_sat-W_mat) )**K2_u_low)
   else
       !condition3: dry
       x = 0.0
   endif
   x = min(x, W_sat*Depth3 - pool_w3)
   x = max(x, 0.0)
   
   !water balance
   pool_w2 = pool_w2 - x
   pool_w3 = pool_w3 + x
   
   !cheker #5
   y = pool_snow + pool_w1 + pool_w2 + pool_w3
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
       write(*,'(a,2f7.1)') 'error@waterbudget_5', pool_total, y
   endif
   
!_____________ Penetration 3: soil_layer_3 -> runoff
   !x: penetrate (runoff) water to basement layer
   if     (pool_w3 > W_fi*Depth3 + 1.0 ) then
      !condition1: flood
      x = K1_s_base * (pool_w3-W_fi*Depth3) * &
          (( (pool_w3/Depth3-W_fi) / max(0.01, W_sat-W_fi) )**K2_s_base)
   elseif (pool_w3 > W_mat*Depth3 + 1.0 ) then
      !condition2: moderate moisture
      x = K1_u_base * (pool_w3-W_mat*Depth3) * &
          (( (pool_w3/Depth3-W_mat) / max(0.01, W_sat-W_mat) )**K2_u_base)
   else
      !condition3: dry
      x = 0.0
   endif
  
   !water balance
   pool_w3          = pool_w3          - x
   hydRecord_runoff = hydRecord_runoff + x
   
   !cheker #6
   pool_total = pool_total - x
   y = pool_snow + pool_w1 + pool_w2 + pool_w3
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_6', pool_total, y
   endif
   
!_____________ Actual transpiration (flux_tr)
   flux_tr_pm      = max(0.0, flux_tr_pm - flux_ic)
   water_available = max(0.0, pool_w1-Depth1*W_wilt) + max(0.0, pool_w2-Depth2*W_wilt)
   a = 0.85 !a = 0.1
   b       = water_available + flux_tr_pm
   c       = water_available * flux_tr_pm
   flux_tr = (b - sqrt( b*b - 4.0*a*c )) / (2.0*a)
   flux_tr = max(flux_tr, 0.0)
   flux_tr = min(flux_tr, water_available)
   
   !** water balance (method 1 / absorbed from first layer first)
!   x = max(0.0, pool_w1-Depth1*W_wilt) !available water in soil layer 1 for transpiration
!   y = max(0.0, pool_w2-Depth2*W_wilt) !available water in soil layer 2 for transpiration
!   if     ( x   >= flux_tr) then
!      !when sufficient water exist in soil layer 1
!      pool_w1 = pool_w1 - flux_tr
!   elseif ( x+y >= flux_tr) then
!      !when sufficient water exist in soil layer 1 + 2
!      pool_w1 = pool_w1 - x
!      pool_w2 = pool_w2 - ( flux_tr - x )
!   else
!      flux_tr = x+y
!      pool_w1 = pool_w1 - x
!      pool_w2 = pool_w2 - y
!   endif
   
   !** water balance (method 2 / absorbed so as to soil-wetness of both layer is equal)
   x = max(0.0, pool_w1/Depth1-W_wilt) !available water index in soil layer 1
   y = max(0.0, pool_w2/Depth2-W_wilt) !available water index in soil layer 1
   if     ( x >= y) then
      a = min(pool_w1-W_wilt*Depth1, flux_tr)
      a = max(0.0, a)
      pool_w1 = pool_w1 - a
      pool_w2 = pool_w2 - (flux_tr-a)
   else
      b = min(pool_w2-W_wilt*Depth2, flux_tr)
      b = max(0.0, b)
      pool_w2 = pool_w2 - b
      pool_w1 = pool_w1 - (flux_tr-b)
   endif
   
   !water recorder
   hydRecord_tr = hydRecord_tr + flux_tr
   
   !cheker #7
   pool_total = pool_total - flux_tr
   y = pool_snow + pool_w1 + pool_w2 + pool_w3
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_7', pool_total, y
   endif

!_____________ Actual evaporation
   a = 0.85 !a = 0.1
   b = pool_w1 + flux_ev_pm
   c = pool_w1 * flux_ev_pm
   flux_ev = (b - sqrt( b*b - 4.0*a*c )) / (2.0*a)
   flux_ev = max(flux_ev, 0.0)
   flux_ev = min(flux_ev, pool_w1)
  
   !water balance
   pool_w1      = pool_w1      - flux_ev
   hydRecord_ev = hydRecord_ev + flux_ev
   
   !cheker #8
   pool_total = pool_total - flux_ev
   y = pool_snow + pool_w1 + pool_w2 + pool_w3
   if ( Logging==1 .and. abs(pool_total-y)>0.1 )  then
      write(*,'(a,2f7.1)') 'error@waterbudget_8', pool_total, y
   endif
   
!_____________ Display annual hydrogy stat (for tuning)
!    if ( doy==365 ) then
!      write(*,*) '    '
!      write(*,*) '    ****** hydrogy stat of the year ******'
!      write(*,*) '    precp(mm)   ro(%)   ic(%)   ev(%)   tr(%)'
!      write(*,'( 4x, f7.1, 4(4x, f5.1) )') &
!             hydRecord_preci                        , &
!            (hydRecord_runoff / hydRecord_preci)*100, &
!            (hydRecord_ic     / hydRecord_preci)*100, &
!            (hydRecord_ev     / hydRecord_preci)*100, &
!            (hydRecord_tr     / hydRecord_preci)*100
!      write(*,*) '    '
!    end if

END SUBROUTINE waterbudget



!*************************************************************************************************
! light interruption index (daily computation)
!*************************************************************************************************
SUBROUTINE ir_index &
   (ir_tree, ir_grass, par_grass, type_grass, C3g_no, C4g_no, &
   par, par_direct, par_diffuse, phenology, eK, pft, tree_exist, la, lai_grass)

USE data_structure
implicit none

!_____________ Define augments
real,intent(OUT)::ir_tree      !radiation-interruption-coefficient by tree corwn
real,intent(OUT)::ir_grass     !radiation-interruption-coefficient by grass leaf
real,intent(OUT)::par_grass    !total PAR on top of the grass layer (micro mol photon m-2 s-1)

integer,intent(IN)::type_grass !grass type priority (3:C3 type, 4:C4 type)
integer,intent(IN)::C3g_no     !PFT number of C3 grass
integer,intent(IN)::C4g_no     !PFT number of C4 grass

real,intent(IN) ::par          !canopy top PAR of mid-day
real,intent(IN) ::par_direct   !canopy top direct PAR of mid-day
real,intent(IN) ::par_diffuse  !canopy top difussed PAR of mid-day (micro mol photon m-2 s-1)

logical,dimension(PFT_no),intent(IN)::phenology  !Phenological status (T:foliation, F:dormance)
real   ,dimension(PFT_no),intent(IN)::eK         !light attenuation coefficient, no dimension
integer,dimension(Max_no),intent(IN)::pft        !Plant Functional Type (1-PFT_no)
logical,dimension(Max_no),intent(IN)::tree_exist !tree exist flag
real   ,dimension(Max_no),intent(IN)::la         !leaf area      (m2)
real,dimension(2)        ,intent(IN)::lai_grass  !c3/c4 leaf area index of grass (m2/m2)

!Define local variables
real    x, y     !for temporal usage
integer no, p    !for loop counter

ir_tree   = 0.00
ir_grass  = 0.00
par_grass = 0.00

IF ( par > 1.0 ) then

!ir_tree: radiation-interruption-coefficient by tree corwn
   x = 0.0 ; y = 0.0 
   do no = 1, Max_no
      p = pft(no)
      if ( tree_exist(no) .and. phenology(p) ) then
         x = x + la(no) * max(EK0(p),eK(p)) !sum of light attenuation index (direct radiation)
         y = y + la(no) * EK0(p)            !sum of light attenuation index (diffused radiation)
      endif
   enddo
   x = x / Max_loc / Max_loc
   y = y / Max_loc / Max_loc
   
   ir_tree = (par_direct/par)*exp(-1.0*x) + (par_diffuse/par)*exp(-1.0*y)
   ir_tree = min(1.0, max(0.0, ir_tree) )
   
!par_grass: PAR on top of the grass layer
   par_grass = par * ir_tree
   
!ir_grass: radiation-interruption-coefficient by grass leaf
   
   !radiation-interruption-coefficient of C3 grass
   x = (par_direct /par)*exp(-1.0*lai_grass(1)* eK(C3g_no)) + &
       (par_diffuse/par)*exp(-1.0*lai_grass(1)*EK0(C3g_no))  
   
   !radiation-interruption-coefficient of C4 grass
   y = (par_direct /par)*exp(-1.0*lai_grass(2)* eK(C4g_no)) + &
       (par_diffuse/par)*exp(-1.0*lai_grass(2)*EK0(C4g_no))  
   
   if (type_grass==3) then
      ir_grass = 0.9*x + 0.1*y
   else
      ir_grass = 0.1*x + 0.9*y
   end if
   ir_grass = min(1.0, max(0.0, ir_grass) )
   
ENDIF

END SUBROUTINE ir_index
