!**************************************************************************************************
! Establishment of woody PFTs
!**************************************************************************************************
SUBROUTINE establish &
    (counter, fire_clo, biome, tmp_coldest_20yr_ave, gdd_20yr_ave, c3g_no, c4g_no, &
    par, par_grass, par_grass_RunningRecord, stat_water_RunningRecord, &
    pool_litter, c_uptake, pft_exist, patch_vacant, &
    pft, age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
    mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
    bole_x, bole_y, crown_x, crown_y, &
    tree_exist, height_limit, radius_limit, mort_regu1, mort_regu2, &
    stat_leaf, stat_leaf_RunningRecord, stat_leaf_crowntop, stat_leaf_crownbottom)

USE data_structure
implicit none

!_____________ Definitions of establishment control parameters
!   P_establish
!      Actual establishment rate when a establishment patch was assigned to a PFT
!   
!   Est_scenario:
!      scenario for establishment
!      0 -> no trees can establish
!      1 -> only specified woody PFTs can establish
!      2 -> every potentially eatablishable PFT have same chance of establishment
!      3 -> in proportion of existing biomass after specific year
!      4 -> in proportion of existing biomass after specific year,
!           while little portion of establishment was randomly selected
!           from all potentialy establishable PFTs
!   
!   Est_pft_OnOff:
!      establishment switch (for Scenerio 1)
!      .true.  -> allow establish
!      .false. -> do not allow establish
!   
!   Est_year_change
!      lenght of year until establishment pattern will change (for Scenerio 3, 4)
!   
!   Est_frac_random
!      Fraction of random establishment after Est_year_change (for Scenerio  4)

!_____________ Definitions of establishment regulator (PFT specific variables)
!   est_capacity
!      It indicates wheather a PFT can establish or not on this site (true or false)
!   
!   est_possibility
!      Frequency that a establishment patch is allocated to a PFT
!      Thus, sum of est_possibility for all PFTs is always 1.0

!_____________ Define local parameter
real,parameter::Thres_stat_water_dry = 0.3 !Threshold stat_water to determine dry month

!_____________ Define Arguments
integer,intent(IN)::counter              !simulation day (1-Simulation_year*365)
integer,intent(IN)::fire_clo             !counter number of last fire (yr)
integer,intent(IN)::biome                !biome type (see subroutine biome_type for definition)
real   ,intent(IN)::tmp_coldest_20yr_ave !20yr running mean of tmp_coldest_month (Celcius)
real   ,intent(IN)::gdd_20yr_ave         !20yr running mean of growing-degree-day-sum
integer,intent(IN)::C3g_no               !PFT number of C3 grass
integer,intent(IN)::C4g_no               !PFT number of C4 grass
real   ,intent(IN)::par                  !par over forest (micro mol photon m-2 s-1)
real   ,intent(IN)::par_grass            !par on grass layer (micro mol photon m-2 s-1)
real,dimension(365)       ,intent(IN)::par_grass_RunningRecord  !1yr record of par_grass
real,dimension(PFT_no,365),intent(IN)::stat_water_RunningRecord !1yr record of stat_water

real   ,intent(INOUT)::pool_litter       !litter pool in a stand (g / stand)
real   ,intent(INOUT)::c_uptake          !monthly carbon uptake (C g / month stand)

logical,dimension(PFT_no)         ,intent(IN)::pft_exist    !PFT exist flag
logical,dimension(Dived,Dived),intent(IN)::patch_vacant !safe site for establishment

integer,dimension(Max_no),intent(INOUT)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(Max_no),intent(INOUT)::age            !tree age (year)
integer,dimension(Max_no),intent(INOUT)::height         !tree height from 1.3m above ground(STEP)
integer,dimension(Max_no),intent(INOUT)::bole           !bole length (STEP)
real   ,dimension(Max_no),intent(INOUT)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real   ,dimension(Max_no),intent(INOUT)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real   ,dimension(Max_no),intent(INOUT)::crown_diameter !crown diameter (m)
real   ,dimension(Max_no),intent(INOUT)::crown_area     !crown area (m2)
real   ,dimension(Max_no),intent(INOUT)::la             !leaf area  (m2)
real   ,dimension(Max_no),intent(INOUT)::mass_leaf      !biomass of foliage  (g)
real   ,dimension(Max_no),intent(INOUT)::mass_trunk     !biomass of stem     (g)
real   ,dimension(Max_no),intent(INOUT)::mass_root      !biomass of root     (g)
real   ,dimension(Max_no),intent(INOUT)::mass_stock     !biomass for stock   (g)
real   ,dimension(Max_no),intent(INOUT)::mass_available !biomass available   (g)
real   ,dimension(Max_no),intent(INOUT)::bole_x         !x_location of bole  (m)
real   ,dimension(Max_no),intent(INOUT)::bole_y         !y_location of bole  (m)
real   ,dimension(Max_no),intent(INOUT)::crown_x        !x_location of crown (m)
real   ,dimension(Max_no),intent(INOUT)::crown_y        !y_location of crown (m)
logical,dimension(Max_no),intent(INOUT)::tree_exist     !tree exist or not (T or F)
integer,dimension(Max_no),intent(INOUT)::height_limit   !max_tree_height proximate trees permit
real   ,dimension(Max_no),intent(INOUT)::radius_limit   !max_canopy_radius proximate trees permit
real   ,dimension(Max_no),intent(INOUT)::mort_regu1     !NPP sum within the previous year (g dm)
real   ,dimension(Max_no),intent(INOUT)::mort_regu2     !mean leaf area of last year (m2/day)

real,dimension(Max_no    ),intent(INOUT)::stat_leaf               !(income-cost)/leaf_mass
real,dimension(Max_no,  7),intent(INOUT)::stat_leaf_RunningRecord !running record of stat_leaf
real,dimension(Max_no    ),intent(INOUT)::stat_leaf_crowntop      !stat_leaf of crown top
real,dimension(Max_no,10 ),intent(INOUT)::stat_leaf_crownbottom   !stat_leaf of crown bottom

!_____________ Define local variables
real   ,dimension(PFT_no)::est_possibility  !see above for definition
logical,dimension(PFT_no)::est_capacity     !see above for definition
real   ,dimension(PFT_no)::mass_sum         !total biomass of examining tree (DM g)
integer                    no_possible      !number of woody PFTs that can establish here
integer,dimension(PFT_no)::dry_month        !maximum successive number of dry month
real   ,dimension(12)    ::stat_water_month !monthly average of stat_water
integer                    yr               !Simulation year from initialization (including spinup period)

integer est_scenario_current !establishment scenario for current year
integer new                  !number of new sapling
integer i, j, k, p, no       !for loop counter
real    x, y                 !for general usage


!_____________ calculate year of this simulation
yr = int( (counter-1)/365 ) + 1

!!_____________ TMP
!if (yr==1) then
!   new=1
!   pft(new) = 4
!   i = 15
!   j = 15
!   
!   tree_exist(new) = .true.
!   bole_x (new)    = real(i) * ( real(Max_loc) / real(Dived) ) - 0.5*real(Max_loc)/real(Dived)
!   bole_y (new)    = real(j) * ( real(Max_loc) / real(Dived) ) - 0.5*real(Max_loc)/real(Dived)
!   crown_x(new)    = bole_x (new)
!   crown_y(new)    = bole_y (new)
!   
!   Call init_tree &
!   (par, par_grass, pool_litter, c_uptake, &
!   pft(new), age(new), height(new), bole(new), dbh_heartwood(new), dbh_sapwood(new), &
!   crown_diameter(new), crown_area(new), la(new), &
!   mass_leaf(new), mass_trunk(new), mass_root(new), mass_stock(new), mass_available(new), &
!   tree_exist(new), height_limit(new), radius_limit(new), &
!   mort_regu1(new), mort_regu2(new), &
!   stat_leaf(new), stat_leaf_RunningRecord(new,1:7), &
!   stat_leaf_crowntop(new), stat_leaf_crownbottom(new,1:10) )
!endif
!return

!_____________ Determine to prceed establishment process
!If tree number is maximum, omit entire subroutine
   no = 0
   do i=1, Max_no
     if (tree_exist(i)) no=no+1
   end do
   if (no >= Max_no) return
   
!If fire occured within 10 years, omit entire subroutine
!   if ( counter-fire_clo <= 10*365 ) return

!_____________ Compute variables that control establishment
!dry_month(1:PFT_no), maximum consecutive of drought month for each PFT
   dry_month(1:PFT_no) = 0
   Do p=1, PFT_no
      stat_water_month(1:12) = 0.0
      
      !sum up stat_water for each month
      Do i=1, 365
         stat_water_month(Month(i)) = stat_water_month(Month(i)) + stat_water_RunningRecord(p,i)
      End do
      
      !make average of stat_water for each month
      Do i=1, 12
         stat_water_month(i) = stat_water_month(i) / real(Day_in_month(i))
      End do
      
      !determine maximum consecutive of drought month for each PFT
      k = 0
      Do i=1, 12*2
         j=i ; if (i>12) j=i-12 !j: month to refer
         k=k+1                  !k: current consective dry month
         if (stat_water_month(j) > Thres_stat_water_dry) k=0
         dry_month(p) = max(dry_month(p), k)
      End do
      dry_month(p) = min(dry_month(p), 12)
      
   End do
   
!est_capacity(1:PFT_no), establishment potential for each PFT based on climatic range
   est_capacity(:) = .true.
   x = sum( par_grass_RunningRecord(1:365) ) / 365.0
   do p = 1, PFT_no
      if ( p==c3g_no .or. p==c4g_no             ) est_capacity(p)=.false.
      if ( tmp_coldest_20yr_ave >=   TC_max(p)  ) est_capacity(p)=.false.
      if ( gdd_20yr_ave         <=   GDD_min(p) ) est_capacity(p)=.false.
      if ( dry_month(p)         >    DM_max(p)  ) est_capacity(p)=.false.
      if ( x                    <=   PAR_min(p) ) est_capacity(p)=.false.
   end do
   
!mass_sum(1:PFT_no), sum of biomass for each woody PFT with establishment capacity
   mass_sum(:) = 0.0
   do no = 1, Max_no
   if ( tree_exist(no) .and. est_capacity(pft(no)) ) then
      mass_sum(pft(no)) = mass_sum(pft(no)) + &
      mass_leaf(no)+mass_trunk(no)+mass_root(no)+mass_available(no)+mass_stock(no)
   end if
   end do
   
!_____________ Compute establishment probability for each PFT
!Initialize variables
   est_possibility(1:PFT_no)= 0.0 !establishment probability, a scenario specific value
   no_possible              = 0   !number of PFT that establish at this site

!Determine establishment pattern of current year
Select case (Est_scenario)
   Case (0)
      est_scenario_current = 0
   Case (1)
      est_scenario_current = 1
   Case (2)
      est_scenario_current = 2
   Case (3)
      if (yr>=Est_year_change .and. counter-fire_clo>=Est_year_change*365) then
         est_scenario_current = 3
      else
         est_scenario_current = 2
      endif
   Case (4)
      if (yr>=Est_year_change .and. counter-fire_clo>=Est_year_change*365) then
         est_scenario_current = 4
      else
         est_scenario_current = 2
      endif
End select

!Modification of establishment pattern for wet tropical regions
   if (biome==3 .or. biome==4) est_scenario_current=2

!Give establishment probability according to the establishment pattern
Select case (est_scenario_current)
   Case (1) !only specified woody PFTs can establish
      do p = 1, PFT_no
      if ( est_capacity(p) .and. Est_pft_OnOff(p) ) then
         no_possible        = no_possible + 1
         est_possibility(p) = 1.0
      endif
      enddo
      
   Case (2) !every potentially eatablishable PFT have same chance of establishment
      do p = 1, PFT_no
      if ( est_capacity(p) ) then
         no_possible        = no_possible + 1
         est_possibility(p) = 1.0
      end if
      end do
      
   Case (3) !in proportion of existing biomass
      do p = 1, PFT_no
      if ( est_capacity(p) ) then
         est_possibility(p) = mass_sum(p) / max( 1, sum(mass_sum(:)) )
         if (est_possibility(p) > 0.00) no_possible=no_possible+1
      endif
      end do
      
   Case (4) !in proportion of existing biomass, while little portion of establishment
            !was randomly selected from all potentialy establishable PFTs
      
      do p = 1, PFT_no
         if (est_capacity(p)) no_possible=no_possible+1
      enddo
      
      do p = 1, PFT_no
      if (est_capacity(p)) then
         x = Est_frac_random / max(1.0, no_possible)                      !fraction stochastic
         y = (1.0-Est_frac_random) * (mass_sum(p) / max(1,sum(mass_sum))) !fraction deterministic
         est_possibility(p) = est_possibility(p) + x + y
      endif
      end do
      
End select

!Adjust est_possibility, so that sum of value will be 1.0
   est_possibility(:) = est_possibility(:) / real( max(1.0, no_possible) )

!_____________ Establishment processes
if (no_possible == 0) return

!loop for each ground mesh
new = 1
Do i = 1, Dived
Do j = 1, Dived
   
   !K: determin PFT of newly establish tree
   x = randf(I_seed)
   k = 1
   do while ( sum(est_possibility(1:k)) < x .and. k<PFT_no)
      k = k+1
   end do
   
   !x: adjusted establishment rate of PFT k
   x = P_establish(k) * real(Max_loc**2) / real(Dived**2)
   
if ( patch_vacant(i,j) ) then
if ( x > randf(I_seed) ) then
   !determine new sapling ID-number
   do while ( tree_exist(new) )
      new = new + 1
   end do
   if ( new > Max_no ) return
   
   !initiate new sapling
   pft(new)        = k
   tree_exist(new) = .true.
   bole_x (new)    = real(i) * ( real(Max_loc) / real(Dived) ) - 0.5*real(Max_loc)/real(Dived)
   bole_y (new)    = real(j) * ( real(Max_loc) / real(Dived) ) - 0.5*real(Max_loc)/real(Dived)
   crown_x(new)    = bole_x (new)
   crown_y(new)    = bole_y (new)
   
   Call init_tree &
   (par, par_grass, pool_litter, c_uptake, &
   pft(new), age(new), height(new), bole(new), dbh_heartwood(new), dbh_sapwood(new), &
   crown_diameter(new), crown_area(new), la(new), &
   mass_leaf(new), mass_trunk(new), mass_root(new), mass_stock(new), mass_available(new), &
   tree_exist(new), height_limit(new), radius_limit(new), &
   mort_regu1(new), mort_regu2(new), &
   stat_leaf(new), stat_leaf_RunningRecord(new,1:7), &
   stat_leaf_crowntop(new), stat_leaf_crownbottom(new,1:10) )
   
end if
end if
End do
End do

END SUBROUTINE establish



!**************************************************************************************************
! Change priority grass
!**************************************************************************************************
SUBROUTINE grass_priority &
     (type_grass, C3g_no, C4g_no, &
     phenology, pheno_clo, pheno_cls, pool_litter, npp_annual, &
     gmass_leaf, gmass_root, gmass_available, gmass_stock)

USE data_structure
implicit none

!*************************   Define augments   *************************
integer,intent(INOUT)::type_grass !grass type priority (3:C3, 4:C4)
integer,intent(INOUT)::C3g_no      !PFT number of C3 grass
integer,intent(INOUT)::C4g_no      !PFT number of C4 grass

logical,dimension(PFT_no),intent(INOUT)::phenology !Phenological status (T:foliation, F:dormance)
integer,dimension(PFT_no),intent(INOUT)::pheno_clo !counter number of last leaf onset
integer,dimension(PFT_no),intent(INOUT)::pheno_cls !counter number of last leaf shedding

real                  ,intent(IN)::pool_litter    !litter pool in a stand (g dm/stand)
real,dimension(PFT_no),intent(IN)::npp_annual     !NPP for each pft (g dm/stand/year)

real,dimension(2) ,intent(INOUT)::gmass_leaf      !c3/c4 grass foliage   biomass (g/m2)
real,dimension(2) ,intent(INOUT)::gmass_root      !c3/c4 grass root      biomass (g/m2)
real,dimension(2) ,intent(INOUT)::gmass_available !c3/c4 grass available biomass (g/m2)
real,dimension(2) ,intent(INOUT)::gmass_stock     !c3/c4 grass stock     biomass (g/m2)

!*************************   Main procedure   *************************
IF     (type_grass==4 .and. npp_annual(C3g_no)*9.0>npp_annual(C4g_no) ) then
   !C4 -> C3 change
   if (Logging==1) then
      write (*,*) 'priority grass changed: C4 -> C3', npp_annual(C3g_no)*9.0, npp_annual(C4g_no)
      write (*,*)
   endif
   
   type_grass  = 3
   call exchange_r (gmass_leaf(1)     , gmass_leaf(2)      )
   call exchange_r (gmass_root(1)     , gmass_root(2)      )
   call exchange_r (gmass_available(1), gmass_available(2) )
   call exchange_r (gmass_stock(1)    , gmass_stock(2)     )
   call exchange_l (phenology(C3g_no) , phenology(C4g_no)  )
   call exchange_i (pheno_clo(C3g_no) , pheno_clo(C4g_no)  )
   call exchange_i (pheno_cls(C3g_no) , pheno_cls(C4g_no)  )
   
ELSEIF (type_grass==3 .and. npp_annual(C3g_no)<npp_annual(C4g_no)*9.0 ) then
   !C3 -> C4 change
   if (Logging==1) then
      write (*,*) 'priority grass changed: C3 -> C4', npp_annual(C3g_no), npp_annual(C4g_no)*9.0
      write (*,*)
   endif
   
   type_grass  = 4
   call exchange_r (gmass_leaf(1)     , gmass_leaf(2)      )
   call exchange_r (gmass_root(1)     , gmass_root(2)      )
   call exchange_r (gmass_available(1), gmass_available(2) )
   call exchange_r (gmass_stock(1)    , gmass_stock(2)     )
   call exchange_l (phenology(C3g_no) , phenology(C4g_no)  )
   call exchange_i (pheno_clo(C3g_no) , pheno_clo(C4g_no)  )
   call exchange_i (pheno_cls(C3g_no) , pheno_cls(C4g_no)  )
   
END IF

END SUBROUTINE grass_priority



!**************************************************************************************************
! Calculate moisture factor for fire regime routine
!**************************************************************************************************
Subroutine moist_factor_calc &
         (moist_factor, pft, &
         mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, &
         gmass_leaf, gmass_root, gmass_available, gmass_stock, type_grass, C3g_no, C4g_no)

USE data_structure
implicit none

!*************************   Define augments   *************************
real,intent(OUT)::moist_factor   !adjusted moisture factor for fire spread

integer,dimension(Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
real   ,dimension(Max_no),intent(IN)::mass_leaf      !biomass of foliage (g)
real   ,dimension(Max_no),intent(IN)::mass_trunk     !biomass of stem    (g)
real   ,dimension(Max_no),intent(IN)::mass_root      !biomass of root    (g)
real   ,dimension(Max_no),intent(IN)::mass_stock     !biomass for stock  (g)
real   ,dimension(Max_no),intent(IN)::mass_available !biomass available  (g)
logical,dimension(Max_no),intent(IN)::tree_exist     !tree exist or not (.TRUE. or .FALSE.) 

real,dimension(2),intent(IN)::gmass_leaf      !grass foliage    biomass (g/m2)
real,dimension(2),intent(IN)::gmass_root      !grass root       biomass (g/m2)
real,dimension(2),intent(IN)::gmass_available !grass available  biomass (g/m2)
real,dimension(2),intent(IN)::gmass_stock     !herbaceous stock biomass (g/m2)
integer          ,intent(IN)::type_grass      !grass type (3:C3, 4:C4)
integer          ,intent(IN)::C3g_no          !PFT number of C3 grass
integer          ,intent(IN)::C4g_no          !PFT number of C4 grass

!*************************   Define local variables   *************************
real,dimension(PFT_no)::mass_sum     !total aboveground biomass for each PFT (g/stand)
real                    mass_overall !total aboveground biomass in this stand (g/stand)
integer                 no, i        !loop counter
real                    a1, a2

!*************************   Main procedures   *************************
!Reset variables
mass_sum(:)  = 0.0
mass_overall = 0.0
moist_factor = 0.0

!mass_total(PFT_no): total above ground biomass for each PFT
   !woody PFTs
   Do no = 1, Max_no
   if (tree_exist(no)) then
      mass_sum(pft(no)) = mass_sum(pft(no)) + mass_leaf(no) + mass_trunk(no) + mass_available(no)
   end if
   End do
    
   !herbaceous PFTs
    a1 = gmass_leaf(1) + gmass_available(1) !above ground biomass of C3 glass for unit area
    a2 = gmass_leaf(2) + gmass_available(2) !above ground biomass of C4 glass for unit area
    
   if (type_grass==3) then
      mass_sum(C3g_no) = a1 * real(Max_loc)*real(Max_loc) * 0.9
      mass_sum(C4g_no) = a2 * real(Max_loc)*real(Max_loc) * 0.1
   else
      mass_sum(C3g_no) = a1 * real(Max_loc)*real(Max_loc) * 0.1
      mass_sum(C4g_no) = a2 * real(Max_loc)*real(Max_loc) * 0.9
   endif

!mass_overall: total biomass in this grid
   mass_overall = sum(mass_sum) !grand total biomass in this forest stand
   mass_overall = max(0.01, mass_overall)

!moistfactor: adjusted litter moisture weighting factor
   Do i = 1, PFT_no
      moist_factor = moist_factor + Moisture_extinction(i) * (mass_sum(i) / mass_overall)
   End do
   
!adjust to the upper limit and lower limit of the value
   moist_factor = min(1.0, max(0.01, moist_factor))

End subroutine moist_factor_calc



!**************************************************************************************************
! Fire regime (for each year calculation)
!**************************************************************************************************
Subroutine fire_regime &
   ( fire_factor, fire_clo, c_emission, phenology, &
   pool_litter, pool_trunk, pool_leaf, pool_root, pool_grass_up, &
   pft, mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, tree_exist, &
   gmass_leaf, gmass_root, gmass_available, gmass_stock, &
   age, height, bole, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, la, &
   lai_grass, type_grass, counter )

USE data_structure
implicit none

!_____________ Define local parameter
!minimum stock mass of grass layer (g DM/m2)
real,parameter::Min_gmass_stock= 50.0

!_____________ Define augments
real   ,intent(IN)   ::fire_factor  !factor for fire probability calculation
integer,intent(INOUT)::fire_clo     !counter number of last fire
real   ,intent(INOUT)::c_emission   !monthly carbon emission (C g / month stand)

logical,dimension(PFT_no),intent(INOUT)::phenology !Phenological status (T:foliation, F:dormance)

real,intent(INOUT)::pool_litter   !litter pool in a stand       (g DM / stand)
real,intent(IN)   ::pool_trunk    !biomass of woody trunk       (g DM / stand)
real,intent(IN)   ::pool_leaf     !biomass of woody leaf        (g DM / stand)
real,intent(IN)   ::pool_root     !biomass of woody root        (g DM / stand)
real,intent(IN)   ::pool_grass_up !biomass of grass aboveground (g DM / stand)

integer,dimension(Max_no),intent(INOUT)::pft            !Plant Functional Type (1-PFT_no)
real   ,dimension(Max_no),intent(INOUT)::mass_leaf      !biomass of foliage (g)
real   ,dimension(Max_no),intent(INOUT)::mass_trunk     !biomass of stem    (g)
real   ,dimension(Max_no),intent(INOUT)::mass_root      !biomass of root    (g)
real   ,dimension(Max_no),intent(INOUT)::mass_stock     !biomass for stock  (g)
real   ,dimension(Max_no),intent(INOUT)::mass_available !biomass available  (g)
logical,dimension(Max_no),intent(INOUT)::tree_exist     !tree exist or not (.TRUE. or .FALSE.) 

integer,dimension(Max_no),intent(INOUT)::age            !Tree age (year)
integer,dimension(Max_no),intent(INOUT)::height         !height from 1.3m above ground (STEP)
integer,dimension(Max_no),intent(INOUT)::bole           !Bole length (STEP)
real   ,dimension(Max_no),intent(INOUT)::dbh_heartwood  !Hertwood diameter at 1.3m   (m)
real   ,dimension(Max_no),intent(INOUT)::dbh_sapwood    !Sapwood diameter at  1.3m   (m)
real   ,dimension(Max_no),intent(INOUT)::crown_diameter !Crown diameter   (m)
real   ,dimension(Max_no),intent(INOUT)::crown_area     !Crown area      (m2)
real   ,dimension(Max_no),intent(INOUT)::la             !Leaf area      (m2)

integer          ,intent(INOUT)::type_grass      !priority grass type (3:C3, 4:C4)
real             ,intent(INOUT)::lai_grass       !leaf area index of grass (m2/m2)
real,dimension(2),intent(INOUT)::gmass_leaf      !grass foliage biomass (g/m2)
real,dimension(2),intent(INOUT)::gmass_root      !grass root biomass (g/m2)
real,dimension(2),intent(INOUT)::gmass_available !grass available biomass (g/m2)
real,dimension(2),intent(INOUT)::gmass_stock     !grass stock biomass (g/m2)

integer,intent(IN):: counter !simulation day (1-Simulation_year*365)

!_____________ Define local variables
integer no              !loop counter for individual trees
integer i               !loop counter
real    fire_prob       !probability of fire occurance in a year
real    x               !for general usage

!Fire_prob: calculate fire probability [A(s) in eq.37]
   x         = fire_factor - 1.0
   fire_prob = fire_factor * EXP( x / (0.45*x*x*x + 2.83*x*x + 2.96*x + 1.04) )
   
!Examine this site satisfies minimum-fuel-load
   !x: total fuel load in this forest stand (g C / m2)
   x = pool_litter + pool_trunk + pool_leaf + pool_root + pool_grass_up
   x = x * C_in_drymass / (Max_loc * Max_loc)
   if ( Fuel_min > x  ) fire_prob = 0.0

!TMP (for succession experiment1)
!if ( counter => 1001 )          then !after 1000yr
!    if ( mod(counter, 200)==1 ) then !for each 200yr interval
!       fire_prob = 1.0
!    else
!       fire_prob = 0.0
!    endif
!else
!    fire_prob = 0.0
!endif

!TMP (for succession experiment2)
!if ( mod(counter,1000)==1 ) then
!   fire_prob = 1.0
!end if

!_____________ When fire occurs
   If ( (fire_prob>randf(I_seed)) .and. (counter-fire_clo>365) ) then
      if (Logging==1) write(*,*) 'Fire occured!, ', counter
      fire_clo = counter
      
      !Litter pool -> 50% lost
      c_emission   = c_emission + 0.50 * pool_litter * C_in_drymass
      pool_litter  = 0.50 * pool_litter
      
      !Grass PFTs -> total above ground biomass lost
      c_emission = c_emission + C_in_drymass * pool_grass_up
      gmass_leaf (1) = 0.0
      gmass_leaf (2) = 0.0
      
      if (type_grass==3) then
         x = 0.9 * Max_loc * Max_loc * max( 0.0 , Min_gmass_stock - gmass_stock(1) ) + &
             0.1 * Max_loc * Max_loc * max( 0.0 , Min_gmass_stock - gmass_stock(2) )    
      else
         x = 0.1 * Max_loc * Max_loc * max( 0.0 , Min_gmass_stock - gmass_stock(1) ) + &
             0.9 * Max_loc * Max_loc * max( 0.0 , Min_gmass_stock - gmass_stock(2) )    
      end if
      pool_litter = max(0.0, pool_litter - x)
      
      gmass_stock(1) = max( gmass_stock(1), Min_gmass_stock )
      gmass_stock(2) = max( gmass_stock(2), Min_gmass_stock )
      
      !Woody PFT: A fraction of trees were killed, and their above ground biomass lost
      Do no = 1, Max_no
      if (tree_exist(no)) then
      if ( randf(I_seed) >  M3(pft(no)) ) then
         !for dead plants
         c_emission   = c_emission                           + &
                        C_in_drymass * mass_leaf(no)         + &
                        C_in_drymass * mass_trunk(no) * 0.50    
         
         pool_litter = pool_litter                           + &
                       mass_trunk(no) * 0.50                 + &
                       mass_root(no)                         + &
                       mass_stock(no)                        + &
                       mass_available(no)                       
         
         tree_exist(no)      = .false.
         pft(no)            = 0
         age(no)            = 0
         height(no)         = 0
         bole(no)           = 0
         dbh_sapwood(no)    = 0.0
         dbh_heartwood(no)  = 0.0
         crown_area(no)     = 0.0
         crown_diameter(no) = 0.0
         la(no)             = 0.0
         mass_leaf(no)      = 0.0
         mass_root(no)      = 0.0
         mass_trunk(no)     = 0.0
         mass_stock(no)     = 0.0
         mass_available(no) = 0.0
      else
         !for survived trees
         c_emission    = c_emission + C_in_drymass * mass_leaf(no)
         mass_leaf(no) = 0.0
         la(no)        = 0.0
      end if
      end if
      End do
      
        !Deciduous PFTs -> change to dormance phase
      Do i=1, PFT_no
      if ( Phenology_type(i) .ne. 0.0 ) then 
         phenology(i) = .false.
      End if
      End do
      
   Endif
   
End subroutine fire_regime



!**************************************************************************************************
! mortality of individual 'no' for the current year
!**************************************************************************************************
SUBROUTINE mortality &
      (tree_exist, pft, age, height, bole, dbh_heartwood, dbh_sapwood, &
      crown_diameter, crown_area, la, &
      mass_leaf, mass_trunk, mass_root, mass_stock, mass_available, &
      bole_x, bole_y, height_limit, radius_limit, &
      stat_leaf, stat_leaf_RunningRecord, stat_leaf_crowntop, stat_leaf_crownbottom, &
      mort_regu1, mort_regu2, mort_regu3, mort_regu4, &
      pool_litter, tmp_coldest_20yr_ave, tmp_hottest_20yr_ave)

USE data_structure
implicit none

!_____________ Define local parameters
   !bottom height of forest layer no 2,3,4,5 (m)
   real,dimension(1:4),parameter::Layer_top = (/1.3, 15.0, 25.0, 36.0/)
   
   !Threshold fraction of crown layer overlapping to declare 'crowded layer' (frac)
   real,parameter::Frac_crowded      = 1.00
   
   !Threshold DBH to have possibility to form gap (m)
   real,parameter::Thres_dbh_GapForm = 0.10
   
   !Fraction of gap formation when large tree die
   real,parameter::Frac_GapForm      = 0.40
   
!_____________ Define arguments
logical,dimension(Max_no),intent(INOUT)::tree_exist     !tree exist or not
integer,dimension(Max_no),intent(INOUT)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(Max_no),intent(INOUT)::age            !Tree age (year)
integer,dimension(Max_no),intent(INOUT)::height         !Tree height from 1.3m above ground(STEP)
integer,dimension(Max_no),intent(INOUT)::bole           !Bole length (STEP)
real   ,dimension(Max_no),intent(INOUT)::dbh_heartwood  !Hertwood diameter at 1.3m (m)
real   ,dimension(Max_no),intent(INOUT)::dbh_sapwood    !Sapwood diameter at  1.3m (m)
real   ,dimension(Max_no),intent(INOUT)::crown_diameter !Crown diameter     (m)
real   ,dimension(Max_no),intent(INOUT)::crown_area     !Crown area        (m2)
real   ,dimension(Max_no),intent(INOUT)::la             !Leaf area         (m2)
real   ,dimension(Max_no),intent(INOUT)::mass_leaf      !biomass of foliage (g)
real   ,dimension(Max_no),intent(INOUT)::mass_trunk     !biomass of stem    (g)
real   ,dimension(Max_no),intent(INOUT)::mass_root      !biomass of root    (g)
real   ,dimension(Max_no),intent(INOUT)::mass_stock     !biomass for stock  (g)
real   ,dimension(Max_no),intent(INOUT)::mass_available !biomass available  (g)

real   ,dimension(Max_no),intent(IN)   ::bole_x         !x_location of bole  (m)
real   ,dimension(Max_no),intent(IN)   ::bole_y         !y_location of bole  (m)

integer,dimension(Max_no),intent(INOUT)::height_limit !max tree height proximate trees permit
real   ,dimension(Max_no),intent(INOUT)::radius_limit !max canopy radius proximate trees permit

real,dimension(Max_no    ),intent(INOUT)::stat_leaf           !(income-expenditure)/leaf_mass
real,dimension(Max_no,  7),intent(INOUT)::stat_leaf_RunningRecord !running record of stat_leaf
real,dimension(Max_no    ),intent(INOUT)::stat_leaf_crowntop    !stat_leaf @ the top crown disk
real,dimension(Max_no,10 ),intent(INOUT)::stat_leaf_crownbottom !stat_leaf @ bottom crown disks

real,dimension(Max_no),intent(INOUT)::mort_regu1 !NPP annual (g dm / individual)
real,dimension(Max_no),intent(INOUT)::mort_regu2 !mean leaf area of last year (m2/day)
real,dimension(Max_no),intent(INOUT)::mort_regu4 !stem diameter increament in last year (m/year)

real,intent(INOUT)::pool_litter          !litter pool in a stand (g / stand )

real,intent(IN)   ::tmp_coldest_20yr_ave !20yr running mean of tmp_coldest_month (Celcius)
real,intent(IN)   ::tmp_hottest_20yr_ave !20yr running mean of tmp_hottest_month (Celcius)
real,intent(IN)   ::mort_regu3           ! heat stress for each year

!_____________ Define local variable
   logical,dimension(Max_no)::death_list     !frag for death assigned trees
   logical,dimension(Max_no)::death_list_gap !frag for death assigned trees
   
   real mort_greff   !mortality as a result of low growth efficiency
   real mort_heart   !mortality as a result of heart stress
   real mort_lim     !mortality as a result of bioclimic limits
   real mort_etc     !mortality as a result of other factors
   real mort_total   !mortality as a result of above all factors
   
   real   ,dimension(5)     ::layer_ca_sum  !sum of crown area for each layer (m2)
   logical,dimension(5)     ::layer_crowded !crowded flag for each layer
   integer,dimension(Max_no)::layer_belong  !crown layer that each tree belongs
   
   integer              ::size_class   !for mortality checker
   integer,dimension(10)::tree_number  !for mortality checker
   integer,dimension(10)::death_number !for mortality checker
   
   integer no, p, i                                 !loop counter
   real    x, y                                     !for general usage
   real    d1, d2, d3, d4, d5, d6, d7, d8, d9, dist !

!_____________ Determine crowded layer
   layer_ca_sum (:) = 0.0     !(1:Top_layer) sum of crown area for each layer (m2)
   layer_crowded(:) = .false. !(1:Top_layer) crowded flag for each layer          
   layer_belong (:) = 0       !(1:Max_no   ) crown layer that each tree belongs   
   
   Do no=1, Max_no 
   If ( tree_exist(no) ) then
      
      x = height(no)*STEP+1.3
      
      if     ( x<Layer_top(1) ) then
         layer_belong(no)=1 ; layer_ca_sum(1)=layer_ca_sum(1)+crown_area(no)
         
      elseif ( x<Layer_top(2) ) then
         layer_belong(no)=2 ; layer_ca_sum(2)=layer_ca_sum(2)+crown_area(no)
         
      elseif ( x<Layer_top(3) ) then
         layer_belong(no)=3 ; layer_ca_sum(3)=layer_ca_sum(3)+crown_area(no)
         
      elseif ( x<Layer_top(4) ) then
         layer_belong(no)=4 ; layer_ca_sum(4)=layer_ca_sum(4)+crown_area(no)
         
      else                          
         layer_belong(no)=5 ; layer_ca_sum(5)=layer_ca_sum(5)+crown_area(no)
         
      endif
      
   End if
   End do
   
   Do i=1, 5
      if ( layer_ca_sum(i) > Frac_crowded*Max_loc*Max_loc ) layer_crowded(i)=.true.
   End do
   
!_____________ Determine trees to be die 1
death_list(:) = .false.

DO no=1, Max_no 
if ( tree_exist(no) .and. age(no)>1 ) then
   
   !reset mortality variables for each PFT
   mort_greff=0.0 ; mort_heart=0.0 ; mort_lim=0.0 ; mort_etc=0.0   ; mort_total=0.0
   
   !set pft number
    p = pft(no)
   
   !mortality 1: growth efficiency
   Select Case (Life_type(p))
      !tropical evergreen trees
      case (1) 
         
         if (mort_regu4(no)<0.007) then
            mort_greff = M4(p) * 0.0178 * exp(-242.57*mort_regu4(no))
         else
            mort_greff = M4(p) * 0.0032
         endif
         
         if ( layer_crowded(layer_belong(no)) ) mort_greff = M5(p)
         
      !other woody PFTs
      case default
         x      = (mort_regu1(no)/1000.0) / max(0.01, mort_regu2(no))
         x      = max(0.0, x)
         mort_greff = M1(p) / (M2(p)**x)
         
         !other method
         !x      = mort_regu1(no) / max(0.1, mort_regu2(no)) / 1000.0
         !x      = max(0.001, x)
         !mort_greff = 1.0 / ( 1.0 + M1(p) * ( x ** M2(p) ) )
         
   End select
   
   !mortality 2: heart stress
   if (Heat_sensitive(p)==1) then
      mort_heart = mort_regu3
   else
      mort_heart = 0.0
   end if
   
   !mortality 3: bioclimic limits (based on 20yrs mean of coldest month temperature)
   if     ( tmp_coldest_20yr_ave < TC_min(p) ) then
       mort_lim = 1.0
   elseif ( Life_type(no)==2 .and. (tmp_hottest_20yr_ave-tmp_coldest_20yr_ave)<43.0 ) then
       mort_lim = 1.0
   else
       mort_lim = 0.0
   endif
   
   !mortality 4: mortality by other factors
   if     ( height(no)-bole(no)<=2 .and. age(no)>3 ) then
      mort_etc = 1.0
   elseif ( mort_regu1(no)<10.0 .and. age(no)>3 )    then
      mort_etc = 1.0
   elseif ( dbh_heartwood(no)+dbh_sapwood(no) > 1.00 )   then
      mort_etc = 1.0
   elseif ( age(no) >= AGE_max(p) )                  then
      mort_etc = 1.0
   else
      mort_etc = 0.0
   endif
    
   !Sum up all mortality components, and calculate probabilty of death
   mort_total   = min(1.0, mort_greff + mort_heart + mort_lim + mort_etc)
   
   !Determine whether die
   if (mort_total > randf(I_seed)) death_list(no)=.true.
   
end if
END DO

death_list_gap(:) = .false.
!_____________ Determine trees to be die 2 (Gap formation)
DO no=1, Max_no 
   
   !When gap forms
   if (  death_list(no)                                         .and. &
        (Life_type(pft(no)) == 1)                               .and. &
        (dbh_heartwood(no)+dbh_sapwood(no) > Thres_dbh_GapForm) .and. &
        (Frac_GapForm > randf(I_seed))                                  ) then
      
      !set central coodinate of new gap
      x = randf(I_seed) * Max_loc ; x = max(0.0, x) ; x = min(Max_loc, x)
      y = randf(I_seed) * Max_loc ; y = max(0.0, y) ; y = min(Max_loc, y)
      
      do i=1, Max_no
      if ( tree_exist(i) .and. age(i)>1 ) then
         
         !Distance from gap center
         d1 = (x - (bole_x(i)-real(Max_loc)) )**2 + (y - (bole_y(i)-real(Max_loc)) )**2
         d2 = (x - (bole_x(i)              ) )**2 + (y - (bole_y(i)-real(Max_loc)) )**2
         d3 = (x - (bole_x(i)+real(Max_loc)) )**2 + (y - (bole_y(i)-real(Max_loc)) )**2
         d4 = (x - (bole_x(i)-real(Max_loc)) )**2 + (y - (bole_y(i)              ) )**2
         d5 = (x - (bole_x(i)              ) )**2 + (y - (bole_y(i)              ) )**2
         d6 = (x - (bole_x(i)+real(Max_loc)) )**2 + (y - (bole_y(i)              ) )**2
         d7 = (x - (bole_x(i)-real(Max_loc)) )**2 + (y - (bole_y(i)+real(Max_loc)) )**2
         d8 = (x - (bole_x(i)              ) )**2 + (y - (bole_y(i)+real(Max_loc)) )**2
         d9 = (x - (bole_x(i)+real(Max_loc)) )**2 + (y - (bole_y(i)+real(Max_loc)) )**2
         
         dist = min(d1,d2,d3,d4,d5,d6,d7,d8,d9)
         dist = sqrt(dist)
         
         if ( dist < crown_diameter(no)/2 ) death_list_gap(i)=.true.
         
      endif
      enddo
      
      !exit !Gap can occur only one time in a year
      
   endif
   
END DO

!_____________ * Mortality Checker
!   size_class      = 0
!   tree_number (:) = 0
!   death_number(:) = 0
!   
!   do no=1, Max_no 
!   if ( tree_exist(no) ) then !when tree exist
!      size_class               = int( (dbh_heartwood(no)+dbh_sapwood(no)) / 0.05 ) + 1
!      size_class               = max(1, min(10, size_class))
!      tree_number (size_class) = tree_number(size_class)  + 1
!      
!      if (death_list(no)) death_number(size_class) = death_number(size_class) + 1
!      
!   end if
!   end do
!   
!   write(*,*) '---------- prop death ---------'
!   write(*,*) real(death_number(1)) / real(max(1, tree_number(1))), &
!              real(death_number(2)) / real(max(1, tree_number(2))), &
!              real(death_number(3)) / real(max(1, tree_number(3))), &
!              real(death_number(4)) / real(max(1, tree_number(4))), &
!              real(death_number(5)) / real(max(1, tree_number(5))), &
!              real(death_number(6)) / real(max(1, tree_number(6)))
!   write(*,*) 
   
!_____________ Kill assigned trees
   do no=1, Max_no 
   if (death_list(no) .or. death_list_gap(no)) then
      Call die_tree &
      ( height(no), bole(no), dbh_heartwood(no), dbh_sapwood(no), &
      crown_diameter(no), crown_area(no), la(no), mort_regu2(no), &
      mass_leaf(no), mass_trunk(no), mass_root(no), mass_stock(no), mass_available(no), &
      age(no), pft(no), &
      height_limit(no), radius_limit(no), mort_regu1(no), &
      stat_leaf(no), stat_leaf_RunningRecord(no,:), &
      stat_leaf_crowntop(no), stat_leaf_crownbottom(no,:),&
      tree_exist(no), pool_litter )
   endif
   end do
   
!_____________ Increament of tree age
   do no=1, Max_no 
   if ( tree_exist(no) ) then !when tree exist
       age(no) = age(no) + 1  !add tree age
   end if
   end do
   
END SUBROUTINE mortality
